/***********************************************************************
Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/
#ifndef NLM_CRYPTO_DATA_H
#define NLM_CRYPTO_DATA_H

#define NLM_CRYPTO_VC_BASE 281
/*
 * is the following table needed for all modes?
Cipher            keylen           iv_len
*/

//Ciphers ====> NLM_CIPHER_AES128	
static int cipher_mode_key_len[NLM_CIPHER_MAX][NLM_CIPHER_MODE_MAX] = {
/*		      ECB    CBC   CFB   OFB   CTR   AESF8    GCM   CCM    8   9   LRW   XTS */
/* BYPASS */       {   0,     0,    0,    0,    0,     0,      0,    0,    0,  0,   0,    0,},
/* DES */          {   8,     8,    0,    0,    0,     0,      0,    0,    0,  0,   0,    0,},
/* 3DES */         {   24,    24,   0,    0,    0,     0,      0,    0,    0,  0,   0,    0,},
/* AES128 */       {   16,    16,   16,   16,   16,    16,     16,   16,   0,  0,   32,   32,},
/* AES192 */       {   24,    24,   24,   24,   24,    24,     24,   24,   0,  0,   40,   48,},
/* AES256 */	   {   32,    32,   32,   32,   32,    32,     32,   32,   0,  0,   48,   64,},
/* There is no mode associated with ARC4 ... probably ignored... */
/* ARC4 */         {   32,    32,   32,   32,   32,    32,     32,   32,   0,  0, 32,   32,}, 
/* SBOX for ARC4 comes in the next cacheline... So, if SBOX is used, few bytes
 * of random padding is done to the key to make the SBOX cacheline aligned*/
/* There is no mode associated with Kasumi F8... probably ignored... */
/* KASUMI F8 */    {   16,    16,   16,   16,   16,    16,     16,   16,   0,  0,  16,   16,},
/* There is no mode associated with Snow3G F8... probably ignored... */
/* SNOW3G F8 */    {   16,    16,   16,   16,   16,    16,     16,   16,   0,  0,  16,   16,},
/* CAMELLIA128*/   {   16,    16,   16,   16,   16,    16,     16,   16,   0,  0,  32,   32,},
/* CAMELLIA192*/   {   24,    24,   24,   24,   24,    24,     24,   24,   0,  0,  40,   48,},
/* CAMELLIA256*/   {   32,    32,   32,   32,   32,    32,     32,   32,   0,  0,  48,   64,},
};

//-1 indicates variable length IV
// In case of AES/Camelia cipher and CBC-MAC auth, IV is not needed.
// In case of AES/Camelia cipher and XCBC-MAC auth, IV is needed only for 
//CBC, CFB, OFB and CTR modes..
static int cipher_mode_iv_len[NLM_CIPHER_MAX][NLM_CIPHER_MODE_MAX] = {
/*		       ECB  CBC   CFB   OFB   CTR  AESF8    GCM  CCM    8   9  LRW   XTS */
/* BYPASS */       {   0,    0,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* DES */          {   0,    8,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* 3DES */         {   0,    8,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* AES128 */       {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,},
/* AES192 */       {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,},
/* AES256 */       {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,},
/* ARC4 */         {   0,    0,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* KASUMI F8 */    {   0,    0,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* SNOW3G F8 */    {   0,    0,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* CAMELLIA128 */  {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,},
/* CAMELLIA192 */  {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,}, 
/* CAMELLIA256 */  {   0,    16,   16,   16,   16,   16      -1,   0,   0,  0,  16,   16,},
};


static int cipher_alg_needs_mode[NLM_CIPHER_MAX] = {
0, //NLM_CIPHER_BYPASS
1, //NLM_CIPHER_DES
1, //NLM_CIPHER_3DES
1, //NLM_CIPHER_AES128
1, //NLM_CIPHER_AES192
1, //NLM_CIPHER_AES256
0, //NLM_CIPHER_ARC4
0, //NLM_CIPHER_KASUMI_F8
0, //NLM_CIPHER_SNOW3G_F8 
1, //NLM_CIPHER_CAMELLIA128
1, //NLM_CIPHER_CAMELLIA192
1, //NLM_CIPHER_CAMELLIA256
};

static int cipher_mode_valid[NLM_CIPHER_MAX][NLM_CIPHER_MODE_MAX] = {
	/* Mode numbers 8 and 9 are undefined.. LRW comes after that */
/*		       ECB  CBC  CFB  OFB  CTR  AESF8  GCM  CCM  8   9  LRW  XTS */
/* BYPASS */       {   0,    0,   0,   0,   0,    0,    0,   0,  0,  0,  0,   0,},
/* DES */          {   1,    1,   0,   0,   0,    0,    0,   0,  0,  0,  0,   0,},
/* 3DES */         {   1,    1,   0,   0,   0,    0,    0,   0,  0,  0,  0,   0,},
/* AES128 */       {   1,    1,   1,   1,   1,    1,    1,   1,  0,  0,  1,   1,},
/* AES192 */       {   1,    1,   1,   1,   1,    1,    1,   1,  0,  0,  1,   1,},
/* AES256 */       {   1,    1,   1,   1,   1,    1,    1,   1,  0,  0,  1,   1,},
/* ARC4 */         {   0,    0,   0,   0,   0,    0,    0,   0,  0,  0,  0,   0,},
/* KASUMI F8 */    {   0,    0,   0,   0,   0,    0,    0,   0,  0,  0,  0,   0,},
/* SNOW3G F8 */    {   0,    0,   0,   0,   0,    0,    0,   0,  0,  0,  0,   0,},
/* CAMELLIA128*/   {   1,    1,   0,   0,   1,    0,    0,   1,  0,  0,  0,   0,},
/* CAMELLIA192*/   {   1,    1,   0,   0,   1,    0,    0,   1,  0,  0,  0,   0,},
/* CAMELLIA256*/   {   1,    1,   0,   0,   1,    0,    0,   1,  0,  0,  0,   0,},
};

static int auth_alg_needs_mode[NLM_AUTH_MAX] = {
0,	// NLM_AUTH_BYPASS
0,	// NLM_AUTH_MD5
1,	// NLM_AUTH_SHA
0,	// NLM_AUTH_UNDEFINED
1,	// NLM_AUTH_AES128
1,	// NLM_AUTH_AES192
1,	// NLM_AUTH_AES256
0,	// NLM_AUTH_KASUMI_F9
0,	// NLM_AUTH_SNOW3G_F9
1,	// NLM_AUTH_CAMELLIA128
1,	// NLM_AUTH_CAMELLIA192
1,	// NLM_AUTH_CAMELLIA256
};

static int auth_mode_key_len[NLM_AUTH_MAX][NLM_AUTH_MODE_MAX] = {
/*	               SHA1 SHA224 SHA256 SHA384 SHA512  CMAC  XCBC CBC_MAC CCM  GCM*/
/* BYPASS */		{0,    0,     0,     0,     0,     0,   0,    0,     0,    0, },
/* MD5 */		{64,   64,    64,   64,    64,    64,  64,   64,    64,   64, },
/* SHA */		{64,   64,    64,   128,   128,    0,   0,    0,     0,    0, },
/* 3 */			{0,    0,     0,     0,     0,     0,   0,    0,     0,    0, },
/* AES128 */		{0,    0,     0,     0,     0,    16,  16,   16,    16,   16, },
/* AES192 */		{0,    0,     0,     0,     0,    24,  24,   24,    24,   24, },
/* AES256 */		{0,    0,     0,     0,     0,    32,  32,   32,    32,   32, },
/* KASUMI_F9 */		{16,  16,    16,    16,    16,    16,  16,   16,    16,   16, },
/* SNOW3G_F9 */		{16,  16,    16,    16,    16,    16,  16,   16,    16,   16, }, //sandip -> verify
/* CAMELLIA128 */	{0,    0,     0,     0,     0,    16,  16,   16,    16,   16, },
/* CAMELLIA192 */	{0,    0,     0,     0,     0,    24,  24,   24,    24,   24, },
/* CAMELLIA256 */	{0,    0,     0,     0,     0,    32,  32,   32,    32,   32, },
/* GHASH */		{0,    0,     0,     0,     0,    32,  32,   32,    32,   32, }, //todo:
};

static int auth_mode_valid[NLM_AUTH_MAX][NLM_AUTH_MODE_MAX] = {
/*	               SHA1 SHA224 SHA256 SHA384 SHA512  CMAC  XCBC CBC_MAC CCM GCM */
/* BYPASS */		{0,   0,     0,     0,     0,     0,   0,    0,      0,  0,  },
/* MD5 */		{1,   1,     1,     1,     1,     1,   1,    1,      1,  1,  }, /* dont care */
/* SHA */		{1,   1,     1,     1,     1,     0,   0,    0,      0,  0,  },
/* 3 */			{0,   0,     0,     0,     0,     0,   0,    0,      0,  0,  },
/* AES128 */		{0,   0,     0,     0,     0,     1,   1,    1,      1,  1,  },
/* AES192 */		{0,   0,     0,     0,     0,     1,   1,    1,      1,  1,  },
/* AES256 */		{0,   0,     0,     0,     0,     1,   1,    1,      1,  1,  },
/* KASUMI_F9 */		{1,   1,     1,     1,     1,     1,   1,    1,      1,  1,  }, /* dont care */
/* SNOW3G_F9 */		{1,   1,     1,     1,     1,     1,   1,    1,      1,  1,  }, /* dont care */
/* CAMELLIA128 */	{0,   0,     0,     0,     0,     1,   1,    1,      1,  1,  },
/* CAMELLIA192 */	{0,   0,     0,     0,     0,     1,   1,    1,      1,  1,  },
/* CAMELLIA256 */	{0,   0,     0,     0,     0,     1,   1,    1,      1,  1,  },
/* GHASH */		{1,   1,     1,     1,     1,     1,   1,    1,      1,  1,  }, //todo
};

#endif
