/***********************************************************************
Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/

// --------------------------------------------------------------------------
// includes

#include <stdio.h>
#include <stdlib.h>

#include "string.h"
#include "system.h"
#include "stdlib.h"
#include "getopt.h"
#include "asmmacros.h"
#include "mips-exts.h"
#include "iomap.h"
#include "pic.h"
#include "spinlock.h"
#include "phnx_tb_defs.h"
#include "phnx_tb.h"
#include "psb.h"

// --------------------------------------------------------------------------
// defines
//

static struct sretry sretry[3] = {
	{0, "Cpu"},
	{1, "l2"},
	{2, "Bridge"},
};

static struct snoop_error_status snoop_error_status[4] = {
	{0, "Bridge"},
	{1, "l1 tag"},
	{2, "l2 ctag"},
	{3, "l2 uctag"},
};

static struct srslt srslt[4] = {
	{0, "Cpu Shared"},
	{1, "Cpu Modified"},
	{2, "L2 Shared"},
	{3, "l2 Modified"},
};

static struct nodeinfo nodeinfo[32] = {
	{0, "Cpu0"},
	{1, "Cpu1"},
	{2, "Cpu2"},
	{3, "Cpu3"},
	{4, "Cpu4"},
	{5, "Cpu5"},
	{6, "Cpu6"},
	{7, "Cpu7"},
	{8, "L2-bank0"},
	{9, "L2-bank1"},
	{10, "L2-bank2"},
	{11, "L2-bank3"},
	{12, "L2-bank4"},
	{13, "L2-bank5"},
	{14, "L2-bank6"},
	{15, "L2-bank7"},
	{16, "16:Unknown"},
	{17, "XGMac0"},
	{18, "XGMac1"},
	{19, "GMac0"}, // gmac0-3
	{20, "SAE"},
	{21, "GPIO"},
	{22, "PCIX"},
	{23, "HT"},
	{24, "DMA"},
	{25, "CDE"},
	{26, "PCIE"}, 
	{27, "USB"},
	{28, "GMac4"}, //gmac4-7
	{29, "29:Unknown"},
	{30, "30:Unknown"},
	{31, "31:Unknown"},
};

static struct cmdinfo cmdinfo[8] =  {
	{0, "Idle"},
	{1, "Write"},
	{2, "Read"},
	{3, "ReadEx"},
	{4, "Upgrade"},
	{5, "Invalidate"},
	{6, "Undefined"},
	{7, "Write error"},
};

#define Message(a,b...) //printf("\n"a"\n",##b)
#define ErrorMsg(a,b...) printf("\n"a"\n",##b)

struct tb_data{
	volatile char data[TB_SIZE];
	volatile int size;
	spinlock_t tb_lock;
};

static struct tb_data tb_data;

// --------------------------------------------------------------------------
// routines
//

int print_tb_entries(struct tb_entry samples[], int num_samples);

static inline void read_tb_entry(unsigned char *tb_entry_ptr)
{
	int j;
	unsigned int *x;

	tb_pop_entry();
	for (j = 0; j < 4; ++j) {
		x = (unsigned int *) (tb_entry_ptr + 4*j);
		*x = tb_read_rddata_reg (j);
	}
}

irqreturn_t tb_int_handler(int irq, struct pt_regs *pt_regs) 
{
	unsigned int  status_reg, ctrl_reg;
	unsigned int  tb_wr_ptr, tb_rd_ptr, tb_first_match_ptr;
	int    m, n;
	unsigned int no_of_junk_entry;
	unsigned int req_read_pos =0;
	status_reg = tb_read_status_reg();
	Message("Interrupt Handler Called.");

	if (!TB_IS_STATUS_DONE(status_reg)){
		ErrorMsg("something is wrong as int must not have been generated");
		return IRQ_HANDLED;
	}

	tb_wr_ptr = TB_GET_WRPTR(status_reg);
	tb_rd_ptr = TB_GET_RDPTR(status_reg);
	tb_first_match_ptr = TB_GET_FIRST_MATCH_PTR(status_reg); 

	ctrl_reg = tb_read_ctrl_reg();
	n = TB_GET_CTRL_REQCNT(ctrl_reg);

	Message("tb_wr_ptr %d, tb_rd_ptr %d, tb_first_match_ptr %d, Count %d",
									tb_wr_ptr,tb_rd_ptr,tb_first_match_ptr,n);

	{
	unsigned int no_of_valid_entry;
	if (!(status_reg & TB_FULL)) {
		req_read_pos = tb_first_match_ptr;
		no_of_valid_entry = n;
	} else {
		if (tb_first_match_ptr >= tb_rd_ptr)
			no_of_valid_entry = tb_first_match_ptr - tb_rd_ptr;
		else 
			no_of_valid_entry = 256 - ( tb_rd_ptr - tb_first_match_ptr);
		req_read_pos = no_of_valid_entry;
	}
	no_of_junk_entry = req_read_pos;
	}

	while(no_of_junk_entry--)
	  tb_pop_entry();

	status_reg = tb_read_status_reg();
	tb_rd_ptr = TB_GET_RDPTR(status_reg);
	Message("After Removin Junk Entry ReadPtr %d",tb_rd_ptr);
	
	tb_data.size = 16 * TB_GET_CTRL_REQCNT(ctrl_reg);
	for (m = 0; m < tb_data.size; m += 16)
		read_tb_entry((unsigned char *)tb_data.data + m);

	status_reg = tb_read_status_reg();
	tb_rd_ptr = TB_GET_RDPTR(status_reg);
	Message("After Popin All Valid Entry ReadPtr %d",tb_rd_ptr);
	printf("Done collecting the samples in irq handler.. size is %d"
			"\n",tb_data.size);
	return IRQ_HANDLED;
}

void tb_reinit(void) 
{
	tb_data.size = 0;
	tb_write_reg_be32(TB_INIT_REG, 0x0);
	tb_write_reg_be32(TB_INIT_REG, 0x1);
}

unsigned int get_tb_reg(int reg)
{
	return tb_read_reg_be32(reg);
}

int set_tb_reg(int reg, unsigned int val)
{
	 tb_write_reg_be32(reg, val);
	 return 0;
}

int collect_samples(struct tb_entry samples[])
{
	memcpy((char *)samples,(char *)tb_data.data,tb_data.size);
	return 1;
}

int program_tracebuffer(struct match_tb *prog_tb)
{

	unsigned int val;
	int tb_error = 0;

	//Disable Trace Buffer - Ctrl Register.
	val = get_tb_reg(TB_CTRL_REG);
	val |= TB_CTRL_DISABLE;
	set_tb_reg(TB_CTRL_REG, val);
	
	// Write ReqMatch 1 Reg.
	val = prog_tb->cmd | (prog_tb->node << 4) | (prog_tb->trans_id << 12) |
		(prog_tb->snoop_error_status << 20) | (prog_tb->cacheable << 24);
	set_tb_reg(TB_REQMATCH_REG_1, val);

	// Write ReqMatch 2 Reg.
	val = prog_tb->snoop_result << 8;
	set_tb_reg(TB_REQMATCH_REG_2, val);

	val = prog_tb->addr >> 5;
	set_tb_reg(TB_RADDR_REG_1, val);

	val = (prog_tb-> addr >> 37) & 0x7;
	set_tb_reg(TB_RADDR_REG_2, val);

	val = prog_tb->control_reg | ( prog_tb->num_req << 16);
	set_tb_reg(TB_CTRL_REG, val);

	return tb_error;
}


void dump_help()
{
	printf("Run this executable by using the command userapp/userapp_os -A\""
			"<args>\"\n");
	fprintf(stderr, "Usage: userapp/userapp_os -A \"<args>\" \n"
	"    -n		Node (default: cpu0 if no match is requested)\n"
	"		Nodes are: (cpu[0-7]/l2-bank[0-7]/xgmac[0-1]/gmac0/gmac4/pcix/\n"
	"		ht/dma/cde/pcie/usb)\n"
	"    -a		Address\n"
	"    -s		Snoop error Status (range: 0-15)\n"
	"    -t		Transaction Id (range: 0-255)\n"
	"    -c		Command (read/write/upgrade/readEx/Invalidate/WriteError)\n"
	"    -o		Snoop result (range: 0-15)\n"
	"    -k		Cacheable transaction(yes/no)\n"
	"    -m		Collection Mode (all/matching default:all)\n"
	"    -r		Number of requested entries(range: 1-255 default: 100)\n"
	"    -x		Continuous collection... (default: one shot)\n"
	"    -h		Quick Help. Prints this menu\n"
	"    -e		exhaustive help\n"
		);

	printf("\n"
	" Detailed explanation follows:\n"
	" There are seven matching criteria which can be specified while\n"
	" launching the app from bootloader. Use userapp -A "". Options specified\n"
	" in quotes followed by -A are passed to the application by bootloader.\n"
	" These options are: node (-n),address(-a), snoop error status(-s)\n"
	" transaction id(-t), command(-c), snoop result(-o) and cacheable(-k).\n"
	" Also, we can specify collection mode(-m) and number of requested entries\n"
	" (-r) to be collected in the tracebuffer. If -x is specified, data is \n" 
	" collected in an infinite loop, otherwise data is collected only once.\n"
	" If no match criterion is specified, match defaults to node=cpu0.\n\n"
	" Node usage: -n <nodename>\n"
	" \t\tNode values can be one of cpu0,cpu1..cpu7, l2-bank0..l2-bank7,\n"
	" \t\txgmac0, xgmac1, gmac0, gmac4, pcix, ht, dma, cde, pcie and usb.\n"
	" \t\tNote that in case of certain chips (eg xls), certain blocks are\n"
	" \t\tnot present (eg xgmac0). gmac0 represnts gmac block 0(gmac0-3)\n"
	" \t\tand gmac1 represents gmac block 1(gmac4-7).\n"
	" \t\tUser has the responsibility to give correct values for node.\n\n"
	" Address usage: -a <physical address> \n"
	" \t\tPhysical address needs to be provided.\n\n"
	" Snoop error status usage: -s <0-15> \n"
	" \t\tFour bits in the snoop correspond to four snoop errors:\n"
	" \t\tbit 0: Brigde, 1: l1 tag, 2: l2 ctag, 3: l2 uctag\n\n"
	" Transaction id usage: -t <0-255> \n"
	" \t\tTransaction if of the transaction. Transaction id are not \n"
	" \t\tcommon across nodes. Different nodes have their own transaction\n"
	" \t\tids.\n\n"
	" Command usage: -c <cmdname>\n"
	" \t\tComnad values are read, write, upgrade, readex, invalide and \n"
	" \t\twriteerror\n\n"
	" Snoop result usage: -o <0-15>\n"
	" \t\tFour bits in the snoop correspond to four snoop results:\n"
	" \t\tbit 0:cpu shared, 1:cpu modified, 2:l2 shared, 3:l2 modified\n\n"
	" Cacheable usage: -k (yes/no)\n"
	" \t\tThis defines if the transaction is cacheable or not.\n\n"
	" Collection mode usage: -m (all/matching)\n"
	" \t\tIf set to all, all the transactions after first matched \n"
	" \t\ttransaction are collected irrespective of the match. If set \n"
	" \t\tto matching, only the transactions which match the defined \n"
	" \t\tcriterion are collected in tracebuffer.\n\n"
	" Number of requested entries usage: -r <1-255>\n"
	" \t\tNumber of transactions to be collected in buffer before cpu is\n"
	" \t\tinterrupted by tracebuffer.\n\n"
	" Contineous collection usage: -x\n"
	" \t\tWhen -x option is used, app contineously polls for samples. Default\n"
	" \t\tis one shot. In one shot mode, samples are collected only once.\n"
	" \t\tCollection rate can be controlled by using -x, and -r options.\n\n"
	" help usage: -h \n"
	" \t\tQuick help. Prints usage in short\n\n"
	" exhaustive help usage: -e\n"
	" \t\tPrints this help\n\n"
	);
	exit(0);
}

static void usage() {
	fprintf(stderr, "Usage: userapp/userapp_os -A \"<args>\" \n"
	"    -n		Node (default: cpu0 if no match is requested)\n"
	"		Nodes are: (cpu[0-7]/l2-bank[0-7]/xgmac[0-1]/gmac0/gmac4/pcix/\n"
	"		ht/dma/cde/pcie/usb)\n"
	"    -a		Address\n"
	"    -s		Snoop error Status (range: 0-15)\n"
	"    -t		Transaction Id (range: 0-255)\n"
	"    -c		Command (read/write/upgrade/readEx/Invalidate/WriteError)\n"
	"    -o		Snoop result (range: 0-15)\n"
	"    -k		Cacheable transaction(yes/no)\n"
	"    -m		Collection Mode (all/matching default:all)\n"
	"    -r		Number of requested entries(range: 1-255 default: 100)\n"
	"    -x		Continuous collection... (default: one shot)\n"
	"    -h		Quick Help. Prints this menu\n"
	"    -e		exhaustive help\n"
		);
	exit(0);
}

struct match_tb *parse_args(int argc, char *argv[])
{
	int c = 0;
	struct match_tb *match_tb;
	int i;

	match_tb = (struct match_tb *)malloc(sizeof(struct match_tb));

	match_tb->node = 0; // cpu 0
	match_tb->addr = 0; 
	match_tb->snoop_error_status = 0; 
	match_tb->trans_id = 0; 
	match_tb->cmd = 0; 
	match_tb->snoop_result = 0;  
	match_tb->cacheable = 0; 
	match_tb->mode = 0; 
	match_tb->num_req = 10; // 10 requests
	match_tb->sleep = 1; // 1 sec
	match_tb->one_shot = 1; 
	match_tb->control_reg = 0;
	
	while ((c = getopt(argc, argv, "n:a:s:t:c:o:k:m:r:xhe")) != -1) {
		switch(c) {
		case 'n': 
			for (i = 0; i < 32; i++) {
				if (!strcasecmp(optarg, nodeinfo[i].nodename))
					break;
			}
			if (i == 32) {
				printf("Incorrect -n option\n");
				printf("-n should be followed by cpu[0-7]/l2-bank[0-7]/xgmac[0-1]/gmac0/gmac4/pcix/ht/dma/cde/pcie/usb\n");
				usage(argv[0]); 
			}
			match_tb->node = i;
			match_tb->control_reg |= TB_CTRL_RCONNID;
			break;
		case 'a': 
			match_tb->addr = strtoll(optarg,NULL,0); 
			match_tb->control_reg |= TB_CTRL_RADDR;
			break;
		case 's': 
			match_tb->snoop_error_status = strtol(optarg,NULL,0);
			if (match_tb->snoop_error_status > 15) {
				printf("Range for -s option is 0-15\n");
				usage(argv[0]); 
			}
			match_tb->control_reg |= TB_CTRL_SSTAT;
			break;
		case 't': 
			match_tb->trans_id = strtol(optarg,NULL,0);
			if (match_tb->trans_id > 255) {
				printf("Range for -t option is 0-255\n");
				usage(argv[0]); 
			}
			match_tb->control_reg |= TB_CTRL_RTHREADID;
			break;
		case 'c': 
			for (i = 0; i < 8; i++) {
				if (!strcasecmp(optarg, cmdinfo[i].cmdname))
				    break;
			}
			if (i == 8) {
				printf("Range for -c option is 0-7\n");
				usage(argv[0]);
			}
			match_tb->cmd = i;
			match_tb->control_reg |= TB_CTRL_RCMD;
			break;
		case 'o': 
			match_tb->snoop_result = strtol(optarg,NULL,0);
			if (match_tb->snoop_result > 15) {
				printf("Range for -o option is 0-15\n");
				usage(argv[0]); 
			}
			match_tb->control_reg |= TB_CTRL_SRSLT;
			break;
		case 'k': 
			if (!strcasecmp(optarg, "yes"))
				match_tb->cacheable = 1; 
			else if (!strcasecmp(optarg, "no"))
				match_tb->cacheable = 0;
			else {
				printf("-k should be followed by yes/no\n");
				usage(argv[0]);
			}
			match_tb->control_reg |= TB_CTRL_RCACHE;
			break;
		case 'm': 
			if (!strcasecmp(optarg, "all"))
				match_tb->mode = 0; 
			else if (!strcasecmp(optarg, "matching")) {
				match_tb->mode = 1;
				match_tb->control_reg |=
					TB_CTRL_COLLMODE_MATCHONLY;
			} else {
				printf("-m should be followed by "
						"all/matching\n");
				usage(argv[0]);
			}
			break;
		case 'r':
			match_tb->num_req = strtol(optarg,NULL, 0);
			if (match_tb->num_req < 0 || match_tb->num_req > 255) {
				printf("Range for -r option is 0-255\n");
				usage(argv[0]); 
			}
			break;
		case 'x':
			match_tb->one_shot = 0;
			break;
		case 'e':
			/* More help */
			dump_help();
			break;
		case 'h':
		default: usage(argv[0]); break;
		}
	}

	if (!(match_tb->control_reg & 0xff)) {
		fprintf(stderr, "Using node=cpu0 as match criterion\n");
		match_tb->control_reg |= TB_CTRL_RCONNID;
	}

	/*

	if (optind < argc) {
		fprintf(stderr, "********************************************\n");
		for(index = optind; index < argc; index++) 
			fprintf(stderr, "Non-option argument \"%s\" \n", argv[index]);		
		fprintf(stderr, "********************************************\n");
		exit(-1);
	}

	*/
	return match_tb;

}

void set_match(struct match_tb *match_tb)
{
	// following defaults are used...

	spin_lock_init(&tb_data.tb_lock);
	match_tb->node = 0; // cpu 0
	match_tb->addr = 0; 
	match_tb->snoop_error_status = 0; 
	match_tb->trans_id = 0; 
	match_tb->cmd = 0; 
	match_tb->snoop_result = 0;  
	match_tb->cacheable = 0; 
	match_tb->mode = 0; 
	match_tb->num_req = 5; // 5 requests
	match_tb->sleep = 1; // 1 sec
	match_tb->one_shot = 1; 
	match_tb->control_reg = TB_CTRL_RCONNID; // match node
}

struct tb_entry samples[256];

void phoenix_tb_setup();

/* This function sets up the interrupt handler for Tracebuffer. 
 * Depending on the argc, argv params, it initializes the tracebuffer
 * with the user supplied values.
 */
struct match_tb *init_tb(int argc, char *argv[])
{
	struct match_tb *arg_tb;
	int cpu_id = processor_id();
	int pic_irq_tb;
	if (is_xls())
		pic_irq_tb = PIC_TB_XLS_IRQ;
	else 
		pic_irq_tb = PIC_TB_XLR_IRQ;
	arg_tb = parse_args(argc,argv);
	request_irq(pic_irq_tb, tb_int_handler, 0, 1 << cpu_id);
	printf("Initializing Tracebuffer...");
	program_tracebuffer(arg_tb);
	tb_reinit();
	printf(" Done.\n");
	return arg_tb;
}

void tracebuffer_loop(struct match_tb *arg_tb)
{
	int pic_irq_tb;

	if (is_xls())
		pic_irq_tb = PIC_TB_XLS_IRQ;
	else 
		pic_irq_tb = PIC_TB_XLR_IRQ;
	
	while (1) {
		collect_samples(samples);
		if (tb_data.size > 0) {
			printf("Number of sample collected = %d\n",tb_data.size/16 );
			print_tb_entries(samples, tb_data.size/16);

			if (arg_tb->one_shot)
				break;	

			tb_reinit();
			printf("\n");
		}
	}
	free_irq(pic_irq_tb);
}

void print_snoop_result(int input)
{
	int i, printed=0;
	if (!input) {
		printf(" None");
		return;
	}
	for (i = 0; i < 4; i++)
		if (input & (1 << i)) {
			if (printed)
				printf(" & ");
			printf("%s", srslt[i].srslt_name);
			printed = 1;
		}

}

void print_snoop_error_status(int input)
{
	int i, printed=0;
	if (!input) {
		printf(" None");
		return;
	}
	for (i = 0; i < 4; i++)
		if (input & (1 << i)) {
			if (printed)
				printf(" & ");
			printf("%s", snoop_error_status[i].sstat_name);
			printed = 1;
		}

}

void print_trans_retry(int input)
{
	int i, printed=0;
	if (!input) {
		printf(" None");
		return;
	}
	for (i = 0; i < 3; i++)
		if (input & (1 << i)) {
			if (printed)
				printf(" & ");
			printf("%s", sretry[i].sretry_name);
			printed = 1;
		}

}

int print_tb_entries(struct tb_entry samples[], int num_samples)
{
	static int print_info=0;
	struct tb_entry entry;
	uint64_t addr;
	uint32_t byte_cacheline;
	int i_nodeid, r_nodeid, cmd;
	unsigned int trans_id, l1_way, r_cache, l2_alloc, r_size;
	unsigned int tag_parity, snoop_result, trans_retry, snoop_error_stat;
	int i;
	if (!print_info) {
		printf("Interpretation of output:\n"
				" Cacheline: physical address >> 5.\n"
				" bitmask: This is a bitmask of 32 bytes"
				" accessed in the given cacheline \n"
				" \teg. If bytes 2 and 8 are accessed, bitmask"
				" value will be 0x104\n"
#if 0
				" Snoop result:\n"
				" \tFour bits in the snoop correspond to four" 
				" snoop results:\n"
				" \tbit 0: cpu shared, 1: cpu modified, 2: l2"
				" shared, 3: l2 modified\n"
				" Snoop error status: \n"
				" \tFour bits in the snoop correspond to four"
				" snoop errors:\n"
				" \tbit 0: Brigde, 1: l1 tag, 2: l2 ctag, 3: l2"
				" uctag\n"
				" Transaction retry:\n"
				" 3 bits in transaction retry correspond to:\n"
				" \t 0: cpu, 1: l2, 2:bridge\n"
#endif
				" Snoop error Status: Only printed if there is an"
				" error\n"
				" Transaction retry: Only printed if"
				" transaction retried.\n"
				" \n");
		print_info = 1;
	}


	for (i = 0; i <num_samples; i++) {
		memcpy(&entry, &samples[i], sizeof(struct tb_entry));

		addr = (((uint64_t)(entry.reg1 & 0xffffff00) << 8) |
				(((uint64_t)entry.reg0 >> 29)& 0x7));
		i_nodeid = (entry.reg0 >> 16) & 0x1f;
		r_nodeid = entry.reg0 & 0x1f;
		cmd = ((entry.reg3) >> 7) & 0x7;
		trans_id = (entry.reg0 >> 21) & 0xff;
		l1_way = (entry.reg3 >> 4) & 0x7;
		r_cache = (entry.reg3 >> 3) &  1;
		l2_alloc = (entry.reg3 >> 2) &  1;
		r_size = (entry.reg3 >> 1) &  1;
		tag_parity = entry.reg2 & 1;
		snoop_result = (entry.reg0 >> 12) & 0xf;
		trans_retry = (entry.reg0 >> 9) & 0x7;
		snoop_error_stat = (entry.reg0 >> 5) & 0xf;

		if (entry.reg3 & 0x2)
			byte_cacheline = 0xffffffff;
		else
			byte_cacheline = (entry.reg2 << 1) | 
				(entry.reg3 & 1 << 31);
		//printf("entry [%d] is: %x %x %x %x\n",num_entry, entry.reg0, 
		//entry.reg1, entry.reg2, entry.reg3);
		printf("######## Tracebuffer Entry : %d ########\n",i);

		printf("Cacheline: 0x%llx, bitmask: 0x%x\n"
				,(unsigned long long)addr, byte_cacheline);

		printf("Initiator: %s, Acceptor: %s, cmd: %s\n",nodeinfo[i_nodeid].nodename, nodeinfo[r_nodeid].nodename, cmdinfo[cmd].cmdname);

		printf("Transaction No: %d, Cachable: %s, l1 way: %d,"
				" l2 alloc: %s\n", trans_id,
				(r_cache==0)?"no":"yes",l1_way,
				(l2_alloc==0)?"no":"yes");
		printf("Snoop result: ");
		print_snoop_result(snoop_result);
		if (snoop_error_stat) {
			printf(", Snoop error: ");
			print_snoop_error_status(snoop_error_stat);
		}
		if (trans_retry) {
			printf(", Transaction retry: ");
			print_trans_retry(trans_retry);
		}
//		printf("\n");
//		printf("Snoop result: %d, Snoop error: %d, transaction retry: %d\n"
//				, snoop_result, snoop_error_stat, trans_retry);

	}

	return 0;
}
