#include <stdio.h>
#include <inttypes.h>
#ifndef __LINUX__
#include "mips-exts.h"
#include "mm.h"
#endif

#include "queue.h"
#include "vuart.h"

/* Read from FDT file */
static unsigned long uart_sharedcfg_base = KSEG0+0x1f000000;
static int port;

volatile queue_t *vuart_queues;

static void __uart_init(void *vuart_baseaddr, int nr_vuart, int initq)
{
	int i;

	vuart_queues = (queue_t *)vuart_baseaddr;

	if (initq) {
		for (i = 0; i < nr_vuart; i += 2 ) {
			/* 2 = 1 producer Q + 1 consumer Q */
			init_queue(&vuart_queues[i], MAX_ELEMENTS);
			init_queue(&vuart_queues[i+1], MAX_ELEMENTS);
		}
	}
}

int __inbyte(int vuart, int qid)
{
	int c;
	queue_t *queue = &vuart_queues[(vuart * 2) + qid];

	if (queue->init_done != 1)
		return EOF;

	while (dequeue(queue, &c) != 0)
		;

	return c;
}

int __inbyte_nowait(int vuart, int qid)
{
	int c;
	queue_t *queue = &vuart_queues[(vuart * 2) + qid];

	if (queue->init_done != 1)
		return EOF;

	if (dequeue(queue, &c) != 0)
		return EOF;

	return c;
}

static void __outbyte(char c, int vuart, int qid)
{
	queue_t *queue = &vuart_queues[(vuart * 2) + qid];

	if (queue->init_done != 1)
		return;

	while (enqueue(queue, c) != 0)
		;

	return;
}

// -----------
//  extern API
// -----------

void vuart_init(void)
{
	__uart_init((void *)uart_sharedcfg_base, NR_VUART, 0);
}


void vuart_outbyte(char c)
{
	__outbyte(c, port, DOMU_WRITE_Q);
}

int vuart_inbyte(void)
{
	return __inbyte(port, DOMU_READ_Q);
}

int vuart_inbyte_nowait(void)
{
	return __inbyte_nowait(port, DOMU_READ_Q);
}

void uart_update_params(uint64_t sharedcfg, int id)
{
	uart_sharedcfg_base = KSEG0 + (sharedcfg & 0x1fffffff);
	port = id;

	printf("(HE) cpu-%d q=0x%p\n", processor_id(), &vuart_queues[(port * 2) + DOMU_WRITE_Q]);

	printf("sharedcfg=%llx, uart_sharedcfg_base=%llx, port = %d\n",
	       (unsigned long long)sharedcfg, (unsigned long long)uart_sharedcfg_base, port);
}
