/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

// netconfig.c
// top level configuration file for XLR
// networking drivers

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "net_config.h"
#include "classifier.h"
#include "iomap.h"
#include "mac.h"
#include "gmac.h"
#include "xgmac.h"
#include "spi4.h"
#include "parser.h"
#include "mac_defines.h"
#include "shmvars.h"

/* global locks & done flags */

parser_tables  parser_tables_gmac ;
parser_tables  parser_tables_xgs0 ;
parser_tables  parser_tables_xgs1 ;
int net_conf_error_status;
int net_conf_bucket_offset = 4;

/* setup l3 cam table */
void setup_l3_ctable (cl_cfg_iface *classify_iface) {

	init_l3_table(p_l3_t);
	// CAM TABLE ENTRY 0 (All Ports and IPv4)
	p_l3_t->entry[0].l3_prot_key         = 0x0800 ;  // Ethernet Type for IPv4
	p_l3_t->entry[0].l2_prot             = L2_ETH ;
	p_l3_t->entry[0].l3_prot_mask        =  1 ;
	p_l3_t->entry[0].port                =  0 ; // Entry for All ports
	p_l3_t->entry[0].port_mask           =  0 ;
	p_l3_t->entry[0].ip_checksum_compute =  1 ;
	p_l3_t->entry[0].l4_prot_offset      =  9 ;
	p_l3_t->entry[0].l3_header_offset    =  0 ;
	p_l3_t->entry[0].offset_0            = 12 ; // IPV4 Source Address
	p_l3_t->entry[0].len_0               =  4 ; 
	p_l3_t->entry[0].offset_1            = 16 ; // IPV4 Destination Address
	p_l3_t->entry[0].len_1               =  4 ;
	p_l3_t->entry[0].offset_2            =  0 ; // Protocol field (This is zeroed out for now)
	p_l3_t->entry[0].len_2               =  0 ;

	// CAM TABLE ENTRY 1 (All Ports and IPv6)
	p_l3_t->entry[1].l3_prot_key         = 0x86DD   ; // ethernet type for IPv6 
	p_l3_t->entry[1].l2_prot             = L2_ETH ;   // port type
	p_l3_t->entry[1].l3_prot_mask        =  1 ;
	p_l3_t->entry[1].port                =  0 ; // Entry for All ports
	p_l3_t->entry[1].port_mask           =  0 ;
	p_l3_t->entry[1].ip_checksum_compute =  1 ;
	p_l3_t->entry[1].l4_prot_offset      =  6 ;
	p_l3_t->entry[1].l3_header_offset    =  0 ;
	p_l3_t->entry[1].offset_0            =  8 ; // IPV6 Source Address
	p_l3_t->entry[1].len_0               =  4 ; 
	p_l3_t->entry[1].offset_1            = 24 ; // IPV6 Destination Address
	p_l3_t->entry[1].len_1               =  4 ;
	p_l3_t->entry[1].offset_2            =  6 ; // Protocol field
	p_l3_t->entry[1].len_2               =  1 ;

	// CAM TABLE ENTRY 2 (All Ports and ARP)
	p_l3_t->entry[2].l3_prot_key         = 0x0806   ; // ethernet type for ARP
	p_l3_t->entry[2].l2_prot             = L2_ETH ;   // port type
	p_l3_t->entry[2].l3_prot_mask        =  1 ;
	p_l3_t->entry[2].port                =  0 ; // 
	p_l3_t->entry[2].port_mask           =  0 ; // Port mask set to zero
	p_l3_t->entry[2].ip_checksum_compute =  0 ;
	p_l3_t->entry[2].l4_prot_offset      =  0 ;
	p_l3_t->entry[2].l3_header_offset    =  0 ;
	p_l3_t->entry[2].offset_0            =  0 ; 
	p_l3_t->entry[2].len_0               =  0 ; 
	p_l3_t->entry[2].offset_1            =  0 ; 
	p_l3_t->entry[2].len_1               =  0 ;
	p_l3_t->entry[2].offset_2            =  0 ; 
	p_l3_t->entry[2].len_2               =  0 ;

	// CAM TABLE ENTRY 3 (All Ports and Custom Entry)
	p_l3_t->entry[3].l3_prot_key         =  classify_iface->ethertype  ; 
	p_l3_t->entry[3].l2_prot             =  L2_ETH ;   // port type
	p_l3_t->entry[3].l3_prot_mask        =  1 ;
	p_l3_t->entry[3].port                =  0 ; // 
	p_l3_t->entry[3].port_mask           =  0 ; // Port mask set to zero
	p_l3_t->entry[3].ip_checksum_compute =  1 ;
	p_l3_t->entry[3].l4_prot_offset      =  classify_iface->offset_l4_prot ;
	p_l3_t->entry[3].l3_header_offset    =  0 ;
	p_l3_t->entry[3].offset_0            =  classify_iface->offset_1 ; 
	p_l3_t->entry[3].len_0               =  classify_iface->len_1    ; 
	p_l3_t->entry[3].offset_1            =  classify_iface->offset_2 ; 
	p_l3_t->entry[3].len_1               =  classify_iface->len_2    ;
	p_l3_t->entry[3].offset_2            =  classify_iface->offset_3 ; 
	p_l3_t->entry[3].len_2               =  classify_iface->len_3    ;

	// CAM TABLE ENTRY 4 (All Ports and IPv4)
	p_l3_t->entry[4].l3_prot_key         = 0x8847;  // Ethernet Type for MPLS
	p_l3_t->entry[4].l2_prot             = L2_ETH;
	p_l3_t->entry[4].l3_prot_mask        =  1;
	p_l3_t->entry[4].port                =  0; // Entry for All ports
	p_l3_t->entry[4].port_mask           =  0;
	p_l3_t->entry[4].ip_checksum_compute =  1;
	p_l3_t->entry[4].l4_prot_offset      =  9;
	p_l3_t->entry[4].l3_header_offset    =  4;  // adding 4 bytes between l2 and l3
	p_l3_t->entry[4].offset_0            = 12; // IPV4 Source Address
	p_l3_t->entry[4].len_0               =  4;
	p_l3_t->entry[4].offset_1            = 16; // IPV4 Destination Address
	p_l3_t->entry[4].len_1               =  4;
	p_l3_t->entry[4].offset_2            =  0; 
	p_l3_t->entry[4].len_2               =  0;

	// Entries 5 to 15 set to init values.
}

// setup l3 cam table
void setup_l4_ctable (cl_cfg_iface *classify_iface) {

	init_l4_table(p_l4_t);

	// CAM TABLE ENTRY 0 (All Ports and TCP)
	p_l4_t->entry[0].l4_prot_key          = 6 ; // 6 is for TCP
	p_l4_t->entry[0].port                 = 0 ; // 
	p_l4_t->entry[0].l3_addr              = 0 ; // 
	p_l4_t->entry[0].l4_prot_mask         = 1 ; // 
	p_l4_t->entry[0].port_mask            = 0 ; // 
	p_l4_t->entry[0].l3_addr_mask         = 0 ; // 
	p_l4_t->entry[0].reserved             = 0 ;
	p_l4_t->entry[0].len_4                = 2 ;
	p_l4_t->entry[0].offset_4             = 2 ; // TCP Destination Port
	p_l4_t->entry[0].len_3                = 2 ;
	p_l4_t->entry[0].offset_3             = 0 ; // TCP Source Port
	p_l4_t->entry[0].reserved_2           = 0 ;

	// CAM TABLE ENTRY 1 (All Ports and UDP)
	p_l4_t->entry[1].l4_prot_key          = 17; // 17 is for UDP
	p_l4_t->entry[1].port                 = 0 ; // 
	p_l4_t->entry[1].l3_addr              = 0 ; // 
	p_l4_t->entry[1].l4_prot_mask         = 1 ; // 
	p_l4_t->entry[1].port_mask            = 0 ; // 
	p_l4_t->entry[1].l3_addr_mask         = 0 ; // 
	p_l4_t->entry[1].reserved             = 0 ;
	p_l4_t->entry[1].len_4                = 2 ;
	p_l4_t->entry[1].offset_4             = 2 ; // UDP Destination Port
	p_l4_t->entry[1].len_3                = 2 ;
	p_l4_t->entry[1].offset_3             = 0 ; // UDP Source Port
	p_l4_t->entry[1].reserved_2           = 0 ;

	// CAM TABLE ENTRY 2 (All Ports and Custom L4)
	p_l4_t->entry[2].l4_prot_key          = classify_iface->ethertype;
	p_l4_t->entry[2].port                 = 0 ; // 
	p_l4_t->entry[2].l3_addr              = 0 ; // 
	p_l4_t->entry[2].l4_prot_mask         = 1 ; // 
	p_l4_t->entry[2].port_mask            = 0 ; // 
	p_l4_t->entry[2].l3_addr_mask         = 0 ; // 
	p_l4_t->entry[2].reserved             = 0 ;
	p_l4_t->entry[2].offset_3             = classify_iface->offset_4 ; // 
	p_l4_t->entry[2].len_3                = classify_iface->len_4 ;
	p_l4_t->entry[2].offset_4             = classify_iface->offset_5 ; // 
	p_l4_t->entry[2].len_4                = classify_iface->len_5 ;
	p_l4_t->entry[2].reserved_2           = 0 ;

	// Entries 3 to 7 set to init values.
}

void save_parser_tables (parser_tables *parser_tables_v) {
	memcpy( &(parser_tables_v->prsr_cfg_entry_v), p_prsr_cfg, sizeof(prsr_cfg_entry)) ;
	memcpy( &(parser_tables_v->l2_table_v), p_l2_t, sizeof(l2_table)) ;
	memcpy( &(parser_tables_v->l3_table_v), p_l3_t, sizeof(l3_table)) ;
	memcpy( &(parser_tables_v->l4_table_v), p_l4_t, sizeof(l4_table)) ;
}

// network init function
void xlr_classifier_init(classifier_config *classifier_conf) {
	int i = 0;
	struct misc_shm_vars *m;

	if((m = nlm_misc_shm_init()) == NULL)
		return ;

	// ensure that init happens only once, even if all threads call this function
	spin_lock (&m->classifier_init_lock);

	if (!m->classifier_init_done) {
		// configure gmac classifier
		if ( classifier_conf->classify_gmac.enable ) {
			// classification method
			init_prsr_cfg(p_prsr_cfg);
			p_prsr_cfg->use_hash        =  (classifier_conf->classify_gmac.ttable_mode == USE_HASH) ;
			 p_prsr_cfg->use_proto       =  (classifier_conf->classify_gmac.ttable_mode == USE_L3L4) ;
			p_prsr_cfg->use_global_port =  (classifier_conf->classify_gmac.ttable_mode == USE_PORT) ;
			p_prsr_cfg->crc_hash_poly = 127 ; 
			p_prsr_cfg->parse_depth = 8 ; 
			p_prsr_cfg->poly_mask = 0x7f ; 

			// setup l2 table
			init_l2_table(p_l2_t);
			for ( i = 0 ; i < 4 ; i ++ ) {
				p_l2_t->entry[i].l2_proto             = L2_ETH ;
				p_l2_t->entry[i].extraHdrProtoSize    = classifier_conf->classify_gmac.len_0;
				p_l2_t->entry[i].extraHdrProtoOffset  = classifier_conf->classify_gmac.offset_0;
				p_l2_t->entry[i].extraHdrSize         = 0;
				p_l2_t->entry[i].proto_offset         = classifier_conf->classify_gmac.offset_l3_prot;
			}

			// setup the l3 table
			setup_l3_ctable(&(classifier_conf->classify_gmac)) ;

			// setup the l4 table
			setup_l4_ctable(&(classifier_conf->classify_gmac)) ;

			// save all classifier structure for gmac
			save_parser_tables(&parser_tables_gmac) ;
		}
		
		// configure xgs0 classifier
		if ( classifier_conf->classify_xgs0.enable ) {
			// classification method
			init_prsr_cfg(p_prsr_cfg);
			p_prsr_cfg->use_hash        =  (classifier_conf->classify_xgs0.ttable_mode == USE_HASH) ;
			p_prsr_cfg->use_proto       =  (classifier_conf->classify_xgs0.ttable_mode == USE_L3L4) ;
			p_prsr_cfg->use_global_port =  (classifier_conf->classify_xgs0.ttable_mode == USE_PORT) ;
			p_prsr_cfg->crc_hash_poly = 127 ; 
			p_prsr_cfg->parse_depth = 8 ; 
			p_prsr_cfg->poly_mask = 0x7f ; 

			// setup l2 table
			init_l2_table(p_l2_t);
			for ( i = 0 ; i < 16 ; i ++ ) {
				p_l2_t->entry[i].l2_proto             = L2_ETH ;
				p_l2_t->entry[i].extraHdrProtoSize    = classifier_conf->classify_xgs0.len_0;
				p_l2_t->entry[i].extraHdrProtoOffset  = classifier_conf->classify_xgs0.offset_0;
				p_l2_t->entry[i].extraHdrSize         = 0;
				p_l2_t->entry[i].proto_offset         = classifier_conf->classify_xgs0.offset_l3_prot;
			}

			// setup the l3 table
			setup_l3_ctable(&(classifier_conf->classify_xgs0)) ;

			// setup the l4 table
			setup_l4_ctable(&(classifier_conf->classify_xgs0)) ;

			// save all classifier structure for xgs0
			save_parser_tables(&parser_tables_xgs0) ;
		}
		
		// configure xgs1 classifier
		if ( classifier_conf->classify_xgs1.enable ) {
			// classification method
			init_prsr_cfg(p_prsr_cfg);
			p_prsr_cfg->use_hash        =  (classifier_conf->classify_xgs1.ttable_mode == USE_HASH) ;
			p_prsr_cfg->use_proto       =  (classifier_conf->classify_xgs1.ttable_mode == USE_L3L4) ;
			p_prsr_cfg->use_global_port =  (classifier_conf->classify_xgs1.ttable_mode == USE_PORT) ;
			p_prsr_cfg->crc_hash_poly = 127 ; 
			p_prsr_cfg->parse_depth = 8 ; 
			p_prsr_cfg->poly_mask = 0x7f ; 

			// setup l2 table
			init_l2_table(p_l2_t);
			for ( i = 0 ; i < 16 ; i ++ ) {
				p_l2_t->entry[i].l2_proto             = L2_ETH ;
				p_l2_t->entry[i].extraHdrProtoSize    = classifier_conf->classify_xgs1.len_0;
				p_l2_t->entry[i].extraHdrProtoOffset  = classifier_conf->classify_xgs1.offset_0;
				p_l2_t->entry[i].extraHdrSize         = 0;
				p_l2_t->entry[i].proto_offset         = classifier_conf->classify_xgs1.offset_l3_prot;
			}

			// setup the l3 table
			setup_l3_ctable(&(classifier_conf->classify_xgs1)) ;

			// setup the l4 table
			setup_l4_ctable(&(classifier_conf->classify_xgs1)) ;

			// save all classifier structure for xgs1
			save_parser_tables(&parser_tables_xgs1) ;
		}
	}
	m->classifier_init_done =1;
	spin_unlock(&m->classifier_init_lock);
}

/* xlr_classifier update writes the values into the chip */

void xlr_classifier_update(classifier_config *classifier_conf, net_config *net_conf) {
		phoenix_reg_t          *mmio ;
		struct misc_shm_vars *m;

		if((m = nlm_misc_shm_init()) == NULL)
			return ;


		printf("Entered xlr_classifier_update: BEFORE spinlock\n");

		// ensure that init happens only once, even if all threads call this function
		spin_lock (&m->classifier_init_lock);
		printf("Entered xlr_classifier_update: AFTER spinlock\n");
		// configure gmac classifier
		if ( classifier_conf->classify_gmac.enable ) {

				if(!(net_conf->gmac0 || net_conf->gmac1 || net_conf->gmac2 || net_conf->gmac3))
						goto skip_gmac_0;
				
				mmio = (phoenix_reg_t *)(phoenix_io_base + PHOENIX_IO_GMAC_0_OFFSET);
				if(net_conf->gmac0)
					update_prsr_cfg(&(parser_tables_gmac.prsr_cfg_entry_v), mmio); // Write the values into chip
				if(net_conf->gmac1)
					update_prsr_cfg(&(parser_tables_gmac.prsr_cfg_entry_v), mmio+0x400); // Write the values into chip
				if(net_conf->gmac2)
					update_prsr_cfg(&(parser_tables_gmac.prsr_cfg_entry_v), mmio+0x800); // Write the values into chip
				if(net_conf->gmac3)
					update_prsr_cfg(&(parser_tables_gmac.prsr_cfg_entry_v), mmio+0xC00); // Write the values into chip

				update_l2_table_block(GMAC_MODE, &(parser_tables_gmac.l2_table_v), mmio, 4, 0, net_conf); // Write the values into chip
				update_l3_table( &(parser_tables_gmac.l3_table_v), mmio ) ; // Write the table into the chip
				update_l4_table( &(parser_tables_gmac.l4_table_v), mmio ) ; // Write the table into the chip
				update_translate_table(&(parser_tables_gmac.ttable_v), mmio);

				update_cam_results_table( &(parser_tables_gmac.tcam_ttable_v), mmio);
				update_cam_table( &(parser_tables_gmac.cam_table_v), mmio);


				// prepad enable (per interface register)
				if(net_conf->gmac0)
					phoenix_write_reg(mmio, DESC_PACK_CTRL, phoenix_read_reg (mmio, DESC_PACK_CTRL)| FIELD_VALUE(BYTE_OFFSET, classifier_conf->classify_gmac.byte_offset) | FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_gmac.prepad_enable) ) ;
				if(net_conf->gmac1)
					phoenix_write_reg((mmio +0x400), DESC_PACK_CTRL, phoenix_read_reg ((mmio +0x400), DESC_PACK_CTRL)|FIELD_VALUE(BYTE_OFFSET, classifier_conf->classify_gmac.byte_offset) | FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_gmac.prepad_enable) ) ;
				if(net_conf->gmac2)
					phoenix_write_reg((mmio +0x800), DESC_PACK_CTRL, phoenix_read_reg ((mmio +0x800), DESC_PACK_CTRL)|FIELD_VALUE(BYTE_OFFSET, classifier_conf->classify_gmac.byte_offset) |FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_gmac.prepad_enable) ) ;
				if(net_conf->gmac3)
					phoenix_write_reg((mmio + 0xc00), DESC_PACK_CTRL, phoenix_read_reg ((mmio +0xC00), DESC_PACK_CTRL)| FIELD_VALUE(BYTE_OFFSET, classifier_conf->classify_gmac.byte_offset) |FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_gmac.prepad_enable) ) ;
#if 0 /* gmac_open is deprecated */

				if (classifier_conf->classify_gmac.rx_enable) {
						if(net_conf->gmac0)
							gmac_open((unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_0_OFFSET));
						if(net_conf->gmac1)
							gmac_open((unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_1_OFFSET));
						if(net_conf->gmac2)
							gmac_open((unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_2_OFFSET));
						if(net_conf->gmac3)
							gmac_open((unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_3_OFFSET));
				}
#endif
skip_gmac_0:

				if(!(net_conf->gmac4 || net_conf->gmac5 || net_conf->gmac6 || net_conf->gmac7))
						goto skip_gmac_1;

				mmio = (phoenix_reg_t *)(phoenix_io_base + PHOENIX_IO_GMAC_4_OFFSET);

				if(net_conf->gmac4)
						update_prsr_cfg(&(parser_tables_gmac.prsr_cfg_entry_v), mmio); // Write the values into chip
				if(net_conf->gmac5)
						update_prsr_cfg(&(parser_tables_gmac.prsr_cfg_entry_v), mmio+0x400); // Write the values into chip
				if(net_conf->gmac6)
						update_prsr_cfg(&(parser_tables_gmac.prsr_cfg_entry_v), mmio+0x800); // Write the values into chip
				if(net_conf->gmac7)
					update_prsr_cfg(&(parser_tables_gmac.prsr_cfg_entry_v), mmio+0xC00); // Write the values into chip

				update_l2_table_block(GMAC_MODE, &(parser_tables_gmac.l2_table_v), mmio, 4, 1, net_conf); // Write the values into chip
				update_l3_table( &(parser_tables_gmac.l3_table_v), mmio ) ; // Write the table into the chip
				update_l4_table( &(parser_tables_gmac.l4_table_v), mmio ) ; // Write the table into the chip
				update_translate_table(&(parser_tables_gmac.ttable_v), mmio);

				update_cam_results_table( &(parser_tables_gmac.tcam_ttable_v), mmio);
				update_cam_table( &(parser_tables_gmac.cam_table_v), mmio);

				// prepad enable (per interface register)
				if(net_conf->gmac4)
					phoenix_write_reg(mmio, DESC_PACK_CTRL, phoenix_read_reg (mmio, DESC_PACK_CTRL)|FIELD_VALUE(BYTE_OFFSET, classifier_conf->classify_gmac.byte_offset) |FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_gmac.prepad_enable) ) ;
				if(net_conf->gmac5)
					phoenix_write_reg((mmio +0x400), DESC_PACK_CTRL, phoenix_read_reg ((mmio +0x400), DESC_PACK_CTRL)|FIELD_VALUE(BYTE_OFFSET, classifier_conf->classify_gmac.byte_offset) |FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_gmac.prepad_enable) ) ;
				if (net_conf->gmac6)
						phoenix_write_reg((mmio +0x800), DESC_PACK_CTRL, phoenix_read_reg ((mmio +0x800), DESC_PACK_CTRL)|FIELD_VALUE(BYTE_OFFSET, classifier_conf->classify_gmac.byte_offset) | FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_gmac.prepad_enable) ) ;
				if (net_conf->gmac7)
						phoenix_write_reg((mmio + 0xc00), DESC_PACK_CTRL, phoenix_read_reg ((mmio +0xC00), DESC_PACK_CTRL)|	FIELD_VALUE(BYTE_OFFSET, classifier_conf->classify_gmac.byte_offset) |FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_gmac.prepad_enable) ) ;

#if 0 /* gmac_open is deprecated */
				if (classifier_conf->classify_gmac.rx_enable) {
					if (net_conf->gmac4)
						gmac_open((unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_4_OFFSET));
					if (net_conf->gmac5)
						gmac_open((unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_5_OFFSET));
					if (net_conf->gmac6)
						gmac_open((unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_6_OFFSET));
					if (net_conf->gmac7)
						gmac_open((unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_7_OFFSET));
				}
#endif
		}
skip_gmac_1:
		// configure xgs0 classifier
		if ( classifier_conf->classify_xgs0.enable ) {
				printf("Entered xlr_classifier_update: AFTER classifier_conf->classify_xgs0.enable\n");
				if (net_conf->xgmac0) {
						mmio = (phoenix_reg_t *)(phoenix_io_base + PHOENIX_IO_XGMAC_0_OFFSET) ;
				} else {
						mmio = (phoenix_reg_t *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET) ;
				}
				update_prsr_cfg(&(parser_tables_xgs0.prsr_cfg_entry_v), mmio); // Write the values into chip

				update_l2_table_xgs(&(parser_tables_xgs0.l2_table_v), mmio); // Write the values into chip

				update_l3_table( &(parser_tables_xgs0.l3_table_v), mmio ) ; // Write the table into the chip
				update_l4_table( &(parser_tables_xgs0.l4_table_v), mmio ) ; // Write the table into the chip
				update_translate_table(&(parser_tables_xgs0.ttable_v), mmio);
				update_cam_results_table( &(parser_tables_xgs0.tcam_ttable_v), mmio);
				update_cam_table( &(parser_tables_xgs0.cam_table_v), mmio);

				phoenix_write_reg(mmio, DESC_PACK_CTRL, phoenix_read_reg (mmio, DESC_PACK_CTRL)|FIELD_VALUE(BYTE_OFFSET, MAC_BYTE_OFFSET) |	FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_xgs0.prepad_enable) ) ;
				if (net_conf->xgmac0) {
						phoenix_write_reg(mmio, BYTE_OFFSET_0_7, classifier_conf->classify_xgs0.byte_offset);
				}
				else {
						phoenix_write_reg(mmio, BYTE_OFFSET_0_7,
										classifier_conf->classify_xgs0.byte_offset|
										(classifier_conf->classify_xgs0.byte_offset<<3)|
										(classifier_conf->classify_xgs0.byte_offset<<6)|
										(classifier_conf->classify_xgs0.byte_offset<<9)|
										(classifier_conf->classify_xgs0.byte_offset<<12)|
										(classifier_conf->classify_xgs0.byte_offset<<15)|
										(classifier_conf->classify_xgs0.byte_offset<<18)|
										(classifier_conf->classify_xgs0.byte_offset<<21));
						phoenix_write_reg(mmio, BYTE_OFFSET_8_F,
										classifier_conf->classify_xgs0.byte_offset|
										(classifier_conf->classify_xgs0.byte_offset<<3)|
										(classifier_conf->classify_xgs0.byte_offset<<6)|
										(classifier_conf->classify_xgs0.byte_offset<<9)|
										(classifier_conf->classify_xgs0.byte_offset<<12)|
										(classifier_conf->classify_xgs0.byte_offset<<15)|
										(classifier_conf->classify_xgs0.byte_offset<<18)|
										(classifier_conf->classify_xgs0.byte_offset<<21));
				}
		}

		// configure xgs1 classifier
		if ( classifier_conf->classify_xgs1.enable ) {
				printf("Entered xlr_classifier_update: AFTER classifier_conf->classify_xgs1.enable\n");
				if (net_conf->xgmac1) {
						mmio = (phoenix_reg_t *)(phoenix_io_base + PHOENIX_IO_XGMAC_1_OFFSET) ;
				} else {
						mmio = (phoenix_reg_t *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET) ;
				}
				update_prsr_cfg(&(parser_tables_xgs1.prsr_cfg_entry_v), mmio); // Write the values into chip

				update_l2_table_xgs(&(parser_tables_xgs1.l2_table_v), mmio); // Write the values into chip

				update_l3_table( &(parser_tables_xgs1.l3_table_v), mmio ) ; // Write the table into the chip
				update_l4_table( &(parser_tables_xgs1.l4_table_v), mmio ) ; // Write the table into the chip
				update_translate_table(&(parser_tables_xgs1.ttable_v), mmio);
				update_cam_results_table( &(parser_tables_xgs1.tcam_ttable_v), mmio);
				update_cam_table( &(parser_tables_xgs1.cam_table_v), mmio);

				phoenix_write_reg(mmio, DESC_PACK_CTRL, phoenix_read_reg (mmio, DESC_PACK_CTRL)|
								FIELD_VALUE(BYTE_OFFSET, MAC_BYTE_OFFSET) |
								FIELD_VALUE(PREPAD_ENABLE, classifier_conf->classify_xgs1.prepad_enable));
				if (net_conf->xgmac1) {
						phoenix_write_reg(mmio, BYTE_OFFSET_0_7, classifier_conf->classify_xgs1.byte_offset);
				}
				else {
						phoenix_write_reg(mmio, BYTE_OFFSET_0_7,
										classifier_conf->classify_xgs1.byte_offset|
										(classifier_conf->classify_xgs1.byte_offset<<3)|
										(classifier_conf->classify_xgs1.byte_offset<<6)|
										(classifier_conf->classify_xgs1.byte_offset<<9)|
										(classifier_conf->classify_xgs1.byte_offset<<12)|
										(classifier_conf->classify_xgs1.byte_offset<<15)|
										(classifier_conf->classify_xgs1.byte_offset<<18)|
										(classifier_conf->classify_xgs1.byte_offset<<21));
						phoenix_write_reg(mmio, BYTE_OFFSET_8_F,
										classifier_conf->classify_xgs1.byte_offset|
										(classifier_conf->classify_xgs1.byte_offset<<3)|
										(classifier_conf->classify_xgs1.byte_offset<<6)|
										(classifier_conf->classify_xgs1.byte_offset<<9)|
										(classifier_conf->classify_xgs1.byte_offset<<12)|
										(classifier_conf->classify_xgs1.byte_offset<<15)|
										(classifier_conf->classify_xgs1.byte_offset<<18)|
										(classifier_conf->classify_xgs1.byte_offset<<21));
				}
		}
}

int find_next_bucket(uint64_t bucket_mask, int *curr_ptr) {
	 int i ;
	 int search_start = 0 ;

	 for ( i = *curr_ptr ; !((i == *curr_ptr) && search_start) ; i = (i+1)%64 ) {
		 search_start = 1 ;
		 if ((bucket_mask >> i) & 0x1) {
			             *curr_ptr = (i+1)%64 ;
			 return (i) ;
		 }
	 }
	 return (-1) ;
}

void ttable_setup_mask (uint32_t cpu_mask, uint32_t use_bucket, cl_cfg_iface *classify_iface, parser_tables *parser_tables_p) {
	unsigned int pde_cpu_mask_0 =0;
	unsigned int pde_cpu_mask_1 =0;
	unsigned int tmp_mask[8];
	unsigned int cpu_mask_t; 
	uint64_t bucket_mask ;
	int start_ptr = 0 ;
	int i;

	// setup first 32 bits
	for (i=0;i<4;i++) {   
		tmp_mask[i] = ((cpu_mask >> (i*4)) & 0x0f) << ((i*8)+net_conf_bucket_offset);
		pde_cpu_mask_0 = pde_cpu_mask_0 | tmp_mask[i];
	}
	// setup 2nd 32 bits
	cpu_mask_t = cpu_mask >> 16;
	for (i=0;i<4;i++) {
		tmp_mask[i] = ((cpu_mask_t >> (i*4)) & 0x0f) << ((i*8)+net_conf_bucket_offset);
		pde_cpu_mask_1 = pde_cpu_mask_1 | tmp_mask[i];
	}
	bucket_mask = ((uint64_t)pde_cpu_mask_1 << 32) | pde_cpu_mask_0 ;

	init_translate_table(&(parser_tables_p->ttable_v));
	init_tcam_translate_table(&(parser_tables_p->tcam_ttable_v));
	parser_tables_p->ttable_v.num_entries = 128 ;
	parser_tables_p->tcam_ttable_v.num_entries = 4;
	for ( i = 0 ; i < 128 ; i ++ ) {
		parser_tables_p->ttable_v.entry[i].class_id      = i%(classify_iface->num_classes) ;
		parser_tables_p->ttable_v.entry[i].bucket_id     = find_next_bucket (bucket_mask, &start_ptr) ;
		parser_tables_p->ttable_v.entry[i].use_bucket_id = use_bucket ;
		if (parser_tables_p->ttable_v.entry[i].bucket_id == -1) 
			printf ("ttable_setup_mask: ttable_entry[%d] mask is all zeroes: mask %x %llx\n",i,cpu_mask,(unsigned long long)bucket_mask) ;
	}
	start_ptr = 0;
	for ( i = 0 ; i < 4 ; i ++ ) {
		parser_tables_p->tcam_ttable_v.entry[i].class_id = 
				i%(classify_iface->num_classes) ;
		parser_tables_p->tcam_ttable_v.entry[i].bucket_id = 
				find_next_bucket (bucket_mask, &start_ptr) ;
		parser_tables_p->tcam_ttable_v.entry[i].use_bucket_id = use_bucket ;
		if (parser_tables_p->tcam_ttable_v.entry[i].bucket_id == -1) 
			printf ("ttable_setup_mask: tcam_ttable_entry[%d] mask is all zeroes: mask %x %llx\n",i,cpu_mask,(unsigned long long)bucket_mask) ;
	}
}

void ttable_port_based_setup (cl_cfg_iface *classify_iface, parser_tables *parser_tables_p) {
	int i ;
	init_translate_table(&(parser_tables_p->ttable_v));
	parser_tables_p->ttable_v.num_entries = 16 ;
	for ( i = 0 ; i < 16 ; i ++ ) { // in case of gmac 4 to 15 are unused
		parser_tables_p->ttable_v.entry[i].class_id      = i%(classify_iface->num_classes) ;
		parser_tables_p->ttable_v.entry[i].use_bucket_id = 0 ;
		if (parser_tables_p->ttable_v.entry[i].bucket_id == -1) 
			printf ("ttable_setup_mask: mask is all zeroes\n") ;
	}
}
