
/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "system.h"
#include "contig_alloc.h"
#include "device.h"
#include "gmac.h"
#include "spi4.h"
#include "xgmac.h"
#include "atomic.h"
#include "mac.h"
#include "mac_defines.h"
#include "shmvars.h"

extern int sgmii_dc_present(int block);
int is_valid_gmac(int gmac);


static  char *device_error_string[11] =     { 
				"Success",
				"Auto Negotiation is not complete",
				"Device structure is incorrect",
				"Operation not supported",
				"This interface is not supported on this board",
				"Port Number is incorrect",
				"Running out of memory",
				"Initialization is not needed",
				"Board is not XLS",
				"Gmac id is invalid",
				"XAUI mode",

};
				 


device_conf * nlm_alloc_device (int id, int type)
{
	device_conf * current_dev = (device_conf * ) contig_alloc( sizeof (device_conf));
	int i =6;
	port_info * intrl = NULL;
	if ( current_dev == NULL ) {
		printf("Memory allocation failed \n");
		return NULL;
	}
	memset(current_dev,0,sizeof(device_conf));
			
	current_dev->magic = 0xdeadbeef;
	current_dev->id = id;
	current_dev->type = type;
	current_dev->max_tx_frame_len = MAC_FRAME_SIZE;
	current_dev->max_rx_frame_len = MAC_FRAME_SIZE;
	current_dev->rx_regular_buf_size = MAC_FRAME_SIZE;
	current_dev->l2_alloc_rx = 0xff;
	current_dev->l2_alloc_tx = 0xff; 
	current_dev->enable_stat = 0; 
	current_dev->enable_prepad = 0;
	for(i =0; i < MAX_SPI4_PORT; i++) {
		current_dev->byte_offset[i] = 0;
		current_dev->phy_conf[i].auto_neg = 1;
		current_dev->phy_conf[i].speed = 100;
		current_dev->phy_conf[i].duplex = 0;
	}

	intrl = ( port_info *) contig_alloc(sizeof( port_info ));

	if ( intrl == NULL ) {
		contig_free(current_dev);
		printf("Memory allocation failed\n");
		return NULL;
	}
	memset(intrl,0,sizeof(port_info));
	if ( type == TYPE_NLM_XGS )
		intrl->block_id = id + 2;
	if ( type == TYPE_NLM_GMAC )
		intrl->block_id = id/4; 

	current_dev->intrl = (void *)intrl;

	return current_dev;
}

int check_device_config( device_conf * current_dev )
{
	int stat = 0;
	int i =0;
	port_info * internal = ( port_info *)current_dev->intrl;
	int block = internal->block_id; 
	struct netblk_shm_vars *m;

	if((m = nlm_netblk_shm_init()) == NULL)
		return ERR_NO_MEM;


	if ( current_dev->magic != 0xdeadbeef ) {
		printf("Use nlm_alloc_device() to allocate the structure\n");
		stat = ERR_INVALID_DEVICE_CONFIG;
	}

	if ( (current_dev->type != TYPE_NLM_GMAC) && (current_dev->type != TYPE_NLM_XGS )) {
		printf("Error in Interface type\n");
		stat = ERR_INVALID_DEVICE_CONFIG;
	}

	if ( ( (current_dev->type == TYPE_NLM_GMAC ) && (current_dev->id > 7 ) ) ||
		( ( current_dev->type == TYPE_NLM_XGS ) && ( current_dev->id > 1 ))){
		printf("Error in interface Id\n");
		stat = ERR_INVALID_DEVICE_CONFIG;
	}

	if ( current_dev->max_tx_frame_len > 0xffff ) {
		printf("Error. Max transmit frame size should not exceed %d",0xffff);
		stat = ERR_INVALID_DEVICE_CONFIG; 
	}

	if ( current_dev->max_rx_frame_len > 0xffff ) {
		printf("Error. Max receive frame size should not exceed %d",0xffff );
		stat = ERR_INVALID_DEVICE_CONFIG;
	}

	if ( current_dev->rx_regular_buf_size > 0x3fff ) {
		printf("Error. Max size of the message in message ring is %d",0x3fff); 
		stat = ERR_INVALID_DEVICE_CONFIG;
	}
	/* We don't support rx buf allocs for sizes > MAC_FRAME_SIZE */
	if(current_dev->rx_regular_buf_size > MAC_FRAME_SIZE) {
		if((void *)(m->blk_conf[block].pkt_mem->alloc_rx_buf) == 
				(void *)alloc_pbuf){
			printf("Error:");
			printf("rx_regular_buf_size > %d is not supported\n", 
					MAC_FRAME_SIZE);
			return ERR_INVALID_DEVICE_CONFIG;
		}
	}

	if ( current_dev->l2_alloc_rx > 0x1ff || current_dev->l2_alloc_tx > 0x1ff ) {
		printf("Error. The rx/tx L2 alloc size allowed is %d\n",0x1ff );
		stat = ERR_INVALID_DEVICE_CONFIG;
	}

	for(i =0; i < MAX_SPI4_PORT; i++) {
		if (  current_dev->byte_offset[i] > 7 ) {
			printf("Error. Byte offest penlmtted is 7\n");
			stat = ERR_INVALID_DEVICE_CONFIG;
		} 
	}

	return stat ;

}



void * alloc_pbuf_default( void * arg)
{
	void * pkt = (void *) alloc_pbuf();
	return pkt;
}

void free_pbuf_default( void * arg, void *data) 
{
	free_pbuf((struct packet *) data);
	return ;
}

uint64_t virt_to_phys_default( void * vaddr)
{
	uint64_t phys = virt_to_phys( (volatile void *)vaddr);
	return phys;
}

int nlm_net_device_block_init( device_conf * current_dev ,  struct block_config * block_data ) 
{
	int i =0;
	port_info * internal = ( port_info *)current_dev->intrl;
	int block = internal->block_id; 
	struct netblk_shm_vars *m;


	if ( (current_dev->type == TYPE_NLM_XGS) && is_xls())
		return INIT_NOT_NEEDED;

	if ( current_dev->type == TYPE_NLM_GMAC) {
		if ( !is_valid_gmac(current_dev->id))
			return INIT_NOT_NEEDED;
	}

	if((m = nlm_netblk_shm_init()) == NULL)
		return ERR_NO_MEM;

	spin_lock(&m->block_init_lock[block]);

	if ( m->block_init[block] == 1 ) {
		spin_unlock(&m->block_init_lock[block]);
		return INIT_NOT_NEEDED;
	}
		
	if ( block_data->pkt_mem != NULL)
		m->blk_conf[block].pkt_mem = block_data->pkt_mem;
	else {

		m->blk_conf[block].pkt_mem = (packet_mem_rt *)contig_alloc(sizeof(packet_mem_rt));

		if ( m->blk_conf[block].pkt_mem ==  NULL) {
			printf("Memory allocation failed\n");
			spin_unlock(&m->block_init_lock[block]);
			return ERR_NO_MEM;
		}
			

		m->blk_conf[block].pkt_mem->alloc_rx_buf = alloc_pbuf_default;
		m->blk_conf[block].pkt_mem->free_rx_buf = free_pbuf_default;;
		m->blk_conf[block].pkt_mem->virt_to_phys = virt_to_phys_default;
		m->blk_conf[block].pkt_mem->phys_to_virt = phys_to_virt; 
	}

	m->blk_conf[block].max_frin_desc = block_data->max_frin_desc;
	m->blk_conf[block].max_tx_desc   = block_data->max_tx_desc;
	m->blk_conf[block].tx_desc_size =  block_data->tx_desc_size;
	m->blk_conf[block].pde_class0_mask = block_data->pde_class0_mask;
	m->blk_conf[block].pde_class1_mask = block_data->pde_class1_mask;
	m->blk_conf[block].pde_class2_mask = block_data->pde_class2_mask;
	m->blk_conf[block].pde_class3_mask = block_data->pde_class3_mask;
	
	for(i =0 ; i< NLM_MAX_DEVICE_BUCKET; i++ )
		m->blk_conf[block].bucket_size[i] = block_data->bucket_size[i];

	for(i =0; i < NLM_MAX_CREDIT_COUNTER; i++)
		m->blk_conf[block].credit_counter[i] = block_data->credit_counter[i];

	m->block_init[block] = 1;
	spin_unlock(&m->block_init_lock[block]);

	return 0;
}

int nlm_net_device_init( device_conf * current_dev, int rgmii )
{
	int stat = 0;

	if ( current_dev->type == TYPE_NLM_GMAC ) {
		stat = nlm_gmac_init( current_dev, rgmii ); 
	}
	else if ( current_dev->type == TYPE_NLM_XGS ) {
		if ( xlr_board_atx_ii())
			stat = nlm_xgmac_init( current_dev );
		else if ( xlr_board_atx_i())
			stat = nlm_spi4_init( current_dev );
		else 
			stat = ERR_DEVICE_NOT_SUPORTTED;
	}
	else
		return ERR_DEVICE_NOT_SUPORTTED;

	return stat;
		
}

int nlm_net_device_open( device_conf * current_dev, int port )
{

	int stat =0;

	if ( current_dev->type == TYPE_NLM_GMAC )
		stat = nlm_gmac_open( current_dev);
	else if ( current_dev->type == TYPE_NLM_XGS ) {
		if ( xlr_board_atx_ii())	
			stat = nlm_xgmac_open( current_dev );
		else if ( xlr_board_atx_i())
			stat = nlm_spi4_port_open( current_dev,port);
		else
			return ERR_DEVICE_NOT_SUPORTTED;
	}
	else
		return ERR_DEVICE_NOT_SUPORTTED;
	return stat;
}

int nlm_net_device_close( device_conf * current_dev, int port )
{

	int stat =0;

	if ( current_dev->type == TYPE_NLM_GMAC )
		stat = nlm_gmac_close( current_dev);
	else if ( current_dev->type == TYPE_NLM_XGS ) {
		if ( xlr_board_atx_ii())	
			stat = nlm_xgmac_close( current_dev );
		else if ( xlr_board_atx_i())
			stat = nlm_spi4_port_close( current_dev,port);
		else
			return ERR_DEVICE_NOT_SUPORTTED;
	}
	else
		return ERR_DEVICE_NOT_SUPORTTED;
	return stat;
}


int clr_enable_device_stat(device_conf * current_dev)
{
	uint32_t stat = phoenix_read_reg( current_dev->mmio,STAT_CTRL);

	stat =  stat & ~0x6;
	
	
	phoenix_write_reg( current_dev->mmio,STAT_CTRL,(stat | FIELD_VALUE(CLEAR_COUNTERS,1)));
	phoenix_write_reg( current_dev->mmio,STAT_CTRL, (stat | FIELD_VALUE(STATS_ENABLE,1)));

	return 0;
}

mac_stat * read_device_stat( device_conf * current_dev)
{

	mac_stat * stat =  (mac_stat *)contig_alloc(sizeof(mac_stat));

	stat->tx_rx_64b = phoenix_read_reg( current_dev->mmio,G_TR_64);
	stat->tx_rx_65b_127b = phoenix_read_reg(current_dev->mmio, G_TR_127);
	stat->tx_rx_127b_255b = phoenix_read_reg(current_dev->mmio, G_TR_255);
	stat->tx_rx_255b_511b = phoenix_read_reg(current_dev->mmio,G_TR_511);
	stat->tx_rx_511b_1023b = phoenix_read_reg(current_dev->mmio, G_TR_1k);
	stat->tx_rx_1024b__1518 = phoenix_read_reg(current_dev->mmio,G_TR_MAX);
	stat->tx_rx_1519b_1522b = phoenix_read_reg(current_dev->mmio,G_TX_MGV);
	
	stat->rx_byte = phoenix_read_reg( current_dev->mmio,G_RBYT);
	stat->rx_pkt = phoenix_read_reg( current_dev->mmio,G_RPKT);
	stat->rx_fcs_err = phoenix_read_reg( current_dev->mmio,G_RFCS);
	stat->rx_multicst = phoenix_read_reg( current_dev->mmio,G_RMCA);
	stat->rx_brdcst = phoenix_read_reg( current_dev->mmio,G_RBCA);
	stat->rx_ctrl = phoenix_read_reg( current_dev->mmio,G_RXCF);
	stat->rx_pause = phoenix_read_reg( current_dev->mmio,G_RXPF);
	stat->rx_unkwn_OPCode = phoenix_read_reg( current_dev->mmio,G_RXUO);
	stat->rx_align_err = phoenix_read_reg( current_dev->mmio,G_RALN);
	stat->rx_frm_len_err = phoenix_read_reg( current_dev->mmio,G_RFLR);
	stat->rx_code_err = phoenix_read_reg( current_dev->mmio,G_RCDC);
	stat->rx_carrier_sense_err = phoenix_read_reg( current_dev->mmio,G_RCSE);
	stat->rx_undersize_pkt_err = phoenix_read_reg( current_dev->mmio,G_RUND);
	stat->rx_oversize_pkt_err = phoenix_read_reg( current_dev->mmio,G_OVR);

	stat->rx_fragments = phoenix_read_reg( current_dev->mmio,G_RFRG);
	stat->rx_jabber = phoenix_read_reg( current_dev->mmio,G_RJBR);
	if (!is_xls())
		stat->rx_dropped = phoenix_read_reg( current_dev->mmio,G_RDRP);

	stat->tx_byte = phoenix_read_reg( current_dev->mmio,G_TBYT);
	stat->tx_pkt = phoenix_read_reg( current_dev->mmio,G_TPKT);
	stat->tx_multicst = phoenix_read_reg( current_dev->mmio,G_TMCA);
	stat->tx_brdcast = phoenix_read_reg( current_dev->mmio,G_TBCA);
	stat->tx_pause_ctrl = phoenix_read_reg( current_dev->mmio,G_TXPF);
	stat->tx_deferral = phoenix_read_reg( current_dev->mmio,G_TDFR);
	stat->tx_excessive = phoenix_read_reg( current_dev->mmio,G_TEDF);
	stat->tx_signal_collision = phoenix_read_reg( current_dev->mmio,G_TSCL) ;
	stat->tx_multiple = phoenix_read_reg( current_dev->mmio,G_TMCL) ;
	stat->tx_late_colliaion = phoenix_read_reg( current_dev->mmio,G_TLCL);
	stat->tx_excessive_collision = phoenix_read_reg( current_dev->mmio,G_TXCL) ;
	stat->tx_total_collision = phoenix_read_reg( current_dev->mmio,G_TNCL);
	if ( is_xls()) {
		stat->tx_pause_fm_honored = phoenix_read_reg( current_dev->mmio,G_TPFH);
		stat->tx_drop = phoenix_read_reg( current_dev->mmio,G_TDRP);
	}
	stat->tx_jabber_fmt = phoenix_read_reg( current_dev->mmio,G_TJBR);
	stat->tx_fcs_err = phoenix_read_reg( current_dev->mmio,G_TFCS);
	stat->tx_ctrl = phoenix_read_reg( current_dev->mmio,G_TXCS);
	stat->tx_oversize = phoenix_read_reg( current_dev->mmio,G_TOVR);
	stat->tx_undersize = phoenix_read_reg( current_dev->mmio,G_TUND);
	stat->tx_fragment = phoenix_read_reg( current_dev->mmio,G_TRFG);
	stat->tx_carry_reg_1 = phoenix_read_reg( current_dev->mmio,CAR1);
	stat->tx_carry_reg_2 = phoenix_read_reg( current_dev->mmio,CAR2);
	stat->tx_carry_mask_reg_1 = phoenix_read_reg( current_dev->mmio,CAM1);
	stat->tx_carry_mask_reg_2 = phoenix_read_reg( current_dev->mmio,CAM2);

	return stat;


}
static void nlm_setup_pde_mask( unsigned int *mmio, int class_id,
	uint32_t pde_cpu_mask_0, uint32_t pde_cpu_mask_1)
{
	int class_offset = class_id * 2;
	printf ("\nPDE_0 class_%d packet spray mask %x\n", class_id,pde_cpu_mask_0);

	// program the pde
	phoenix_write_reg(mmio,PDE_CLASS_0 + class_offset ,  pde_cpu_mask_0);

	printf ("PDE_1 class_%d packet spray mask %x\n", class_id,pde_cpu_mask_1);
	// program the pde
	phoenix_write_reg(mmio,PDE_CLASS_0+ class_offset+1,pde_cpu_mask_1);

}


void nlm_setup_pde(device_conf * current_dev )
{
	port_info * internal = ( port_info *)current_dev->intrl;
	int block = internal->block_id; 
	struct netblk_shm_vars *m;

	if((m = nlm_netblk_shm_init()) == NULL)
		return;

	nlm_setup_pde_mask((unsigned int *)current_dev->mmio,0,
			m->blk_conf[block].pde_class0_mask & 0xffffffff, (m->blk_conf[block].pde_class0_mask >> 32) & 0xffffffff);
	nlm_setup_pde_mask((unsigned int *)current_dev->mmio,1,
			m->blk_conf[block].pde_class1_mask & 0xffffffff, (m->blk_conf[block].pde_class1_mask >> 32) & 0xffffffff);
	nlm_setup_pde_mask((unsigned int *)current_dev->mmio,2,
			 m->blk_conf[block].pde_class2_mask & 0xffffffff, (m->blk_conf[block].pde_class2_mask >> 32) & 0xffffffff);
	nlm_setup_pde_mask((unsigned int *)current_dev->mmio,3,
			m->blk_conf[block].pde_class3_mask & 0xffffffff, (m->blk_conf[block].pde_class3_mask >> 32) & 0xffffffff);
	
}

void print_device_err_msg( int i )
{
	printf("%s\n",device_error_string[i]);
}
	




	

	
