/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

// netconfig.c
// top level configuration file for XLR
// networking drivers
#include <stdio.h>
#include <stdlib.h>

#include "string.h"
#include "stdlib.h"
#include "net_config.h"
#include "iomap.h"
#include "mac.h"
#include "gmac.h"
#include "psb.h"
#include "spill.h"
#include "compiler.h"
#include "asmmacros.h"
#include "xgmac.h"
#include "spi4.h"
#include "vitesse_highlevel.h"
#include "xlr_boot_lib.h"
#include "system.h"
#include "device.h"

#include "msgring.h"
#include "contig_alloc.h"
#include "vitesse_io.h"
#include "shmvars.h"

#define DEBUG 1 

// global locks & done flags
extern int is_valid_gmac(int gmac);

device_conf * net_device_config[10] = { [0 ... 9] = 0 };


extern uint64_t virt_to_phys_default( void * vaddr);
extern volatile struct xlr_vcpu_wakeup_info *xlr_vcpu_wakeup_info;
extern void serdes_autoconfig( int gmac );
extern struct block_config  blk_conf[4];
extern void nlm_spill_setup(struct net_config_struct *net_config);
// net_config net_conf;

struct packet *(*net_conf_alloc_pbuf)(void) = alloc_pbuf;
void (*net_conf_free_pbuf)(struct packet *) = free_pbuf;

void (*net_conf_spill_setup)(struct net_config_struct *) = nlm_spill_setup;
extern void nlm_xgmac_soft_reset( phoenix_reg_t * mmio);
extern void nlm_spi4_soft_reset( phoenix_reg_t * mmio);
extern void nlm_gmac_soft_reset(int id);

extern int sgmii_dc_present(int block);

net_config net_conf_gmac  = {
    .gmac0 = 1,
    .gmac1 = 1,
    .gmac2 = 1,
    .gmac3 = 1,
    .gmac4 = 1,
    .gmac5 = 1,
    .gmac6 = 1,
    .gmac7 = 1,

    .gmac0_frin_spill = 0,
    .gmac0_frin_spill_entries = MAX_FRIN_SPILL_ENTRIES,
    .gmac0_frout_spill = 0,
    .gmac0_frout_spill_entries = MAX_FROUT_SPILL_ENTRIES,
    .gmac0_rx_spill0 = 0,
    .gmac0_rx_spill0_entries = MAX_RX_SPILL_ENTRIES,
    .gmac0_rx_spill1 = 0,
    .gmac0_rx_spill1_entries = MAX_RX_SPILL_ENTRIES,
    .gmac0_rx_spill2 = 0,
    .gmac0_rx_spill2_entries = MAX_RX_SPILL_ENTRIES,
    .gmac0_rx_spill3 = 0,
    .gmac0_rx_spill3_entries = MAX_RX_SPILL_ENTRIES,

    .gmac1_frin_spill = 0,
    .gmac1_frin_spill_entries = MAX_FRIN_SPILL_ENTRIES,
    .gmac1_frout_spill = 0,
    .gmac1_frout_spill_entries = MAX_FROUT_SPILL_ENTRIES,
    .gmac1_rx_spill0 = 0,
    .gmac1_rx_spill0_entries = MAX_RX_SPILL_ENTRIES,
    .gmac1_rx_spill1 = 0,
    .gmac1_rx_spill1_entries = MAX_RX_SPILL_ENTRIES,
    .gmac1_rx_spill2 = 0,
    .gmac1_rx_spill2_entries = MAX_RX_SPILL_ENTRIES,
    .gmac1_rx_spill3 = 0,
    .gmac1_rx_spill3_entries = MAX_RX_SPILL_ENTRIES,

    .xgs0_frin_spill = 1,
    .xgs0_frin_spill_entries = MAX_FRIN_SPILL_ENTRIES,
    .xgs0_frout_spill = 1,
    .xgs0_frout_spill_entries = MAX_FROUT_SPILL_ENTRIES,
    .xgs0_rx_spill0 = 1,
    .xgs0_rx_spill0_entries = MAX_RX_SPILL_ENTRIES,
    .xgs0_rx_spill1 = 1,
    .xgs0_rx_spill1_entries = MAX_RX_SPILL_ENTRIES,
    .xgs0_rx_spill2 = 1,
    .xgs0_rx_spill2_entries = MAX_RX_SPILL_ENTRIES,
    .xgs0_rx_spill3 = 1,
    .xgs0_rx_spill3_entries = MAX_RX_SPILL_ENTRIES,

    .xgs1_frin_spill = 1,
    .xgs1_frin_spill_entries = MAX_FRIN_SPILL_ENTRIES,
    .xgs1_frout_spill = 1,
    .xgs1_frout_spill_entries = MAX_FROUT_SPILL_ENTRIES,
    .xgs1_rx_spill0 = 1,
    .xgs1_rx_spill0_entries = MAX_RX_SPILL_ENTRIES,
    .xgs1_rx_spill1 = 1,
    .xgs1_rx_spill1_entries = MAX_RX_SPILL_ENTRIES,
    .xgs1_rx_spill2 = 1,
    .xgs1_rx_spill2_entries = MAX_RX_SPILL_ENTRIES,
    .xgs1_rx_spill3 = 1,
    .xgs1_rx_spill3_entries = MAX_RX_SPILL_ENTRIES,

    .fifo_mode0 =0,
    .fifo_mode1 =0,
    .fifo_mode2 =0,
    .fifo_mode3 =0,
   
    .rgmii_mode = 0,
    .split_mode = 0,

    .gmac_rx_enable = 1,
    .gmac_free_descriptors_0 = 128,
    .gmac_free_descriptors_1 = 0,
    .gmac0_pde_mask_0 = 0x0,
    .gmac0_pde_mask_1 = 0x0,
    .gmac1_free_descriptors_0 = 128,
    .gmac1_free_descriptors_1 = 0,
    .gmac1_pde_mask_0 = 0x0,
    .gmac1_pde_mask_1 = 0x0,
    
    .spi0 = 0,
    .xgmac0 = 1,
    .xgs0_free_descriptors_0 = 128,
    
    .spi1 = 0,
    .xgmac1 = 1,
    .xgs1_free_descriptors_1 = 128,
    
    .abm = 1,
    .gmac0_auto_negotiation = 1,
    .gmac1_auto_negotiation = 1,
    .gmac2_auto_negotiation = 1,
    .gmac3_auto_negotiation = 1,
    .gmac4_auto_negotiation = 1,
    .gmac5_auto_negotiation = 1,
    .gmac6_auto_negotiation = 1,
    .gmac7_auto_negotiation = 1,
    .default_gmac_speed = MAC_SPEED_1000,
    .default_gmac_duplexity = 1,
    .regular_buffer_size = MAC_FRAME_SIZE,
    .gmac_frame_length =  { [0 ... 7] = MAC_FRAME_SIZE},
    .xgs_tx_frame_length = { MAC_FRAME_SIZE,MAC_FRAME_SIZE},
    .xgs_rx_frame_length = { MAC_FRAME_SIZE,MAC_FRAME_SIZE},
    .error_id = -1,

};

net_config net_conf_xgmac = {
    .gmac0 = 0,
    .gmac1 = 0,
    .gmac2 = 0,
    .gmac3 = 0,
    
    .gmac0_frin_spill = 0,
    .gmac0_frin_spill_entries = 0,
    .gmac0_frout_spill = 0,
    .gmac0_frout_spill_entries = 0,
    .gmac0_rx_spill0 = 0,
    .gmac0_rx_spill0_entries = 0,
    .gmac0_rx_spill1 = 0,
    .gmac0_rx_spill1_entries = 0,
    .gmac0_rx_spill2 = 0,
    .gmac0_rx_spill2_entries = 0,
    .gmac0_rx_spill3 = 0,
    .gmac0_rx_spill3_entries = 0,

    .gmac1_frin_spill = 0,
    .gmac1_frin_spill_entries = 0,
    .gmac1_frout_spill = 0,
    .gmac1_frout_spill_entries = 0,
    .gmac1_rx_spill0 = 0,
    .gmac1_rx_spill0_entries = 0,
    .gmac1_rx_spill1 = 0,
    .gmac1_rx_spill1_entries = 0,
    .gmac1_rx_spill2 = 0,
    .gmac1_rx_spill2_entries = 0,
    .gmac1_rx_spill3 = 0,
    .gmac1_rx_spill3_entries = 0,

    .xgs0_frin_spill = 1,
    .xgs0_frin_spill_entries = MAX_FRIN_SPILL_ENTRIES,
    .xgs0_frout_spill = 1,
    .xgs0_frout_spill_entries = MAX_FROUT_SPILL_ENTRIES,
    .xgs0_rx_spill0 = 1,
    .xgs0_rx_spill0_entries = MAX_RX_SPILL_ENTRIES,
    .xgs0_rx_spill1 = 1,
    .xgs0_rx_spill1_entries = MAX_RX_SPILL_ENTRIES,
    .xgs0_rx_spill2 = 1,
    .xgs0_rx_spill2_entries = MAX_RX_SPILL_ENTRIES,
    .xgs0_rx_spill3 = 1,
    .xgs0_rx_spill3_entries = MAX_RX_SPILL_ENTRIES,

    .xgs1_frin_spill = 1,
    .xgs1_frin_spill_entries = MAX_FRIN_SPILL_ENTRIES,
    .xgs1_frout_spill = 1,
    .xgs1_frout_spill_entries = MAX_FROUT_SPILL_ENTRIES,
    .xgs1_rx_spill0 = 1,
    .xgs1_rx_spill0_entries = MAX_RX_SPILL_ENTRIES,
    .xgs1_rx_spill1 = 1,
    .xgs1_rx_spill1_entries = MAX_RX_SPILL_ENTRIES,
    .xgs1_rx_spill2 = 1,
    .xgs1_rx_spill2_entries = MAX_RX_SPILL_ENTRIES,
    .xgs1_rx_spill3 = 1,
    .xgs1_rx_spill3_entries = MAX_RX_SPILL_ENTRIES,

    .split_mode = 0,
    .gmac_rx_enable = 0,
    .gmac_free_descriptors_0 = 0,
    .gmac_free_descriptors_1 = 0,
    .gmac1_free_descriptors_0 = 0,
    .gmac1_free_descriptors_1 = 0,
    
    .spi0 = 0,
    .xgmac0 = 1,
    .xgs0_free_descriptors_0 = 128,
    .xgs0_pde_mask_0 = 0x0,
    .xgs0_pde_mask_1 = 0x0,
    
    .spi1 = 0,
    .xgmac1 = 1,
    .xgs1_free_descriptors_1 = 128,
    .xgs1_pde_mask_0 = 0x0,
    .xgs1_pde_mask_1 = 0x0,
    
    .abm = 1,
    .gmac0_auto_negotiation = 0,
    .gmac1_auto_negotiation = 0,
    .gmac2_auto_negotiation = 0,
    .gmac3_auto_negotiation = 0,
    .gmac4_auto_negotiation = 0,
    .gmac5_auto_negotiation = 0,
    .gmac6_auto_negotiation = 0,
    .gmac7_auto_negotiation = 0,
    .default_gmac_speed = MAC_SPEED_100,
    .default_gmac_duplexity = 1,
    .regular_buffer_size = MAC_FRAME_SIZE,
    .gmac_frame_length =  { [0 ... 7] = MAC_FRAME_SIZE},
    .xgs_tx_frame_length = { MAC_FRAME_SIZE,MAC_FRAME_SIZE},
    .xgs_rx_frame_length = { MAC_FRAME_SIZE,MAC_FRAME_SIZE},
    .error_id = -1,
};

net_config net_conf_spi4 = {
    .gmac0 = 0,
    .gmac1 = 0,
    .gmac2 = 0,
    .gmac3 = 0,

    .gmac0_frin_spill = 0,
    .gmac0_frin_spill_entries = 0,
    .gmac0_frout_spill = 0,
    .gmac0_frout_spill_entries = 0,
    .gmac0_rx_spill0 = 0,
    .gmac0_rx_spill0_entries = 0,
    .gmac0_rx_spill1 = 0,
    .gmac0_rx_spill1_entries = 0,
    .gmac0_rx_spill2 = 0,
    .gmac0_rx_spill2_entries = 0,
    .gmac0_rx_spill3 = 0,
    .gmac0_rx_spill3_entries = 0,

    .gmac1_frin_spill = 0,
    .gmac1_frin_spill_entries = 0,
    .gmac1_frout_spill = 0,
    .gmac1_frout_spill_entries = 0,
    .gmac1_rx_spill0 = 0,
    .gmac1_rx_spill0_entries = 0,
    .gmac1_rx_spill1 = 0,
    .gmac1_rx_spill1_entries = 0,
    .gmac1_rx_spill2 = 0,
    .gmac1_rx_spill2_entries = 0,
    .gmac1_rx_spill3 = 0,
    .gmac1_rx_spill3_entries = 0,

    .xgs0_frin_spill = 1,
    .xgs0_frin_spill_entries = MAX_FRIN_SPILL_ENTRIES,
    .xgs0_frout_spill = 1,
    .xgs0_frout_spill_entries = MAX_FROUT_SPILL_ENTRIES,
    .xgs0_rx_spill0 = 1,
    .xgs0_rx_spill0_entries = MAX_RX_SPILL_ENTRIES,
    .xgs0_rx_spill1 = 1,
    .xgs0_rx_spill1_entries = MAX_RX_SPILL_ENTRIES,
    .xgs0_rx_spill2 = 1,
    .xgs0_rx_spill2_entries = MAX_RX_SPILL_ENTRIES,
    .xgs0_rx_spill3 = 1,
    .xgs0_rx_spill3_entries = MAX_RX_SPILL_ENTRIES,

    .xgs1_frin_spill = 1,
    .xgs1_frin_spill_entries = MAX_FRIN_SPILL_ENTRIES,
    .xgs1_frout_spill = 1,
    .xgs1_frout_spill_entries = MAX_FROUT_SPILL_ENTRIES,
    .xgs1_rx_spill0 = 1,
    .xgs1_rx_spill0_entries = MAX_RX_SPILL_ENTRIES,
    .xgs1_rx_spill1 = 1,
    .xgs1_rx_spill1_entries = MAX_RX_SPILL_ENTRIES,
    .xgs1_rx_spill2 = 1,
    .xgs1_rx_spill2_entries = MAX_RX_SPILL_ENTRIES,
    .xgs1_rx_spill3 = 1,
    .xgs1_rx_spill3_entries = MAX_RX_SPILL_ENTRIES,

    .split_mode = 0,
    .gmac_rx_enable = 0,
    .gmac_free_descriptors_0 = 0,
    .gmac_free_descriptors_1 = 0,
    .gmac1_free_descriptors_0 = 0,
    .gmac1_free_descriptors_1 = 0,
    
    .spi0 = 1,
    .xgmac0 = 0,
    .xgs0_free_descriptors_0 = 128,
    .xgs0_pde_mask_0 = 0x0,
    .xgs0_pde_mask_1 = 0x0,
    
    .spi1 = 1,
    .xgmac1 = 0,
    .xgs1_free_descriptors_1 = 128,
    .xgs1_pde_mask_0 = 0x0,
    .xgs1_pde_mask_1 = 0x0,
    
    .abm = 1,
    .gmac0_auto_negotiation = 0,
    .gmac1_auto_negotiation = 0,
    .gmac2_auto_negotiation = 0,
    .gmac3_auto_negotiation = 0,
    .gmac4_auto_negotiation = 0,
    .gmac5_auto_negotiation = 0,
    .gmac6_auto_negotiation = 0,
    .gmac7_auto_negotiation = 0,
    .default_gmac_speed = MAC_SPEED_100,
    .default_gmac_duplexity = 1,
    .spi4_0_auto_negotiation = 0x3ff,
    .spi4_1_auto_negotiation = 0x3ff,
    .default_spi4_speed = MAC_SPEED_1000,
    .default_spi4_duplexity = 1,
    .regular_buffer_size = MAC_FRAME_SIZE,
    .gmac_frame_length =  { [0 ... 7] = MAC_FRAME_SIZE},
    .xgs_tx_frame_length = { MAC_FRAME_SIZE,MAC_FRAME_SIZE},
    .xgs_rx_frame_length = { MAC_FRAME_SIZE,MAC_FRAME_SIZE},
    .error_id = -1,
};

struct phnx_mac phnx_mac_devices[] = {
	{ .instance = GMAC_0, .type = GMAC, .name = "gmac0",
	  .io_offset = PHOENIX_IO_GMAC_0_OFFSET
	},
	{ .instance = GMAC_1, .type = GMAC, .name = "gmac1",
	  .io_offset = PHOENIX_IO_GMAC_1_OFFSET
	},
	{ .instance = GMAC_2, .type = GMAC, .name = "gmac2",
	  .io_offset = PHOENIX_IO_GMAC_2_OFFSET
	},
	{ .instance = GMAC_3, .type = GMAC, .name = "gmac3",
	  .io_offset = PHOENIX_IO_GMAC_3_OFFSET
	},
        { .instance = GMAC_4, .type = GMAC, .name = "gmac4",
          .io_offset = PHOENIX_IO_GMAC_4_OFFSET
        },
        { .instance = GMAC_5, .type = GMAC, .name = "gmac5",
          .io_offset = PHOENIX_IO_GMAC_5_OFFSET
        },
        { .instance = GMAC_6, .type = GMAC, .name = "gmac6",
          .io_offset = PHOENIX_IO_GMAC_6_OFFSET
        },
        { .instance = GMAC_7, .type = GMAC, .name = "gmac7",
          .io_offset = PHOENIX_IO_GMAC_7_OFFSET
        },
	{ .instance = XGMAC_0, .type = XGMAC, .name = "xgmac0",
	  .io_offset = PHOENIX_IO_XGMAC_0_OFFSET
	},
	{ .instance = XGMAC_1, .type = XGMAC, .name = "xgmac1",
	  .io_offset = PHOENIX_IO_XGMAC_1_OFFSET
	},
	{ .instance = SPI4_0, .type = SPI4, .name = "spi4_0",
	  .io_offset = PHOENIX_IO_SPI4_0_OFFSET
	},
	{ .instance = SPI4_1, .type = SPI4, .name = "spi4_1",
	  .io_offset = PHOENIX_IO_SPI4_1_OFFSET
	}
};

int xlsb0_in_xaui(int block)
{
	unsigned int gpio_xaui = 0;
	phoenix_reg_t *gpio_mmio =
		(unsigned int *)(phoenix_io_base + PHOENIX_IO_GPIO_OFFSET);
	if (is_xls_board_xi_xii()) {
		gpio_xaui = ((phoenix_read_reg(gpio_mmio,21) >> 24) & 0x3);
		switch(gpio_xaui){
			case 0x1:
				return block==0?1:0;
			case 0x2:
				return block==1?1:0;
			case 0x3:
				return 1;
			default:
				return 0;
		}
	}
	return 0;
}
	
extern int is_valid_gmac(int gmac);
static int is_netconf_autoneg_enable ( int gmac, net_config *net_conf )
{
	int value = 0;
	switch(gmac)
	{
		case 0:
			value = net_conf->gmac0_auto_negotiation;
			break;
		case 1:
			value = net_conf->gmac1_auto_negotiation;;
			break;
		case 2:
			value = net_conf->gmac2_auto_negotiation;
			break;
		case 3:
			value = net_conf->gmac3_auto_negotiation;
			break;
		case 4:
			value = net_conf->gmac4_auto_negotiation;
			break;
		case 5:
			value = net_conf->gmac5_auto_negotiation;
			break;
		case 6:
			value = net_conf->gmac6_auto_negotiation;
			break;
		case 7:
			value = net_conf->gmac7_auto_negotiation;
			break;
	}
	return value;
}
static  int is_netconf_gmac_enable ( int gmac, net_config *net_conf )
{
	int value = 0;

	switch(gmac)
	{
		case 0:
			value = net_conf->gmac0;
			break;
		case 1:
			value = net_conf->gmac1;
			break;
		case 2:
			value = net_conf->gmac2;
			break;
		case 3:
			value = net_conf->gmac3;
			break;
		case 4:
			value = net_conf->gmac4;
			break;
		case 5:
			value = net_conf->gmac5;
			break;
		case 6:
			value = net_conf->gmac6;
			break;
		case 7:
			value = net_conf->gmac7;
			break;
	}


	return value;
}
static int is_netconf_xgs_enabled( int xgmac, net_config * net_conf)
{
	uint32_t val = 0; 

	switch(xgmac) 	{
		case 0:
			if ( xlr_board_atx_ii())
				val = net_conf->xgmac0;
			if ( xlr_board_atx_i())
				val = net_conf->spi0;
			break;
		case 1:
			if ( xlr_board_atx_ii())
				val = net_conf->xgmac1;
			if ( xlr_board_atx_i())
				val = net_conf->spi1;
			break;
	}

	return val;
				

}

static uint64_t device_pde_setup( unsigned int cpu_mask)
{
	unsigned int pde_cpu_mask_0 =0;
	unsigned int pde_cpu_mask_1 =0;
	unsigned int tmp_mask[8];
	unsigned int cpu_mask_t;
	uint64_t pde_mask = 0ULL;
	int i =0;
	for (i=0;i<4;i++) {
		tmp_mask[i] = (((cpu_mask >> (i*4)) & 0x0f) << ((i*8)+net_conf_bucket_offset));
		pde_cpu_mask_0 = pde_cpu_mask_0 | tmp_mask[i];
	}
	cpu_mask_t = cpu_mask >> 16;
	for (i=0;i<4;i++) {
		tmp_mask[i] = ((cpu_mask_t >> (i*4)) & 0x0f) << ((i*8)+net_conf_bucket_offset);
		pde_cpu_mask_1 = pde_cpu_mask_1 | tmp_mask[i];
	}

	pde_mask = (uint64_t)pde_cpu_mask_1 << 32 | pde_cpu_mask_0;	

	return pde_mask;

}
static struct block_config *  nlm_fillin_xgs_params( net_config * net_conf, device_conf * current_dev)
{
	int i =0;
	struct block_config * blk = ( struct block_config *)contig_alloc(sizeof(struct block_config));

	if(blk == NULL)
		return NULL;
	blk->pkt_mem = (packet_mem_rt *)contig_alloc(sizeof(packet_mem_rt));
	if(blk->pkt_mem == NULL) {
		contig_free(blk);
		return NULL;
	}
	blk->pkt_mem->alloc_rx_buf = (alloc_rx_buf_func_t)net_conf_alloc_pbuf;
	blk->pkt_mem->free_rx_buf = (free_rx_buf_func_t)net_conf_free_pbuf;;
	blk->pkt_mem->virt_to_phys = virt_to_phys_default;
	blk->pkt_mem->phys_to_virt = phys_to_virt; 

	if(net_conf->xgs_tx_frame_length[current_dev->id] == 0)
		net_conf->xgs_tx_frame_length[current_dev->id] = MAC_FRAME_SIZE;
	current_dev->max_tx_frame_len = net_conf->xgs_tx_frame_length[current_dev->id]; 

	if(net_conf->xgs_rx_frame_length[current_dev->id] == 0)
		net_conf->xgs_rx_frame_length[current_dev->id] = MAC_FRAME_SIZE;

	current_dev->max_rx_frame_len = net_conf->xgs_rx_frame_length[current_dev->id];

	if(net_conf->regular_buffer_size == 0)
		net_conf->regular_buffer_size = MAC_FRAME_SIZE;

	current_dev->rx_regular_buf_size = net_conf->regular_buffer_size; 
	current_dev->l2_alloc_rx = 0xff;
	current_dev->l2_alloc_tx = 0xff;
	current_dev->enable_prepad = 0;
	current_dev->enable_stat = 0;
	current_dev->phy_conf[0].auto_neg = is_netconf_autoneg_enable(current_dev->id , net_conf);
	current_dev->phy_conf[0].speed = net_conf->default_gmac_speed;
	current_dev->phy_conf[0].duplex = net_conf->default_gmac_duplexity;
	blk->tx_desc_size = 1;
	current_dev->byte_offset[0] =  net_conf->gmac_byte_offset[current_dev->id];
	current_dev->error_mask = net_conf_error_status;
	current_dev->error_id = net_conf->error_id;

	for(i =0; i < NLM_MAX_SPI4_PORT_PER_BLOCK; i++ ) {
		current_dev->phy_conf[i].speed = net_conf->default_spi4_speed;
		current_dev->phy_conf[i].duplex = net_conf->default_spi4_duplexity;
	}

	if ( current_dev->id == 0) {
		for(i =0 ;i < NLM_MAX_SPI4_PORT_PER_BLOCK ;i++) {
			if ( (net_conf->spi4_0_auto_negotiation & ( 1 << i)) != 0)
				current_dev->phy_conf[i].auto_neg = 1;
			else
				current_dev->phy_conf[i].auto_neg = 0;
		}
		blk->max_frin_desc = net_conf->xgs0_free_descriptors_0 ;
		blk->max_tx_desc  = net_conf->xgs0_free_descriptors_0 ;
		blk->pde_class0_mask = net_conf->xgs0_pde_mask_0 | 
				(uint64_t)net_conf->xgs0_pde_mask_1 << 32ULL ;
		if ( blk->pde_class0_mask == 0ULL && xlr_vcpu_wakeup_info != NULL) { 
			uint32_t cpu_mask  = (xlr_vcpu_wakeup_info+processor_id())->psb_os_cpu_map; 
			blk->pde_class0_mask = device_pde_setup( cpu_mask); 
		} 

		blk->pde_class1_mask = 0ULL;
		blk->pde_class2_mask = 0ULL;
		blk->pde_class3_mask = 0ULL;

		for ( i = 0; i < 16; i++)
			blk->bucket_size[i] = bucket_sizes.bucket[MSGRNG_STNID_XMAC0_00_TX +i];
		blk->bucket_size[16] = bucket_sizes.bucket[MSGRNG_STNID_XMAC0JFR];
		blk->bucket_size[17] = bucket_sizes.bucket[MSGRNG_STNID_XMAC0RFR];

		for(i =0; i < NLM_MAX_CREDIT_COUNTER; i++ )
			blk->credit_counter[i] = cc_table_xgs_0.counters[i>>3][i&0x07];

		for(i =0; i < NLM_MAX_SPI4_PORT_PER_BLOCK ; i++)
			current_dev->byte_offset[i] =  net_conf->xgs0_byte_offset[i];

		current_dev->xgs_type = net_conf->spi0 == 1 ? SPI4 : XGMAC;
	}
	else {
		for(i =0 ;i < NLM_MAX_SPI4_PORT_PER_BLOCK;i++) {
			if ( (net_conf->spi4_1_auto_negotiation & ( 1 << i)) != 0)
				current_dev->phy_conf[i].auto_neg = 1;
			else
				current_dev->phy_conf[i].auto_neg = 0;
		}
		blk->max_frin_desc = net_conf->xgs1_free_descriptors_1 ;
		blk->max_tx_desc = net_conf->xgs1_free_descriptors_1 ;
		blk->pde_class0_mask = net_conf->xgs1_pde_mask_0 |
			(uint64_t)net_conf->xgs1_pde_mask_1 << 32ULL;;
		if ( blk->pde_class0_mask == 0ULL && xlr_vcpu_wakeup_info != NULL) { 
			uint32_t cpu_mask  = (xlr_vcpu_wakeup_info+processor_id())->psb_os_cpu_map; 
			blk->pde_class0_mask = device_pde_setup( cpu_mask); 
		} 
		blk->pde_class1_mask = 0ULL;
		blk->pde_class2_mask = 0ULL;
		blk->pde_class3_mask = 0ULL;

		for ( i = 0; i < 16; i++)
			blk->bucket_size[i] = bucket_sizes.bucket[MSGRNG_STNID_XMAC1_00_TX +i];
		blk->bucket_size[16] = bucket_sizes.bucket[MSGRNG_STNID_XMAC1JFR];
		blk->bucket_size[17] = bucket_sizes.bucket[MSGRNG_STNID_XMAC1RFR];
		for(i =0; i < NLM_MAX_CREDIT_COUNTER; i++ )
			blk->credit_counter[i] = cc_table_xgs_1.counters[i>>3][i&0x07];

		for(i =0; i < NLM_MAX_SPI4_PORT_PER_BLOCK ; i++)
			current_dev->byte_offset[i] =  net_conf->xgs1_byte_offset[i];

		current_dev->xgs_type = net_conf->spi1 == 1 ? SPI4 : XGMAC;
		
	}

	return blk;
}
static struct block_config * nlm_fillin_gmac_params( net_config * net_conf, device_conf * current_dev) 
{
	int i =0;
	struct bucket_size * bkt_size = NULL;
	struct stn_cc *cc = NULL;
	struct block_config * blk = ( struct block_config *)contig_alloc(sizeof(struct block_config));

	if(blk == NULL)
		return NULL;
	blk->pkt_mem = (packet_mem_rt *)contig_alloc(sizeof(packet_mem_rt));
	if(blk->pkt_mem == NULL) {
		contig_free(blk);
		return NULL;
	}
	blk->pkt_mem->alloc_rx_buf = (alloc_rx_buf_func_t)net_conf_alloc_pbuf;
	blk->pkt_mem->free_rx_buf = (free_rx_buf_func_t)net_conf_free_pbuf;;
	blk->pkt_mem->virt_to_phys = virt_to_phys_default;
	blk->pkt_mem->phys_to_virt = phys_to_virt; 


	/* set the device config with all assumptions */

	if(net_conf->gmac_frame_length[current_dev->id] == 0)
		net_conf->gmac_frame_length[current_dev->id] = MAC_FRAME_SIZE;
	current_dev->max_tx_frame_len =  net_conf->gmac_frame_length[current_dev->id];

	current_dev->max_rx_frame_len =  net_conf->gmac_frame_length[current_dev->id];
	if(net_conf->regular_buffer_size == 0)
		net_conf->regular_buffer_size = MAC_FRAME_SIZE;
	current_dev->rx_regular_buf_size = net_conf->regular_buffer_size;

	current_dev->l2_alloc_rx = 0xff;
	current_dev->l2_alloc_tx = 0xff;
	current_dev->enable_prepad = 0;
	current_dev->enable_stat = 0;
	current_dev->phy_conf[0].auto_neg = is_netconf_autoneg_enable(current_dev->id , net_conf);
	current_dev->phy_conf[0].speed = net_conf->default_gmac_speed;
	current_dev->phy_conf[0].duplex = net_conf->default_gmac_duplexity;
	blk->tx_desc_size = 1;
	current_dev->byte_offset[0] =  net_conf->gmac_byte_offset[current_dev->id];
	current_dev->error_mask = net_conf_error_status;
	current_dev->error_id = net_conf->error_id;

	if (current_dev->id < 4 ) {
		blk->max_frin_desc = net_conf->gmac_free_descriptors_0/4 ;
		blk->max_tx_desc  = net_conf->gmac_free_descriptors_0/4 ;
		blk->pde_class0_mask = net_conf->gmac0_pde_mask_0 | 
					( (uint64_t)net_conf->gmac0_pde_mask_1 << 32ULL);
		if ( blk->pde_class0_mask == 0ULL && xlr_vcpu_wakeup_info != NULL) { 
			uint32_t cpu_mask  = (xlr_vcpu_wakeup_info+processor_id())->psb_os_cpu_map; 
			blk->pde_class0_mask = device_pde_setup( cpu_mask); 
		} 
		blk->pde_class1_mask = 0ULL;
		blk->pde_class2_mask = 0ULL;
		blk->pde_class3_mask = 0ULL;
		if ( is_xls()){
			bkt_size = &xls_bucket_sizes;
			cc = &xls_cc_table_gmac0;
		}
		else {
			bkt_size = &bucket_sizes;
			cc = &cc_table_gmac;
		}
		blk->bucket_size[0] = bkt_size->bucket[MSGRNG_STNID_GMAC0];
		blk->bucket_size[1] = bkt_size->bucket[MSGRNG_STNID_GMAC0_FR];
		blk->bucket_size[2] = bkt_size->bucket[MSGRNG_STNID_GMAC0_TX0]; 
		blk->bucket_size[3] = bkt_size->bucket[MSGRNG_STNID_GMAC0_TX1];
		blk->bucket_size[4] = bkt_size->bucket[MSGRNG_STNID_GMAC0_TX2];
		blk->bucket_size[5] = bkt_size->bucket[MSGRNG_STNID_GMAC0_TX3];

		for( i =0 ; i < 128; i++)
			blk->credit_counter[i] = cc->counters[i>>3][i&0x07];

		

	} else {
		blk->max_frin_desc = net_conf->gmac1_free_descriptors_0/4  ;
		blk->max_tx_desc   = net_conf->gmac1_free_descriptors_0/4   ;
		blk->pde_class0_mask  = net_conf->gmac1_pde_mask_0 
						| (uint64_t)net_conf->gmac1_pde_mask_1 <<  32ULL;
		if ( blk->pde_class0_mask == 0ULL && xlr_vcpu_wakeup_info != NULL) { 
			uint32_t cpu_mask  = (xlr_vcpu_wakeup_info+processor_id())->psb_os_cpu_map; 
			blk->pde_class0_mask = device_pde_setup( cpu_mask); 
		} 
		blk->pde_class1_mask = 0ULL;
		blk->pde_class2_mask = 0ULL;
		blk->pde_class3_mask = 0ULL;
		blk->bucket_size[0] = xls_bucket_sizes.bucket[MSGRNG_STNID_GMAC1];
		blk->bucket_size[1] = xls_bucket_sizes.bucket[MSGRNG_STNID_GMAC1_FR];
		blk->bucket_size[2] = xls_bucket_sizes.bucket[MSGRNG_STNID_GMAC1_TX0];
		blk->bucket_size[3] = xls_bucket_sizes.bucket[MSGRNG_STNID_GMAC1_TX1];
		blk->bucket_size[4] = xls_bucket_sizes.bucket[MSGRNG_STNID_GMAC1_TX2];
		blk->bucket_size[5] = xls_bucket_sizes.bucket[MSGRNG_STNID_GMAC1_TX3];

		for( i =0 ; i < 128; i++)
			blk->credit_counter[i] = xls_cc_table_gmac1.counters[i>>3][i&0x07];
		
	}
	return blk;
			
}
// network init function

void xlr_net_init(net_config *net_conf) 
{

	uint64_t size =  MAX_PACKETS * sizeof(struct packet) ;
	uint32_t cpu_mask = 0x0;
	int j;
	int i =0;
	int index =0;
	struct block_config * blk = NULL;
	struct net_shm_vars *m;
	#define XLR_MAX_GMAC 8
	#define XLR_MAX_XGS 2
 
	if(xlr_vcpu_wakeup_info != NULL)
		cpu_mask  = (xlr_vcpu_wakeup_info+processor_id())->psb_os_cpu_map;

	if((m = nlm_net_shm_init()) == NULL)  {
		printf("Error !!! net shm init failed \n");
		return;
	}

	// ensure that init happens only once, even if all threads call this function
	spin_lock (&m->init_lock);
  		
	if (!m->init_done) {
		
		if ( (net_conf->packet_mem_size != 0) && (net_conf->packet_mem_start != 0)){
			printf("Packet Memory :  Virtual start address  %p\n",net_conf->packet_mem_start);
			printf("Packet Memory : Physical start address  %llx\n", 
			       (unsigned long long)
			       virt_to_phys((void *)net_conf->packet_mem_start));
	    		packet_init( net_conf->packet_mem_start, net_conf->packet_mem_size );
	
		} else {
			printf("Packet Memory : Virtual start Address is %llx\n",(KSEG0+ 0x1000000));
			printf("Packet Memory : Physical start Address is %llx\n", 
		    	   (unsigned long long)
					virt_to_phys((void *)(KSEG0+ 0x1000000)));
			packet_init( (char *) ( KSEG0 + 0x1000000), size );
		}

		for ( i=0; i < XLR_MAX_GMAC ; i++,index++)  {
			if( !is_valid_gmac(i)) 
				continue;
			
			if ( !is_netconf_gmac_enable( i , net_conf))
				continue;

			printf("Initializing  gmac_%d ......\n",i);

			net_device_config[index] = nlm_alloc_device(i,TYPE_NLM_GMAC);

			if ( net_device_config[index] == NULL )
				continue;

			blk = nlm_fillin_gmac_params( net_conf, net_device_config[index]);
			if ( blk ==  NULL ) {
				continue;
			}
	
			if ( nlm_net_device_block_init( net_device_config[index], blk) == INIT_NOT_NEEDED ) {
			}
			contig_free(blk);

			if ( nlm_net_device_init( net_device_config[index], net_conf->rgmii_mode) != 0 ){
				printf("gmac_%d : Initialization  Failed\n",i);
				contig_free( net_device_config[index]);
				continue;
			}

			if ( nlm_net_device_open( net_device_config[index],0 ) != 0 )
				printf("gmac_%d: Open Failed\n",i);

		}
		for( i = 0; i < XLR_MAX_XGS; i++,index++) {
			if ( !is_netconf_xgs_enabled(i, net_conf)){
				continue ;
			}

			if ( xlr_board_atx_ii())
				printf("Initializing XGMAC_%d ......\n",i);
			else if( xlr_board_atx_i())
				printf("Initializing SPI4_%d ......\n",i);
			else
				break; 

			net_device_config[index] = nlm_alloc_device(i,TYPE_NLM_XGS);

			if ( net_device_config[index] == NULL )
				continue;
			blk = nlm_fillin_xgs_params( net_conf, net_device_config[index]);

			if ( blk ==  NULL )
				continue;

			if ( nlm_net_device_block_init( net_device_config[index], blk) == INIT_NOT_NEEDED ) {
			}
			contig_free(blk);

			if ( nlm_net_device_init( net_device_config[index], 0) !=  0 ) {
				contig_free( net_device_config[index]);
				continue;
			}

			for( j =0; j < NLM_MAX_SPI4_PORT_PER_BLOCK; j++) {
				nlm_net_device_open( net_device_config[index], j);
			}
		}
	}
	m->init_done =1;
	spin_unlock(&m->init_lock);
	
}

void xlr_net_deinit( net_config *net_conf )
{
	int i =0;
	int index = 0;
	struct net_shm_vars *m;

	if((m = nlm_net_shm_init()) == NULL)  {
		printf("Error !!! net shm init failed \n");
		return;
	}


	spin_lock (&m->init_lock);
	if ( m->init_done == 1 ) {
		for( i=0; i < NLM_MAX_GMAC ; i++,index++)  {
			if( !is_valid_gmac(i))
				 continue;

			if ( !is_netconf_gmac_enable( i , net_conf))
				continue;
			printf("Resetting gmac_%d\n",i);
			nlm_gmac_soft_reset(i);
			contig_free(net_device_config[index]->intrl);
			contig_free(net_device_config[index]);
			net_device_config[index] = 0;
		}
		for( i = 0; i < NLM_MAX_XGS; i++,index++) {
			if ( !is_netconf_xgs_enabled(i, net_conf)){
				continue ;
			}
			if (  xlr_board_atx_ii()){
				printf("Resetting  XGMAC_%d \n",i);
				nlm_xgmac_soft_reset(net_device_config[index]->mmio);
				
			}
			else if ( xlr_board_atx_i()){
				printf("Resetting  SPI4_%d\n",i);
				megis_phoenix_init(net_device_config[index]->id + 1);
				nlm_spi4_soft_reset( net_device_config[index]->mmio);
			}
			else
				break;

			contig_free(net_device_config[index]->intrl);
			contig_free(net_device_config[index]);
			net_device_config[index] = 0;
		}

				
	packet_deinit();
	}
	m->init_done = 0;
	spin_unlock(&m->init_lock);
}
		
	

#define MAX_PARSE_ARGS 20
static void prepad_parse_values(uint64_t value, char *input, uint64_t result[])
{
	uint64_t bitMask[MAX_PARSE_ARGS];
	uint32_t bitStart[MAX_PARSE_ARGS];
	char *end_p = NULL;
	uint32_t i = 0x0;
	uint32_t numArgs = 0x0, pBitStart = 0x0;

	while(1) {
		bitStart[numArgs] = strtoul(input, &end_p, 10);
		numArgs++;
		if(numArgs >= MAX_PARSE_ARGS)  {
			printf("%s : Exceeded the limit\n", __FUNCTION__);
			return;
		}
		if(end_p == NULL)
			break;
		if(*end_p == ',' || *end_p == ' ') {
			input = end_p + 1;
			continue;
		} else
			break;
	}

	for(i = 0x0; i < numArgs; i++) {
		bitMask[i] = (1 << ((bitStart[i] - pBitStart) + 1)) - 1; 
		pBitStart = bitStart[i] + 1;
	}

	pBitStart = 0x0;
	for(i = 0x0; i < numArgs; i++) {
		result[i] = (value >> pBitStart) & bitMask[i];
		pBitStart = bitStart[i] + 1;
	}
}

void nlm_extract_prepad(char *buf)
{
	uint64_t value = 0x0, i = 0x0, count;
	unsigned char *temp = (unsigned char *)buf;
	uint64_t result[20];

	value = *((uint64_t *)(&temp[16])); /* 64 to 127  */
	printf("key = 0x%08x 0x%08x", (uint32_t)(value >> 32), (uint32_t)(value & 0x00000000ffffffffULL));
	value = 0x0;

	value = *((uint64_t *)(&temp[24])); /*  0 to 63 */
	printf(" 0x%08x 0x%08x\n", (uint32_t)(value >> 32), (uint32_t)(value & 0x00000000ffffffffULL));
	value = 0x0;

	value = *((uint64_t *)(&temp[8])); /* 128 to 183 only parsed here  */ 
	prepad_parse_values(value, "2,6,7,39,40,41,42,43,44,45,46,47,48,49,50,51", result);
	printf("L4   L3    32H         TCPC IPC L3h L2h L4E1 L4E0 L3E2 L3E1 L3E0 EHV L4CH L3CH\n");
	printf("0x%02x 0x%02x 0x%08x     %d   %d   %d   %d   %d    %d    %d    %d    %d    %d    %d    %d\n", 
			(uint32_t)result[0], (uint32_t)result[1], (uint32_t)result[3], 
			(uint32_t)result[4], (uint32_t)result[5], (uint32_t)result[6], (uint32_t)result[7], 
			(uint32_t)result[8], (uint32_t)result[9], 
			(uint32_t)result[10], (uint32_t)result[11], (uint32_t)result[12], (uint32_t)result[13], 
			(uint32_t)result[14], (uint32_t)result[15]);


	value = 0x0;
	for(i = 8, count = 0; i > 1 ; i--, count++) /* 184 to 239 bits*/
		value = value | ((uint64_t)(temp[i]) << (count * 8));

	printf("value=0x%llx \n", (unsigned long long)value);
	prepad_parse_values(value, "19,23,30,31,37,39,41,47,48", result);
	printf("SQ           HASH      BK UC UB\n");
	printf("0x%08x   0x%02x(%03d) %d  %d  %d\n", 
			(uint32_t)result[0], (uint32_t)result[2], (uint32_t)result[2], 
			(uint32_t)result[4], (uint32_t)result[6], (uint32_t)result[8]);
}

