/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

//#include "pci.h"
#include <stdio.h>
#include <stdlib.h>

#include "mac_defines.h"
#include "xgmac.h"
#include "types.h"
#include "mips-exts.h"
#include "mipsregs.h"
#include "msgring.h"
#include "iomap.h"
#include "netdevice.h"
#include "spinlock.h"
#include "packet.h"
#include "pod.h"
#include "mdio.h"
#include "asmmacros.h"
#include "spill.h"
#include "classifier.h"
#include "iomap.h"
#include "mm.h"
#include "system.h"
#include "atomic.h"
#include "shmvars.h"

static int xgmac_mdio_setup  (volatile unsigned int *_mmio); 
static void nlm_xgmac_inject_free_desc(device_conf *current_dev); 

static uint32_t xgmac_offsets[] = { 	PHOENIX_IO_XGMAC_0_OFFSET, 
					PHOENIX_IO_XGMAC_1_OFFSET };
extern struct block_config  blk_conf[4];
classifier_config classifier_conf_xgmac = {
	/* GMAC INIT */
	.classify_gmac.enable = 0,
	.classify_gmac.rx_enable = 0,
	.classify_gmac.ttable_mode = USE_HASH,
	.classify_gmac.prepad_enable = 0,
        .classify_gmac.byte_offset = 0,
	.classify_gmac.num_classes = 4,
	.classify_gmac.ethertype = 0xffff,
	.classify_gmac.offset_l4_prot = 0,
	.classify_gmac.offset_0 = 0,
	.classify_gmac.len_0 = 0,
	.classify_gmac.offset_1 = 0,
	.classify_gmac.len_1 = 0,
	.classify_gmac.offset_2 = 0,
	.classify_gmac.len_2 = 0,
	.classify_gmac.offset_3 = 0,
	.classify_gmac.len_3 = 0,
	.classify_gmac.offset_4 = 0,
	.classify_gmac.len_4 = 0,
	.classify_gmac.offset_5 = 0,
	.classify_gmac.len_5 = 0,

	/* XGS0 INIT */
	.classify_xgs0.enable = 1,
	.classify_xgs0.rx_enable = 1,
	.classify_xgs0.ttable_mode = USE_HASH,
	.classify_xgs0.prepad_enable = 0,
        .classify_xgs0.byte_offset = 0,
	.classify_xgs0.num_classes = 4,
	.classify_xgs0.ethertype = 0xffff,
	.classify_xgs0.offset_l4_prot = 0,
	.classify_xgs0.offset_0 = 0,
	.classify_xgs0.len_0 = 0,
	.classify_xgs0.offset_1 = 0,
	.classify_xgs0.len_1 = 0,
	.classify_xgs0.offset_2 = 0,
	.classify_xgs0.len_2 = 0,
	.classify_xgs0.offset_3 = 0,
	.classify_xgs0.len_3 = 0,
	.classify_xgs0.offset_4 = 0,
	.classify_xgs0.len_4 = 0,
	.classify_xgs0.offset_5 = 0,
	.classify_xgs0.len_5 = 0,

	/* XGS1 INIT */
	.classify_xgs1.enable = 1,
	.classify_xgs1.rx_enable = 1,
	.classify_xgs1.ttable_mode = USE_HASH,
	.classify_xgs1.prepad_enable = 0,
        .classify_xgs1.byte_offset = 0,
	.classify_xgs1.num_classes = 4,
	.classify_xgs1.ethertype = 0xffff,
	.classify_xgs1.offset_l4_prot = 0,
	.classify_xgs1.offset_0 = 0,
	.classify_xgs1.len_0 = 0,
	.classify_xgs1.offset_1 = 0,
	.classify_xgs1.len_1 = 0,
	.classify_xgs1.offset_2 = 0,
	.classify_xgs1.len_2 = 0,
	.classify_xgs1.offset_3 = 0,
	.classify_xgs1.len_3 = 0,
	.classify_xgs1.offset_4 = 0,
	.classify_xgs1.len_4 = 0,
	.classify_xgs1.offset_5 = 0,
	.classify_xgs1.len_5 = 0,
};
void nlm_xgmac_soft_reset( phoenix_reg_t * mmio)
{
	int i ;

	phoenix_write_reg( mmio, XG_CONFIG_0, FIELD_VALUE( MACRST, 1 ));
	for (i=0; i <0xfffff  ; i++) { }	
	phoenix_write_reg (mmio,  XG_CONFIG_0,FIELD_VALUE(MACRST,0));

	for (i=0; i <0xfffff  ; i++){}
 	phoenix_write_reg (mmio, XG_CONFIG_0,FIELD_VALUE(RSTRCTL,1)   |
					FIELD_VALUE(RSTRFN,1)    |
					FIELD_VALUE(RSTTCTL,1)   |
					FIELD_VALUE(RSTTFN,1)
							);
	for (i=0; i <0xfffff  ; i++){ }	

	phoenix_write_reg(mmio, RX_CONTROL, FIELD_VALUE(SINGLE_PORT,1));

	while ((GET_FIELD(phoenix_read_reg( mmio, RX_CONTROL), RX_HALT)) == 0);
  
	// soft reset all Xgmac's
	phoenix_write_reg (mmio, RX_CONTROL, FIELD_VALUE(SOFT_RESET,1) | FIELD_VALUE(SINGLE_PORT,1));

	while ((GET_FIELD(phoenix_read_reg(mmio, RX_CONTROL),SOFT_RESET_DONE)) == 0);
 
	phoenix_write_reg (mmio, RX_CONTROL, FIELD_VALUE(SOFT_RESET,0));

	/* Reset does not seem to clear this field */
	phoenix_write_reg (mmio, CORE_CONTROL, 0);
  
	phoenix_write_reg (mmio, XG_CONFIG_0,
			FIELD_VALUE(RSTRCTL,0)   |
			FIELD_VALUE(RSTRFN,0)    |
			FIELD_VALUE(RSTTCTL,0)   |
			FIELD_VALUE(RSTTFN,0)
			);
	
	printf("[%s] Soft Reset Complete for port \n", __FUNCTION__);
}

int nlm_xgmac_init( device_conf * current_dev )
{	
	volatile unsigned short *cpld ;
	uint32_t i =0;
	port_info * internal_info = NULL;
	struct xgmac_shm_vars *m;

	
	if ( !xlr_board_atx_ii()) {
		printf("XGMAC interface is not supported on this board\n");
		return ERR_DEVICE_NOT_SUPORTTED;
	}

	if ( current_dev == NULL )
		return ERR_INVALID_DEVICE_CONFIG;

	if((current_dev->shm = nlm_xgmac_shm_init()) == NULL)
		return ERR_NO_MEM;
	m = current_dev->shm;

	spin_lock(&m->xgmac_lock[current_dev->id]) ;


	if ( check_device_config( current_dev ) ==  ERR_INVALID_DEVICE_CONFIG ){
		spin_unlock(&m->xgmac_lock[current_dev->id]) ;
		return ERR_INVALID_DEVICE_CONFIG;
	}


	if ( m->xgmac_dev_init[current_dev->id] == 1 ) {
		spin_unlock(&m->xgmac_lock[current_dev->id]) ;
		return INIT_NOT_NEEDED;
	}

	internal_info = ( port_info *) current_dev->intrl;

	m->xgmac_free_desc[current_dev->id] = 0;

	current_dev->mmio = ( phoenix_reg_t *) (phoenix_io_base + xgmac_offsets[current_dev->id]);

	phoenix_write_reg( current_dev->mmio, RX_CONTROL, FIELD_VALUE(RX_ENABLE,0) |
					 FIELD_VALUE(ERROR_STATUS, current_dev->error_mask));
	phoenix_write_reg ( current_dev->mmio, TX_CONTROL, FIELD_VALUE(TX_THRESHOLD,1518) |
					 FIELD_VALUE(TX_ENABLE,0)) ;
	
	nlm_xgmac_soft_reset(current_dev->mmio);


	// setup cpld base address register
	cpld = (volatile unsigned short *)CPLD_BASE_ADDR;

	current_dev->max_tx_frame_len = current_dev->max_tx_frame_len /4;
	current_dev->max_rx_frame_len = current_dev->max_rx_frame_len;

	/* Set Max Frame length for Tx and Rx */
	phoenix_write_reg( current_dev->mmio, XG_MAX_FRAME_LENGTH, 
			 FIELD_VALUE(MXFRMWCTX,current_dev->max_tx_frame_len) |
			 FIELD_VALUE(MXFRMBCRX,current_dev->max_rx_frame_len));

	/* set up Pad calibration */
	phoenix_write_reg ( current_dev->mmio, PAD_CALIBRATION_0, FIELD_VALUE(LVDS_TENLMNATION,0) |
		FIELD_VALUE(HSTL_TENLMNATION,0) |
		FIELD_VALUE(CAL_PRESET,0) |
		FIELD_VALUE(TX_EN_COUNT,1) |
		FIELD_VALUE(RX_EN_COUNT,1) |
		FIELD_VALUE(TX_N_PRESET,0) |
		FIELD_VALUE(TX_P_PRESET,0) |
		FIELD_VALUE(RX_N_PRESET,0) |
		FIELD_VALUE(RX_P_PRESET,0)) ;

	/* configure the DMA_CREDIT Registers */
	phoenix_write_reg( current_dev->mmio, DMA_CR_0, FIELD_VALUE(WR0_CREDITS,7) |
		FIELD_VALUE(RD0_CREDITS,7) |
		FIELD_VALUE(WR1_CREDITS,7) |
		FIELD_VALUE(RD1_CREDITS,7) |
		FIELD_VALUE(WR2_CREDITS,7) |
		FIELD_VALUE(RD2_CREDITS,7) |
		FIELD_VALUE(WR3_CREDITS,7) |
		FIELD_VALUE(RD3_CREDITS,7) |
		FIELD_VALUE(WR4_CREDITS,7) |
		FIELD_VALUE(RD4_CREDITS,7)) ;

	phoenix_write_reg(current_dev->mmio, DMA_CR_1, FIELD_VALUE(WR5_CREDITS,7) |
		FIELD_VALUE(RD5_CREDITS,7) |
		FIELD_VALUE(WR6_CREDITS,7) |
		FIELD_VALUE(RD6_CREDITS,7) |
		FIELD_VALUE(WR7_CREDITS,7) |
		FIELD_VALUE(RD7_CREDITS,7) |
		FIELD_VALUE(WR8_CREDITS,7) |
		FIELD_VALUE(RD8_CREDITS,7) |
		FIELD_VALUE(WR9_CREDITS,7) |
		FIELD_VALUE(RD9_CREDITS,7)) ;

	phoenix_write_reg(current_dev->mmio, DMA_CR_2, FIELD_VALUE(WR10_CREDITS,7) |
		FIELD_VALUE(RD10_CREDITS,7) |
		FIELD_VALUE(WR11_CREDITS,7) |
		FIELD_VALUE(RD11_CREDITS,7) |
		FIELD_VALUE(WR12_CREDITS,7) |
		FIELD_VALUE(RD12_CREDITS,7) |
		FIELD_VALUE(WR13_CREDITS,7) |
		FIELD_VALUE(RD13_CREDITS,7) |
		FIELD_VALUE(WR14_CREDITS,7) |
		FIELD_VALUE(RD14_CREDITS,7)) ;

	phoenix_write_reg(current_dev->mmio, DMA_CR_3, FIELD_VALUE(WR15_CREDITS,7) |
		FIELD_VALUE(RD15_CREDITS,7) |
		FIELD_VALUE(SP_CLASS_WR_CREDITS,2) |
		FIELD_VALUE(SP_CLASS_RD_CREDITS,2) |
		FIELD_VALUE(REG_FR_IN_WR_CREDITS,2) |
		FIELD_VALUE(REG_FR_IN_RD_CREDITS,2) |
		FIELD_VALUE(FR_OUT_WR_CREDITS,2) |
		FIELD_VALUE(FR_OUT_RD_CREDITS,2) |
		FIELD_VALUE(P2P_CHANNEL_RD_CREDITS,2));

	/* Clock divider for mdio */
	phoenix_write_reg (current_dev->mmio,XGMAC_CLK_DIV, FIELD_VALUE(XGMAC_MDIO_CLKDIV,0x3e)) ;
	
	/* Egress Fifo Carving slots */
	phoenix_write_reg ( current_dev->mmio, FIFO_CARVING_SLOTS0, FIELD_VALUE(CHANNEL_0_SLOT,0xf));
	phoenix_write_reg ( current_dev->mmio, FIFO_CARVING_SLOTS1, 0x0);

	/* Turn on L2 Alloc */
	phoenix_write_reg (current_dev->mmio, L2_ALLOC_CTRL,
		FIELD_VALUE(RX_L2_ALLOCATE,current_dev->l2_alloc_rx) );
		//FIELD_VALUE(TX_L2_ALLOCATE,current_dev->l2_alloc_tx));

	/* Bring CPLD out of reset */
	cpld[0x2] = 0xFFFF;
	cpld[0x8] = 0xFFFF;
	cpld[0x9] = 0xFFFF;

	// init xgs mdio interface
	xgmac_mdio_setup (current_dev->mmio);

	// setup spill areas
	nlm_config_spill(current_dev);
	// set transmit threahold for egress fifo
		
	phoenix_write_reg ( current_dev->mmio, TX_CONTROL, FIELD_VALUE(TX_THRESHOLD,1518)) ;

	if ( current_dev->rx_regular_buf_size & 0x1f ) {
		/* the frame size given is not aligned to cache line( 32 bit)*/
		current_dev->rx_regular_buf_size = (current_dev->rx_regular_buf_size & ~(0x01f)) + 32;
	}

	if ( current_dev->enable_prepad == 1 )
		current_dev->rx_regular_buf_size = current_dev->rx_regular_buf_size + 32;


	phoenix_write_reg(current_dev->mmio, DESC_PACK_CTRL,
				       FIELD_VALUE(BYTE_OFFSET,current_dev->byte_offset[0])    |// Byte offset of packet data
				       FIELD_VALUE(PREPAD_ENABLE,current_dev->enable_prepad)                |
				       FIELD_VALUE(REGULAR_SIZE,current_dev->rx_regular_buf_size));

	phoenix_write_reg (current_dev->mmio,BYTE_OFFSET_0_7,
		FIELD_VALUE(BYTE_OFFSET_0, current_dev->byte_offset[0])); 


	for(i = 0; i < 16; i++) {
		phoenix_write_reg( current_dev->mmio,XGS_TX0_BUCKET_SIZE+i,blk_conf[current_dev->id + 2].bucket_size[i]);
	}
	phoenix_write_reg( current_dev->mmio,XGS_RFR_BUCKET_SIZE, blk_conf[current_dev->id + 2].bucket_size[17]);


	for(i = 0; i < NLM_MAX_CREDIT_COUNTER; i++) {
		phoenix_write_reg( current_dev->mmio,CC_CPU0_0 + i,blk_conf[current_dev->id + 2].credit_counter[i]);
	}

	if ( current_dev->enable_stat )
		clr_enable_device_stat(current_dev);


        /* configure the XGMAC Registers */
        phoenix_write_reg( current_dev->mmio, XG_CONFIG_1,
                                         FIELD_VALUE(TFEN,1) |
                                         FIELD_VALUE(RFEN,1));

	nlm_setup_pde(current_dev);
	phoenix_write_reg( current_dev->mmio, XG_CONFIG_1, 0x50000025);
	SET_PORT_INIT(internal_info->state);
	m->xgmac_dev_init[current_dev->id] = 1;
	spin_unlock(&m->xgmac_lock[current_dev->id]) ;

	return 0;

	
}

int  nlm_xgmac_open(device_conf * current_dev )
{
	struct xgmac_shm_vars *m;

	if ( !xlr_board_atx_ii()) {
		printf("XGMAC interface is not supported on this board\n");
		return ERR_DEVICE_NOT_SUPORTTED;
	}


	if ( current_dev == NULL ) 
		return ERR_INVALID_OPERATION;

	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;

	
	spin_lock(&m->xgmac_lock[current_dev->id]) ;
	port_info * internal_inf =  ( port_info *)current_dev->intrl;

	if ( internal_inf == NULL ) {
		spin_unlock(&m->xgmac_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}

	if ( PORT_OPEN(internal_inf->state) ) {
		spin_unlock(&m->xgmac_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}

	nlm_xgmac_inject_free_desc( current_dev);

	phoenix_write_reg( current_dev->mmio, RX_CONTROL, FIELD_VALUE(RX_ENABLE,1) |
                                            FIELD_VALUE(ERROR_STATUS, current_dev->error_mask));
	
	phoenix_write_reg ( current_dev->mmio, TX_CONTROL, FIELD_VALUE(TX_THRESHOLD,1518) |
					 FIELD_VALUE(TX_ENABLE,1)) ;

	SET_PORT_OPEN(internal_inf->state);
	spin_unlock(&m->xgmac_lock[current_dev->id]) ;
	return 0;

}

int  nlm_xgmac_close( device_conf * current_dev )
{

	struct xgmac_shm_vars *m;

	if ( !xlr_board_atx_ii()) {
		printf("XGMAC interface is not supported on this board\n");
		return ERR_DEVICE_NOT_SUPORTTED;
	}


	if ( current_dev == NULL ) 
		return ERR_INVALID_OPERATION;

	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;


	spin_lock(&m->xgmac_lock[current_dev->id]) ;
	port_info * internal_inf =  ( port_info *)current_dev->intrl;

	if ( internal_inf == NULL ) {
		spin_unlock(&m->xgmac_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}

	if ( PORT_CLOSE(internal_inf->state)) {
		spin_unlock(&m->xgmac_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}

	phoenix_write_reg( current_dev->mmio, RX_CONTROL, FIELD_VALUE(RX_ENABLE,0) |
                                            FIELD_VALUE(ERROR_STATUS, current_dev->error_mask));

	
	phoenix_write_reg ( current_dev->mmio, TX_CONTROL, FIELD_VALUE(TX_THRESHOLD,1518) |
					 FIELD_VALUE(TX_ENABLE,0)) ;
	SET_PORT_CLOSE( internal_inf->state);
	spin_unlock(&m->xgmac_lock[current_dev->id]) ;
	return 0;
}

static int xgmac_mdio_setup  (volatile unsigned int *_mmio) {
	int i ;
	uint32_t rd_data ;

	for ( i = 0 ; i < 4 ; i ++ ) {
		rd_data = xmdio_read(_mmio,1,0x8000+i);
		rd_data = rd_data & 0xffffdfff ; 
		xmdio_write(_mmio,1,0x8000+i,rd_data); 
	}  
	return 0;
}

// descriptor injection
void nlm_xgmac_inject_free_desc(device_conf *current_dev) {
	int i;
	port_info * internal = ( port_info *)current_dev->intrl;
	int block = internal->block_id;
	int num_desc = blk_conf[block].max_frin_desc;
	int sent_cnt =0;
	struct msgrng_msg free_msg;
	void  *pbuf = 0;
	void * (*alloc_rxbuf)(void *arg) = blk_conf[block].pkt_mem->alloc_rx_buf;
	void * arg =  blk_conf[block].pkt_mem->arg;
	int desc_bucket = MSGRNG_STNID_XMAC0RFR;
	struct xgmac_shm_vars *m;

	if((m = current_dev->shm) == NULL)
		return;

	
	if ( m->xgmac_free_desc[current_dev->id] == 1 )
		return;

	if ( current_dev->id == 1 )
		desc_bucket = MSGRNG_STNID_XMAC1RFR;
	
	free_msg.msg1 = free_msg.msg2 = free_msg.msg3 = 0;
	printf("Sending free desc to xgmac station %d\n",desc_bucket);
	for(i = 0; i < num_desc; i++) {
		pbuf = alloc_rxbuf(arg); 
		if ( pbuf == NULL)
			break;
		else
			sent_cnt++;
		(void )mac_make_desc_rfr(&free_msg, 0, 0, virt_to_phys(pbuf));
		while(message_send(1, 2, desc_bucket, &free_msg));
	}
	printf("Free packets Request=%d, Sent=%d\n",num_desc,sent_cnt);
	m->xgmac_free_desc[current_dev->id] = 1;
}

