/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef __FIFO_H__
#define __FIFO_H__
#include "spinlock.h"

#define FIFO_FULL  -1
#define FIFO_EMPTY -2
struct fifo
{
    int head;
    int tail;
    int size;
};

/*
 * Name : fifo_init
 * Description : initialize fifo
 * Input :
 * Output :
 */
static __inline__ void fifo_init (struct fifo *instance, int size)
{
    instance->head = 0;
    instance->tail = 0;
    instance->size = size;
}

/*
 * Name : get_fifo_insert
 * Description : get the index to insert an item into the fifo
 * Input :
 * Output :
 */
static __inline__ int get_fifo_insert (struct fifo *instance)
{
    int val = ((instance->head + 1) % instance->size);

    if (val != instance->tail) {
        return (val);
    }
    else { /* fifo full */
        return FIFO_FULL;
    }
}

/*
 * Name : fifo_insert
 * Description : insert an item into the fifo
 * Input :
 * Output :
 */
static __inline__ void fifo_insert (struct fifo *instance)
{
    instance->head = (instance->head + 1) % instance->size;
    barrier();
}

/*
 * Name : get_fifo_remove
 * Description : get the index to remove an item from fifo
 * Input :
 * Output :
 */
static __inline__ int get_fifo_remove (struct fifo *instance)
{
    if (instance->tail == instance->head) { /*fifo empty */
        return FIFO_EMPTY;
    }
    else {
        return ((instance->tail + 1) % instance->size);
    }
}

/*
 * Name : fifo_remove
 * Description : remove an item from fifo
 * Input :
 * Output :
 */
static __inline__ void fifo_remove (struct fifo *instance)
{
    instance->tail = (instance->tail + 1) % instance->size;
    barrier();
}

#endif
