/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

/********************************************************************************
 hexutil.h - A utility to convert ascii chars to hex and back

	MODIFICATION HISTORY
 	--------------------
 	v1.0,16oct05,Suresh DS   written

********************************************************************************/

#ifndef __HEXUTIL_H__
#define __HEXUTIL_H__

const static unsigned char i_to_hex_chars[] = "0123456789abcdef";

/*
  Converts '[0-9,a-f]' to integer equivalent.
*/
static __inline__ unsigned char hex_to_uchar(unsigned char h)
{
	if (h >= 'a' && h <= 'f')
		return h - 'a' + 10;

	if (h >= '0' && h <= '9')
		return h - '0';

	if (h >= 'A' && h <= 'F')
		return h - 'A' + 10;

	return 0;
}

/*
  Converts the low nibble of i to its hex character equivalent.
*/
static __inline__ unsigned char uchar_to_hex(unsigned char i)
{
	return i_to_hex_chars[i & 0xf];
}

/*
  Converts 8 hex bytes to a unsigned int.
*/
static __inline__ unsigned int hexbuf_to_uint(unsigned char *hbuf)
{
	unsigned int count;
	unsigned int retval;

	for (count = 0, retval = 0; count < 8 && *hbuf; count++, hbuf++)
		retval = (retval << 4) + hex_to_uchar(*hbuf);

	return retval;
}

/*
  For xlr-mips64:
  Converts 16 hex bytes to a unsigned long long.
*/
static __inline__ unsigned long long hexbuf_to_ulonglong(unsigned char *hbuf)
{
	unsigned int count;
	unsigned long long retval;

	for (count = 0, retval = 0; count < 16 && *hbuf; count++, hbuf++)
		retval = (retval << 4) + hex_to_uchar(*hbuf);

	return retval;
}

/*
  Converts unsigned int to 8-byte hbuf.
*/
static __inline__ void uint_to_hexbuf(unsigned int i, unsigned char *hbuf)
{
	unsigned int count;

	for (hbuf += 7, count = 0; count < 8; count++, hbuf--) {
		*hbuf = uchar_to_hex(i & 0xf);
		i >>= 4;
	}
}

/*
  For xlr-mips64:
  Converts unsigned long long to 16-byte hbuf.
*/
static __inline__ void
ulonglong_to_hexbuf(unsigned long long i, unsigned char *hbuf)
{
	unsigned int count;

	for (hbuf += 15, count = 0; count < 16; count++, hbuf--) {
		*hbuf = uchar_to_hex(i & 0xf);
		i >>= 4;
	}
}

/*
  Converts array of unsigned char's to hex equivalent.

  hbuf -- where to put hex characters
  ibuf -- where to get integers from
  ilen -- number of *ibuf's to convert
*/
static __inline__ void
ucharbuf_to_hbuf(unsigned char *hbuf, unsigned char *ibuf, unsigned int ilen)
{
	while (ilen--) {
		*hbuf++ = uchar_to_hex(*ibuf >> 4);
		*hbuf++ = uchar_to_hex(*ibuf);
		ibuf++;
	}
}

/*
  Converts array of hex characters to integer equivalents.
 
  ibuf -- where to put integers
  hbuf -- where to get hex characters from

  ilen -- number of *ibuf's to convert
*/
static __inline__ void
hexbuf_to_ucharbuf(unsigned char *ibuf, unsigned char *hbuf, unsigned int ilen)
{
	while (ilen--) {
		*ibuf = hex_to_uchar(*hbuf++) << 4;
		*ibuf += hex_to_uchar(*hbuf++);
		ibuf++;
	}
}

#endif
