/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

/* configuration structure for networking devices */

#ifndef __CLASSIFIER_CONFIG_H
#define __CLASSIFIER_CONFIG_H

#include "types.h"
#include "parser.h"

typedef enum ttable_mode_enum {
	USE_HASH,
	USE_L3L4,
	USE_PORT,
	USE_MSB7
} ttable_mode_type ;

typedef struct parser_tables_struct {
	prsr_cfg_entry    prsr_cfg_entry_v ;
    l2_table          l2_table_v ;
    l3_table          l3_table_v ;
    l4_table          l4_table_v ;
	ttable            ttable_v ;
    cam_table         cam_table_v ;
    tcam_ttable		tcam_ttable_v;
} parser_tables ;

typedef struct cl_cfg_iface_struct {
	/* Global enables */
	uint32_t          enable ;
	ttable_mode_type  ttable_mode ;
	uint32_t          prepad_enable ;
        uint32_t          byte_offset;
	uint32_t          num_classes ;
	uint32_t          rx_enable ;

	/* L2 Table Custom Entry */
	uint32_t          offset_0 ;
	uint32_t          len_0 ;
	uint32_t          offset_l3_prot ;
	
	/* L3 Table Custom Entry */
	uint32_t          ethertype ; // 2bytes
	uint32_t          offset_l4_prot ;
	uint32_t          offset_1 ;
	uint32_t          len_1 ;
	uint32_t          offset_2 ;
	uint32_t          len_2 ;
	uint32_t          offset_3 ;
	uint32_t          len_3 ;
	
	/* L4 Table Custom Entry */
	uint32_t          l4_proto ; // single byte
	uint32_t          offset_4 ;
	uint32_t          len_4 ;
	uint32_t          offset_5 ;
	uint32_t          len_5 ;

} cl_cfg_iface ;

/* top config structure */

typedef struct classifier_config_struct {
	cl_cfg_iface      classify_gmac ;
	cl_cfg_iface      classify_xgs0 ;
	cl_cfg_iface      classify_xgs1 ;
} classifier_config ;

/* structure instantiation */
// extern classifier_config    classifier_conf;
extern parser_tables        parser_tables_gmac;
extern parser_tables        parser_tables_xgs0;
extern parser_tables        parser_tables_xgs1;

/* prototype of init function */

extern void xlr_classifier_init(classifier_config *classifier_conf);
extern void xlr_classifier_update(classifier_config *classifier_conf, net_config *net_conf);

/* Translate Table populating functions */

extern void ttable_setup_mask (uint32_t cpu_mask, 
                               uint32_t use_bucket, 
                               cl_cfg_iface *classify_iface, 
                               parser_tables *parser_tables_p);

extern void ttable_port_based_setup (cl_cfg_iface *classify_iface, 
                                     parser_tables *parser_tables_p);

extern classifier_config classifier_conf_gmac;
extern classifier_config classifier_conf_xgmac;
extern classifier_config classifier_conf_spi4;

#endif
