#ifndef _DEVICE_H
#define _DEVICE_H

#define TYPE_NLM_GMAC 0x1
#define TYPE_NLM_XGS 0x2


enum {
	GMAC_BLOCK_0 = 0,
	GMAC_BLOCK_1,
	XGS_BLOCK_0,
	XGS_BLOCK_1
};
	


enum {
INITILIZED = 0x1,
OPEN = 0x2,
CLOSE =0x4
};

#define SET_PORT_INIT(x) ( x = INITILIZED )
#define SET_PORT_OPEN(x) ( x = INITILIZED | OPEN )
#define SET_PORT_CLOSE(x) ( x = INITILIZED | CLOSE)

#define PORT_INIT(x) ( x & INITILIZED )
#define PORT_OPEN(x) ( x & OPEN )
#define PORT_CLOSE(x) ( x & CLOSE)

#define NLM_MAX_GMAC 8
#define NLM_MAX_XGS 2
#define NLM_MAX_SPI4_PORT 20
#define NLM_MAX_SPI4_PORT_PER_BLOCK 10
#define NLM_MAX_DEVICE_BUCKET 18 
#define NLM_MAX_CREDIT_COUNTER 128
#define MAX_SPI4_PORT 16

enum {
ERR_AUTONEG_NOT_COMPLETE = 1,
ERR_INVALID_DEVICE_CONFIG ,
ERR_INVALID_OPERATION ,
ERR_DEVICE_NOT_SUPORTTED ,
ERR_INVALID_PORT ,
ERR_NO_MEM ,
INIT_NOT_NEEDED ,
ERR_NT_XLS ,
ERR_INVALID_GMAC,
ERR_XAUI_MODE,

};

enum {
	GMAC_00,
	GMAC_01,
	GMAC_02,
	GMAC_03,
	GMAC_04,
	GMAC_05,
	GMAC_06,
	GMAC_07,
	SPI4_0_0,
	SPI4_0_1,
	SPI4_0_2,
	SPI4_0_3,
	SPI4_0_4,
	SPI4_0_5,
	SPI4_0_6,
	SPI4_0_7,
	SPI4_0_8,
	SPI4_0_9,
	SPI4_0_10,
	SPI4_0_11,
	SPI4_0_12,
	SPI4_0_13,
	SPI4_0_14,
	SPI4_0_15,
	SPI4_1_0,
	SPI4_1_1,
	SPI4_1_2,
	SPI4_1_3,
	SPI4_1_4,
	SPI4_1_5,
	SPI4_1_6,
	SPI4_1_7,
	SPI4_1_8,
	SPI4_1_9,
	SPI4_1_10,
	SPI4_1_11,
	SPI4_1_12,
	SPI4_1_13,
	SPI4_1_14,
	SPI4_1_15,
	XGMAC_PORT_0,
	XGMAC_PORT_1
};

typedef void * (*alloc_rx_buf_func_t)(void *arg) ;
typedef void (*free_rx_buf_func_t)(void *arg, void *data);
typedef struct packet_mem_rt_t {
	void * (*alloc_rx_buf)(void *arg);
	void * arg;
	void (*free_rx_buf)(void *arg, void *data);
	uint64_t (*virt_to_phys)(void *vaddr);
	void *(*phys_to_virt)(uint64_t paddr);
}packet_mem_rt;


typedef  struct port_info_t
{
	int state;
	int rgmii;
	int phy_id;
	int block_id;
	unsigned long phy_addr;
	int pcs_id;
	unsigned long pcs_addr;
	int serdes_id;
	unsigned long serdes_addr;
}port_info;

typedef struct device_config_t {
	uint32_t magic;       /* this field is checked to verify 
				whether the device structure is created using API */
	int type;
	int id;              /* gmac id */
	int xgs_type;
	phoenix_reg_t *mmio;
	int max_tx_frame_len; /* maximum length of frame that can be passed from the Network Accelerators to the MAC */
	int max_rx_frame_len; /* maximum length of frame that can be passed to the Network Accelerators from the MAC */
	int rx_regular_buf_size; /* size of each message unit in the FMN, i.e max size of msg0/msg1/msg2/msg3 */
	int enable_stat;      /* enabling the gmac statistic */
	int byte_offset[MAX_SPI4_PORT];      /* packets byte offset */
	int l2_alloc_rx;      /* number of cache line that are used to store the receive packet */
	int l2_alloc_tx;      /* number of cache lines that are used for storing packets to transmit */
	int enable_prepad; 
	int error_mask;
	int error_id;	      /* In case of Tx abort, desc are sent to this station instead of 
				FBI given in the descriptors  */	
	struct phy_config  { /* Phy_config will be used by spi4 and gmac but not xgmac */
		int auto_neg;
		int speed;         /* Can be 1000/100/10. Default speed or speed read from the phy, if auto neg is enabled*/
		int duplex;        /* half 0, full 1. Default duplexity/ duplexity read from the phy*/
	}phy_conf[MAX_SPI4_PORT]; 
	void * intrl;         /* filled internally */
	void * shm;
}device_conf;

struct block_config {
	int max_frin_desc;    /* number of free in desc per station */
	int max_tx_desc;      /* max number of tx descriptor per station  */		
	int tx_desc_size;     /* number of msg unit that each FMN message contain */
	uint64_t pde_class0_mask;
	uint64_t pde_class1_mask;
	uint64_t pde_class2_mask;
	uint64_t pde_class3_mask;

	unsigned short bucket_size[NLM_MAX_DEVICE_BUCKET];
	unsigned short credit_counter[NLM_MAX_CREDIT_COUNTER];
	packet_mem_rt *pkt_mem; /*this has function pointers for those that works on 
				the packet       */
};

	

typedef struct mac_stat_t {
        /* transmit and receive counter */
        uint32_t tx_rx_64b;
        uint32_t tx_rx_65b_127b;
        uint32_t tx_rx_127b_255b;
        uint32_t tx_rx_255b_511b;
        uint32_t tx_rx_511b_1023b;
        uint32_t tx_rx_1023b_1518b;
        uint32_t tx_rx_1024b__1518;
        uint32_t tx_rx_1519b_1522b;

        /* Receive counters */
        uint32_t rx_byte;
        uint32_t rx_pkt;
        uint32_t rx_fcs_err;
        uint32_t rx_multicst;
        uint32_t rx_brdcst;
        uint32_t rx_ctrl;
        uint32_t rx_pause;
        uint32_t rx_unkwn_OPCode;
        uint32_t rx_align_err;
        uint32_t rx_frm_len_err;
        uint32_t rx_code_err;
        uint32_t rx_carrier_sense_err;
        uint32_t rx_undersize_pkt_err;
        uint32_t rx_oversize_pkt_err;
        uint32_t rx_fragments;
        uint32_t rx_jabber;
        uint32_t rx_dropped;

        /* Transmit counters */
        uint32_t tx_byte;
        uint32_t tx_pkt;
        uint32_t tx_multicst;
        uint32_t tx_brdcast;
        uint32_t tx_pause_ctrl;
        uint32_t tx_deferral;
        uint32_t tx_excessive;
        uint32_t tx_signal_collision;
        uint32_t tx_multiple;
        uint32_t tx_late_colliaion;
        uint32_t tx_excessive_collision;
        uint32_t tx_total_collision;
        uint32_t tx_pause_fm_honored;
        uint32_t tx_drop;
        uint32_t tx_jabber_fmt;
        uint32_t tx_fcs_err;
        uint32_t tx_ctrl;
        uint32_t tx_oversize;
        uint32_t tx_undersize;
        uint32_t tx_fragment;
        uint32_t tx_carry_reg_1;
        uint32_t tx_carry_reg_2;
        uint32_t tx_carry_mask_reg_1;
        uint32_t tx_carry_mask_reg_2;
}mac_stat;


device_conf * nlm_alloc_device(int id, int type);
int nlm_net_device_block_init( device_conf * current_dev,  struct block_config * block_data );
int nlm_net_device_init( device_conf * current_dev, int rgmii );
int nlm_net_device_open( device_conf * current_dev, int port );
int nlm_net_device_close( device_conf * current_dev, int port );
int check_device_config( device_conf * current_dev);
int clr_enable_device_stat(device_conf * current_dev);
mac_stat * read_device_stat( device_conf * current_dev);
void nlm_setup_pde(device_conf * current_dev );
void print_device_err_msg( int i );

#endif
