#ifndef _DEVICES_H
#define _DEVICES_H

#include <sys/types.h>
#include <reent.h>

#define MAX_DEVICES 100

struct device_ops {
	const char *name;
	int (*open_r)(struct _reent *reent, const char *path, int flags, int mode);
	int (*close_r)(struct _reent *reent, int fd);
	_ssize_t (*read_r)(struct _reent *reent, int fd, char *ptr, size_t len);
	_ssize_t (*write_r)(struct _reent *reent, int fd, const char *ptr, size_t len);
	int (*ftrunc)(struct _reent *reent, int fd, off_t len);
	void *(*mmap)(struct _reent *reent, int fd, void *addr, size_t len, int prot, int flags, off_t off);
	int (*munmap)(struct _reent *reent, void *addr, size_t len);
	void (*init)(void);
	void *obj;
};

extern struct device_ops dev_stdio_in;
extern struct device_ops dev_stdio_out;
extern struct device_ops dev_stdio_err;
extern struct device_ops dev_tmpfs;
extern struct device_ops dev_mem;

int fdget(void);
void fdput(int fd);
void fdset(int fd);
/* there should not be any device add, this should be open + create */
int dev_add(struct device_ops *dev, int fd);
struct device_ops *dev_get(int fd);

#endif /* _DEVICES_H */
