/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _GMAC_H
#define _GMAC_H

//#include <mac.h>
#include "types.h"
#include "msgring.h"
#include "spinlock.h"
#include "cache.h"
#include "mips-exts.h"
#include "netdevice.h"
#include "iomap.h"
#include "device.h"
/*
 * GMAC Interface Address Definitions
 */

enum gmac_config {
	G_CONFIG_1                 = 0x00,
	G_CONFIG_2,
	G_IPG,
	G_HALF_DUPLEX,
	G_MAX_FRAME_LENGTH,
	G_TEST                     = 0x07,
	G_MII_MGMT_CONFIG,
	G_MII_MGMT_COMMAND,
	G_MII_MGMT_ADDRESS,
	G_MII_MGMT_CONTROL,
	G_MII_MGMT_STATUS,
	G_MII_MGMT_INDICATORS,
	G_INTERFACE_CONTROL,
	G_INTERFACE_STATUS,
	G_SA_PART_1,
	G_SA_PART_2,

	G_TR_64                    = 0x20,
	G_TR_127,
	G_TR_255,
	G_TR_511,
	G_TR_1k,
	G_TR_MAX,
	G_TX_MGV,

	G_RBYT,
	G_RPKT,
	G_RFCS,
	G_RMCA,
	G_RBCA,
	G_RXCF,
	G_RXPF,
	G_RXUO,
	G_RALN,
	G_RFLR,
	G_RCDC,
	G_RCSE,
	G_RUND,
	G_OVR,
	G_RFRG,	
	G_RJBR,
	G_RDRP,

	G_TBYT         = 0x38,
	G_TPKT,
	G_TMCA,
	G_TBCA,
	G_TXPF,
	G_TDFR,
	G_TEDF,
	G_TSCL,
	G_TMCL,
	G_TLCL,
	G_TXCL,
	G_TNCL,
	G_TPFH,
	G_TDRP,
	G_TJBR          = 0x46 ,
	G_TFCS,
	G_TXCS,
	G_TOVR,
	G_TUND,
	G_TRFG,
	CAR1,
	CAR2,
	CAM1,
	CAM2,


	G_MAC_ADDR0_0              = 0x50,
	G_MAC_ADDR0_1,
	G_MAC_ADDR1_0,
	G_MAC_ADDR1_1,
	G_MAC_ADDR2_0,
	G_MAC_ADDR2_1,
	G_MAC_ADDR3_0,
	G_MAC_ADDR3_1,
	G_MAC_ADDR_MASK2_0,
	G_MAC_ADDR_MASK2_1,
	G_MAC_ADDR_MASK3_0,
	G_MAC_ADDR_MASK3_1,
	G_MAC_FILTER_CONFIG,

};

enum {
	FDX_HALF         = 0x00,
	FDX_FULL,
};

enum {
	MAC_SPEED_1000     = 1000,
	MAC_SPEED_100 = 100,
	MAC_SPEED_10 = 10,
};

enum {
		SGMII_SPEED_10   = 0x00000000,
		SGMII_SPEED_100  = 0x02000000,
		SGMII_SPEED_1000 = 0x04000000,
};

enum {
	PCS_CTRL = 0x00,
	PCS_STATUS,
	PCS_SGMII_ADRV = 0x04,
	PCS_SGMII_LINK_PG_ABLTY,
	PCS_EXPANSION,
	PCS_NXT_PG_TRANS,
	PCS_LINK_PARTNER_ABLTY_NXT_PG,
	PCS_EXT_STATUS = 0X0F,
	PCS_JITTER_DIAG,
	PCS_TBI_CTRL,
};
	
	


#define MAC_FRAME_SIZE 1536

// gmac driver function prototypes

/* function prototypes */
void arizona_gmac_phy_init(phoenix_reg_t * gmac_offset);
int arizona_gmac_phy_read (unsigned long gmac_offset, int num, int reg, uint32_t *data); 
int arizona_gmac_phy_write (unsigned long gmac_offset, int num, int reg, uint32_t data);
void serdes_regs_init( void );

int nlm_gmac_init(device_conf  * current_dev, int rgmii);
int nlm_gmac_open(device_conf  * current_dev);
int nlm_gmac_close(device_conf  * current_dev);
int nlm_check_mac_speed( device_conf * current_dev);
#endif
