/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _I2C_H
#define _I2C_H

#define WIDTH 
#define I2C_CONFIG     0
#define I2C_CLKDIV     1
#define I2C_DEVADDR    2
#define I2C_ADDR       3
#define I2C_DATAOUT    4
#define I2C_DATAIN     5
#define I2C_STATUS     6
#define I2C_START      7
#define I2C_BYTECNT    8
#define I2C_HDSTATIM   9

#define I2C_READ_XFR   1
#define I2C_WRITE_XFR  0
#define I2C_NODATA_XFR 2

/* Assuming i2c controller frequency of 66MHz and SCLK max of 400KHz */
#define I2C_SCLK_400k_DIV 0x52

/* Assuming i2c controller frequency of 66MHz and SCLK max of 200KHz */
#define I2C_SCLK_200k_DIV 0xA4
#define I2C_CLOCK_DIVISOR I2C_SCLK_400k_DIV

// macros

#define I2C_LOST_ARB(status) ((status) & 0x20)
#define I2C_STARTERR(status) ((status) & 0x10)
#define I2C_ACKERR(status) ((status) & 0x8)
#define I2C_DATARDY(status) ((status) & 0x4)
#define I2C_SDOEMPTY(status) ((status) & 0x2)
#define I2C_BUSY(status) ((status) & 0x1)

#define I2C_RESTART_XFER(status) (I2C_STARTERR(status) || I2C_LOST_ARB(status))

#define ARIZONA_EEPROM_BUS 1
#define ARIZONA_EEPROM_ADDR 0x50

#define ARIZONA_EEPROM_SIGNATURE_OFFSET 0
#define ARIZONA_EEPROM_CKSUM_OFFSET 4
#define ARIZONA_EEPROM_BNAME_OFFSET 0x10
#define ARIZONA_EEPROM_MAJOR_OFFSET 0x18
#define ARIZONA_EEPROM_MINOR_OFFSET 0x19
#define ARIZONA_EEPROM_BOARDREV_OFFSET 0x1A
#define ARIZONA_EEPROM_BOARD_ID_OFFSET0 0x1C
#define ARIZONA_EEPROM_BOARD_ID_OFFSET1 0x1D
#define ARIZONA_EEPROM_AVAILABLE_OFFSET 0x1E
#define ARIZONA_EEPROM_ETH_MAC_OFFSET 0x20
#define ARIZONA_EEPROM_QDR2_OFFSET 0x26
#define ARIZONA_EEPROM_QDR2_SIZE_OFFSET 0x27
#define ARIZONA_EEPROM_RLD2A_OFFSET 0x28
#define ARIZONA_EEPROM_RLD2A_SIZE_OFFSET 0x29
#define ARIZONA_EEPROM_RLD2B_OFFSET 0x2a
#define ARIZONA_EEPROM_RLD2B_SIZE_OFFSET 0x2b

#define ARIZONA_SPD_BUS 0
#define ARIZONA_SPD_EEPROM0 0x54
#define ARIZONA_SPD_EEPROM1 0x55
#define ARIZONA_SPD_EEPROM2 0x56
#define ARIZONA_SPD_EEPROM3 0x57

#define ARIZONA_TEMPSENSOR_BUS 1
#define ARIZONA_TEMPSENSOR_ADDR 0x4C
#define ARIZONA_TEMPSENSOR_EXT_TEMP 1

/* Prototypes of Functions */

/*******************************************************************************
 * Initialize iobase pointers etc
 ******************************************************************************/
extern void xlr_i2c_init();
extern int xlr_i2c_intr_init(int bus, unsigned int vcpu_mask);

/*******************************************************************************
 *
 * Write_board_eeprom:
 * Writes a value to an offset on the board eeprom
 * Inputs:  internal_addr(address within device) , data to write
 * Returns: 0 (success) -1 (failure). 
 *
 ******************************************************************************/
extern int write_board_eeprom(unsigned offset, unsigned char data);

/*******************************************************************************
 *
 * read_board_eeprom
 * Reads a character from an offset in the board eeprom
 * Inputs: offset inside the eeprom
 * Returns: 0 (success) -1 (failure). On success, data contains the character read back
 *
 ******************************************************************************/
extern int read_board_eeprom(unsigned offset, unsigned char *data);

/*******************************************************************************
 *
 * read_spd_eeprom
 * Reads a character from an offset in the spd eeprom
 * Inputs: offset inside the eeprom
 * Returns: 0 (success) -1 (failure). On success, data contains the character read back
 *
 ******************************************************************************/
extern int read_spd_eeprom(unsigned offset, unsigned char *data);

/*******************************************************************************
 *
 * read_thermsensor
 * Reads current temperature
 * Inputs: none
 * Returns: 0 (success) -1 (failure). On success, data contains the temperature
 *
 ******************************************************************************/
extern int read_ext_tempsensor(unsigned char *data);

#endif
