/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/
#ifndef __MEMPOOL_KERN_H__
#define __MEMPOOL_KERN_H__

#include <limits.h>
#include "list.h"
#include "spinlock.h"
#include "mm.h"
#ifndef NAME_MAX
#define NAME_MAX 256
#endif

/* The below variables all are not configurable. But depending on the usage 
the allocater can tune those paramters */
struct mempool
{
	char name[NAME_MAX];
	size_t size;
	size_t align;
	int flags;
	int dlimit; /* when it is full, the mem will be moved out of the pool */
	spinlock_t  dlock; /* lock for depot list */
	int ndelem; /* number of elements in the depot */
	struct list depot; /* depot list, shared by all the cpus */
	int climit; /* cache limits. otherwise it will be moved to depot queue */
	spinlock_t      clock[MAX_CPUS]; /* cache lock */
	int ncelem[MAX_CPUS]; /* number of elements in the cache */
	struct list cache[MAX_CPUS]; /* per cpu list */
	void *virtstart;  /* virt start of this pool */
	unsigned long long physstart; /* physical start of this pool */
};

static inline int get_my_cpu(void)
{
	int pid;
	extern int getpid();
	pid = getpid();
	if(pid >= MAX_CPUS)
		return 0;
	return pid;
}

static inline int get_num_active_cpus(void)
{
	/* number of cpus , the application is running */
	/* TODO :  */
	return MAX_CPUS;
}

#define SPIN_LOCK_INIT spin_lock_init
#ifndef NLM_LINUXU
	#define SPIN_LOCK_IRQ_SAVE spin_lock_irqsave
	#define SPIN_UNLOCK_IRQ_RESTORE spin_unlock_irqrestore
#endif

#endif /* MEMPOOL */
