/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _PACKET_H
#define _PACKET_H

#include "types.h"
#include "stddef.h"
#include "list.h"
#include "net_config.h"

#define PKT_DATA_LEN 1592

#define PKT_SEC_AUTH_LEN 32
#define PKT_SEC_CTRL_DESC_LEN 128
#define PKT_SEC_PKT_DESC_LEN 32

#define MAX_PACKETS 16384
#define DEFAULT_SPILL_ENTRIES (2048)

#define MAX_FRIN_SPILL_ENTRIES DEFAULT_SPILL_ENTRIES
#define MAX_FROUT_SPILL_ENTRIES DEFAULT_SPILL_ENTRIES
#define MAX_RX_SPILL_ENTRIES DEFAULT_SPILL_ENTRIES

struct packet {
	uint8_t     data[PKT_DATA_LEN]; 		/* New cacheline */
	uint32_t    len;
	uint32_t    seq_num;
	uint8_t             padding[24];
} __attribute__((aligned(32)));

struct size_1_desc {
	uint64_t entry0;
};

struct size_2_desc {
	uint64_t entry0;
	uint64_t entry1;
};

struct size_3_desc {
	uint64_t entry0;
	uint64_t entry1;
	uint64_t entry2;
};

struct size_4_desc {
	uint64_t entry0;
	uint64_t entry1;
	uint64_t entry2;
	uint64_t entry3;
};

struct fr_desc {
	struct size_1_desc d1;
};

union rx_tx_desc {
	struct size_2_desc d2;
	struct size_3_desc d3;
	struct size_4_desc d4;
};

#define RX_DESC_ENTRIES 2
#define TX_DESC_ENTRIES 2

extern struct packet    *packets;
extern struct packet    *alloc_pbuf(void);
extern void free_pbuf(struct packet *pbuf);
extern int  packet_init(char * addr , uint64_t size);
extern void  packet_deinit( void);
extern struct packet *(*net_conf_alloc_pbuf)(void);
extern void (*net_conf_spill_setup)(struct net_config_struct *);

static inline int pkt_valid(struct packet *pbuf)
{
	/* if (pbuf < &packets[0] || pbuf >= &packets[MAX_PACKETS])
		return 0; 
	*/
	return 1;
}

static __inline__ void *pkt_get_data_ptr(struct packet *pbuf)
{
	return pbuf->data;
}

static __inline__ uint16_t pkt_get_len(struct packet *pbuf)
{
	return pbuf->len;
}

#endif
