/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _SHMVARS_H
#define _SHMVARS_H

#include <stdlib.h>
#include <unistd.h>
#include <sched.h>
#include <fcntl.h>
#include <sys/mman.h>

struct net_shm_vars {
	spinlock_t init_lock;
	int init_done;
};

struct gmac_shm_vars {
	spinlock_t shmlock;
	int shminitdone;
	int gmac_speed[NLM_MAX_GMAC]; 
	int gmac_duplex[NLM_MAX_GMAC]; 
	volatile unsigned int gmac_common_init[2];
	int gmac_free_desc[NLM_MAX_GMAC];
	int gmac_dev_init[NLM_MAX_GMAC];
	spinlock_t   gmac_lock[NLM_MAX_GMAC];
	volatile int gmac_xaui_mode[NLM_MAX_GMAC];
};

struct xgmac_shm_vars {
	spinlock_t   xgmac_lock[NLM_MAX_XGS];
	int xgmac_free_desc[NLM_MAX_XGS];
	int xgmac_dev_init[NLM_MAX_XGS];
};

struct spi4_shm_vars {
	spinlock_t shmlock;
	int shminitdone;
	int spi4_speed[NLM_MAX_SPI4_PORT];
	int spi4_duplex[NLM_MAX_SPI4_PORT];
	int spi4_free_desc[NLM_MAX_XGS];
	int spi4_dev_init[NLM_MAX_XGS];
	spinlock_t   spi4_lock[NLM_MAX_XGS];
};

struct netblk_shm_vars {
	struct block_config  blk_conf[4];
	int block_init[4];
	spinlock_t   block_init_lock[4];
	volatile unsigned int nlm_spill_init[4];
};

struct misc_shm_vars {
	spinlock_t shmlock;
	int shminitdone;
	spinlock_t   classifier_init_lock;
	volatile int classifier_init_done;
	spinlock_t      * pkt_pool_spinlock;  
	int             * pkt_pool_inited;
	void		* pool_header;

};

static inline void *nlm_net_shm_init(void)
{
	int fd, rv;
	const char *name = "/netshm";
	struct net_shm_vars *m;

	fd = shm_open(name, O_CREAT | O_RDWR, 0600);
	if(fd < 0) {
		printf("shm_open failed for %s\n", name);
		return NULL;
	}

	rv = ftruncate(fd, sizeof(struct net_shm_vars));
	if(rv < 0) {
		printf("ftruncate failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	m = mmap(NULL, sizeof(struct net_shm_vars), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if(m == MAP_FAILED) {
		printf("mmap failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}
	return m;

err_exit:
	shm_unlink(name);
	return NULL;
}



static inline void *nlm_gmac_shm_init(void)
{
	int fd, rv;
	const char *name = "/gmacshm";
	struct gmac_shm_vars *m;
	int i;

	fd = shm_open(name, O_CREAT | O_RDWR, 0600);
	if(fd < 0) {
		printf("shm_open failed for %s\n", name);
		return NULL;
	}

	rv = ftruncate(fd, sizeof(struct gmac_shm_vars));
	if(rv < 0) {
		printf("ftruncate failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	m = mmap(NULL, sizeof(struct gmac_shm_vars), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if(m == MAP_FAILED) {
		printf("mmap failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}
	spin_lock(&m->shmlock);
	if(!m->shminitdone) {
		/* Shared memory is initialized to zero, modify only non zero init vars */
		for(i = 0; i < NLM_MAX_GMAC; i++) {
			m->gmac_speed[i] = -1;
			m->gmac_duplex[i] = -1;
		}
		m->shminitdone = 1;
	}

	spin_unlock(&m->shmlock);
	return m;

err_exit:
	shm_unlink(name);
	return NULL;
}


static inline void *nlm_xgmac_shm_init(void)
{
	int fd, rv;
	const char *name = "/xgmacshm";
	struct xgmac_shm_vars *m;

	fd = shm_open(name, O_CREAT | O_RDWR, 0600);
	if(fd < 0) {
		printf("shm_open failed for %s\n", name);
		return NULL;
	}

	rv = ftruncate(fd, sizeof(struct xgmac_shm_vars));
	if(rv < 0) {
		printf("ftruncate failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	m = mmap(NULL, sizeof(struct xgmac_shm_vars), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if(m == MAP_FAILED) {
		printf("mmap failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	return m;

err_exit:
	shm_unlink(name);
	return NULL;
}


static inline void *nlm_spi4_shm_init(void)
{
	int fd, rv;
	const char *name = "/spi4shm";
	struct spi4_shm_vars *m;
	int i;

	fd = shm_open(name, O_CREAT | O_RDWR, 0600);
	if(fd < 0) {
		printf("shm_open failed for %s\n", name);
		return NULL;
	}

	rv = ftruncate(fd, sizeof(struct spi4_shm_vars));
	if(rv < 0) {
		printf("ftruncate failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	m = mmap(NULL, sizeof(struct spi4_shm_vars), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if(m == MAP_FAILED) {
		printf("mmap failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}
	spin_lock(&m->shmlock);
	if(!m->shminitdone) {
		/* Shared memory is initialized to zero, modify only non zero init vars */
		for(i = 0; i < NLM_MAX_GMAC; i++) {
			m->spi4_speed[i] = -1;
			m->spi4_duplex[i] = -1;
		}
		m->shminitdone = 1;
	}

	spin_unlock(&m->shmlock);
	return m;

err_exit:
	shm_unlink(name);
	return NULL;
}


static inline void *nlm_netblk_shm_init(void)
{
	int fd, rv;
	const char *name = "/netblkshm";
	struct netblk_shm_vars *m;

	fd = shm_open(name, O_CREAT | O_RDWR, 0600);
	if(fd < 0) {
		printf("shm_open failed for %s\n", name);
		return NULL;
	}

	rv = ftruncate(fd, sizeof(struct netblk_shm_vars));
	if(rv < 0) {
		printf("ftruncate failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	m = mmap(NULL, sizeof(struct netblk_shm_vars), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if(m == MAP_FAILED) {
		printf("mmap failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	return m;

err_exit:
	shm_unlink(name);
	return NULL;
}

static inline void *nlm_misc_shm_init(void)
{
	int fd, rv;
	const char *name = "/miscshm";
	struct misc_shm_vars *m;

	fd = shm_open(name, O_CREAT | O_RDWR, 0600);
	if(fd < 0) {
		printf("shm_open failed for %s\n", name);
		return NULL;
	}

	rv = ftruncate(fd, sizeof(struct misc_shm_vars));
	if(rv < 0) {
		printf("ftruncate failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	m = mmap(NULL, sizeof(struct misc_shm_vars), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if(m == MAP_FAILED) {
		printf("mmap failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	spin_lock(&m->shmlock);
	if(!m->shminitdone) {
		m->pkt_pool_inited = (int *)0xdeadbeef;
		m->shminitdone = 1;
	}

	spin_unlock(&m->shmlock);
	return m;

err_exit:
	shm_unlink(name);
	return NULL;
}

#endif
