/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _TEST_LIB_H
#define _TEST_LIB_H

#include <stdio.h>
#include <stdlib.h>

#include "types.h"
#include "msgring.h"
#include "netdevice.h"
#include "mac.h"
#include "iomap.h"
#include "pkt_gen.h"
#include "analyzer.h"
#include "mdio.h"

// PSB APP environment includes
#include "psb.h"

// Standard RTL message
static inline void write_rtl_msg(uint32_t code, uint32_t value) {
	
	uint64_t oss = ((uint64_t)value << 32) | (uint64_t)code;
	__asm__ volatile (".set push\n"
	                  ".set mips64\n"
	                  "dmtc0 %0, $22, 7\n"
	                  ".set pop\n"
	                  :
	                  : "r"(oss)
	                  );
}

// message to indicate that a test has passed
static inline void test_pass() {
#ifdef _RTL
	write_rtl_msg(0x900dbeef,0x900dbeef);
	asm volatile("wait");
#else
	printf ("Test Passed\n");
	printf ("900dbeef\n");
	exit(0);
#endif
}

// message to indicate test has failed
static inline void test_fail(int code) {
#ifdef _RTL
	write_rtl_msg(0xbaadbeef,code);
	asm volatile("wait");
#else
	printf ("Test Failed with Code %0d\n", code);
	printf ("baadbeef\n");
	exit(-1);
#endif
}

// message to tenlmnate test
static inline void tenlmnate_sim () {
	int code=0;
#ifdef _RTL
	write_rtl_msg(0xface0fff,code);
	asm volatile("wait");	   
#else
	printf ("Phoenix Stopped with Code %0d\n", code);
	printf ("face0fff\n");
	asm volatile("wait");
#endif
}

// write messages to STDout
static inline void test_msg_write(uint32_t code, uint32_t value) {
	
#ifdef _RTL	
	uint64_t oss = ((uint64_t)value << 32) | (uint64_t)code;
	__asm__ volatile (".set push\n"
	                  ".set mips64\n"
	                  "dmtc0 %0, $22, 7\n"
	                  ".set pop\n"
	                  :
	                  : "r"(oss)
	                  );

#else
	printf ("Test Code--> %0d value (%0d)         Test Code --> %0x value (%0x)\n", code, value, code, value);
#endif 
}

// Fork off simulation thread 
static inline void fork_func(unsigned long func_ptr, unsigned int mask) {
#ifdef _PSB_APP
#error "fork_func() not defined"
#else
	 //run_othercpus((unsigned long)func_ptr, mask);
#endif
}

// init function
static inline void test_init(){
}

// exit function
static inline void test_exit() {
}

// Used in all RTL tests
static inline void config_injector(int injector, unsigned long long value) {
	  __asm__ volatile (".set push\n"
	                    ".set mips64\n"
	                    "dmtc0 $0, $22, 0\n"
	                    "dmtc0 %0, $22, 0\n"
	                    ".set pop\n" 
	                    : 
	                    : "r"(value)
	                    );
}

#endif
