/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _NETOS_TIME_H
#define _NETOS_TIME_H

#include <pt_regs.h>
#include "irq.h"
#include "list.h"


#define PIC_CLKS_PER_SEC 66666666ULL

#undef HZ
#define HZ 1000

/*
 * TODO: detenlmne the following constant at run time
 */
extern unsigned int nlm_cpu_frequency;
#define CYCLES_PER_JIFFY (nlm_cpu_frequency/HZ)

/* can't do floating in the kernel, so use 66 as an approximation */
#define PIC_CLKS_PER_USEC 66    // (PIC_CLKS_PER_SEC / 1000000)
#define PIC_CLKS_PER_TIMER_TICK (PIC_CLKS_PER_SEC / HZ)

struct timer_list_node {
   list_t  peer_tm_slot;           /* list of timer which are sorted based on the clock ticks */ 
   list_t  eq_tm_slot;             /* list of timer that gets expired at the same clock tick  */ 
   unsigned long long ticks_to_expire;   /* number of ticks left (from now) before expiry */
   unsigned long long ori_timer_ticks;   /* ticks required for expiry when the timer is created   */ 
   int periodic;                         /* Periodic timer ? 1: 0*/
   int timer_state;                      /* State of the timer */
   void (*function)(unsigned long);      /* timer routine */
   unsigned long data;                   /* argument for the timer routine */                
   unsigned int action; 
};

enum action_bit {
	NONE = 0,
	RUN ,
	STOP,
	FREE
};
	
enum timer_state {
    CREATED,
    RUNNING,
    EXPIRED,
    STOPPED,
    DESTROYED
};

enum timer_error {
	ERR_ZERO_TICKS = -5,
	ERR_NULL_PARAM ,
	ERR_EMPTY_HEAD ,
	ERR_NON_RUNNING,
	ERR_EXPIRED_STATE,
	TIMER_OK
};

extern int phoenix_timer_setup(void);
irqreturn_t phoenix_timer_interrupt(int irq, struct pt_regs *regs);

extern unsigned long long get_current_timer_tick();
int is_timer_enabled();
void  nlm_timer_run();
void nlm_timer_init(void);
int  nlm_timer_stop(void * timer_to_stop);
int nlm_timer_delete( void  * timer_to_delete );
int nlm_timer_start(void  * timer_to_add);
int nlm_list_time_to_expire( void * timer_to_probe, uint64_t *ticks_left);
void * nlm_timer_create(unsigned long long ticks_to_expire,
                unsigned int is_periodic, void (*function)(unsigned long), 
					unsigned long data);
#endif
