/* XLP CPU specific */

#ifndef _XLP_CPU_H
#define _XLP_CPU_H

#include "types.h"

#define CHIP_PROCESSOR_ID_XLR       0x00
#define CHIP_PROCESSOR_ID_XLS_608   0x80  
#define CHIP_PROCESSOR_ID_XLS_408   0x88
#define CHIP_PROCESSOR_ID_XLS_404   0x8c
#define CHIP_PROCESSOR_ID_XLS_208   0x8e
#define CHIP_PROCESSOR_ID_XLS_204   0x8f
#define CHIP_PROCESSOR_ID_XLS_108   0xce
#define CHIP_PROCESSOR_ID_XLS_104   0xcf

/* Defines for 'XLS B0' */
#define CHIP_PID_XLS_616_B0         0x40
#define CHIP_PID_XLS_608_B0         0x4a
#define CHIP_PID_XLS_416_B0         0x44
#define CHIP_PID_XLS_412_B0         0x4c
#define CHIP_PID_XLS_408_B0         0x4e
#define CHIP_PID_XLS_404_B0         0x4f

/* New for XLP */
#define CHIP_PID_XLP                0x00

/* CPU Internal Blocks specific to XLP .
 * These are accessed using the mfcr/mtcr
 * instructions. Blocks [0-5] are same for
 * XLR and XLP
 */
#define CPU_BLOCKID_MAP					0x0a
/* Offsets of interest from the 'MAP' Block */
#define BLKID_MAP_THREADMODE			0x00 
#define BLKID_MAP_EXT_EBASE_ENABLE		0x04 
#define BLKID_MAP_CCDI_CONFIG			0x08
#define BLKID_MAP_THRD0_CCDI_STATUS		0x0c	
#define BLKID_MAP_THRD1_CCDI_STATUS		0x10
#define BLKID_MAP_THRD2_CCDI_STATUS		0x14	
#define BLKID_MAP_THRD3_CCDI_STATUS		0x18
#define BLKID_MAP_THRD0_DEBUG_MODE		0x1c
#define BLKID_MAP_THRD1_DEBUG_MODE		0x20
#define BLKID_MAP_THRD2_DEBUG_MODE		0x24
#define BLKID_MAP_THRD3_DEBUG_MODE		0x28
#define BLKID_MAP_MISC_STATE			0x60
#define BLKID_MAP_DEBUG_READ_CTL		0x64
#define BLKID_MAP_DEBUG_READ_REG0		0x68
#define BLKID_MAP_DEBUG_READ_REG1		0x6c

#define CPU_BLOCKID_SCH				7
#define CPU_BLOCKID_SCU				8
#define CPU_BLOCKID_FPU				9

/* ----------------------------------
 *   XLP RESET Physical Address Map
 * ----------------------------------
 * PCI ECFG : 0x18000000 - 0x1bffffff 
 * PCI CFG  : 0x1c000000 - 0x1cffffff 
 * FLASH    : 0x1fc00000 - 0x1fffffff 
 * ----------------------------------
 */

/* The DEFAULT_XLP_IO_BASE value is what is
 * programmed in the NBU's (NorthBridge Unit) 
 * ECFG_BAR register. The NBU itself is 
 * accessible as [BDF:0,0,0].
 */
#define DEFAULT_XLP_IO_BASE         	0xffffffffb8000000ULL
#define DEFAULT_XLP_IO_BASE_VIRT       	0xb8000000 	/* IO_BASE for Assembly macros */
#define DEFAULT_CPU_IO_BASE         	DEFAULT_XLP_IO_BASE
#define DEFAULT_CPU_IO_BASE_VIRT       	DEFAULT_XLP_IO_BASE_VIRT
#define CPU_IO_SIZE 					(64<<20)	/* Size of the ECFG Space      */
#define HDR_OFFSET						0x100		/* Skip 256 bytes of cfg. hdrs */

/* The On-Chip functional blocks for XLP */

/* ------------------------------------------------------------------------*/
/* Accesses Based on Enhanced Configuration Mechanism					   */
/* ------------------------------------------------------------------------*/
/* Interface 						   | 	Bus    |	Dev    	|   Func   */
/* ------------------------------------------------------------------------*/
#define BRIDGE							(0x00<<20) | (0x00<<15) | (0x00<<12)
#define	PIC								(0x00<<20) | (0x00<<15) | (0x04<<12)
#define	CMS								(0x00<<20) | (0x04<<15) | (0x00<<12)
#define UART0							(0x00<<20) | (0x06<<15) | (0x00<<12)
#define UART1							(0x00<<20) | (0x06<<15) | (0x01<<12)
#define I2C0							(0x00<<20) | (0x06<<15) | (0x02<<12)
#define I2C1							(0x00<<20) | (0x06<<15) | (0x03<<12)
#define	GPIO							(0x00<<20) | (0x06<<15) | (0x04<<12)
#define SYS								(0x00<<20) | (0x06<<15) | (0x05<<12)
#define	JTAG							(0x00<<20) | (0x06<<15) | (0x06<<12)
#define	NOR								(0x00<<20) | (0x07<<15) | (0x00<<12)
#define	NAND							(0x00<<20) | (0x07<<15) | (0x01<<12)
#define	SPI								(0x00<<20) | (0x07<<15) | (0x02<<12)
#define	MMC								(0x00<<20) | (0x07<<15) | (0x03<<12)

/* ------------------------------------------------------------------------*/

#define CPU_MMIO_OFFSET(x)				DEFAULT_CPU_IO_BASE_VIRT + (x) + HDR_OFFSET

#define PCIX          					0x09000
#define HT            					0x0A000
#define PCIE_0                          0x1E000
#define PCIE_1                          0x1F000
#define SRAM                            0x07000
#define PIOBUS                          0x19000

/* Networking Blocks */
#define GMAC0                           0x0C000
#define GMAC1                           0x0D000
#define GMAC2                           0x0E000
#define GMAC3                           0x0F000
#define GMAC4                           0x20000
#define GMAC5                           0x21000
#define GMAC6                           0x22000
#define GMAC7                           0x23000

#define L2            					0x1b000

/**/
#define XGMAC0                          0x11000
#define XGMAC1                          0x13000
#define SPI40                           0x10000
#define SPI41                           0x12000

#define SECURITY                        0x0B000
/**/

#define DDR2_CHN0                       0x01000
#define DDR2_CHN1                       0x02000
#define DDR2_CHN2                       0x03000
#define DDR2_CHN3                       0x04000

#define RLD2_CHN0                       0x05000
#define RLD2_CHN1                       0x06000

#define USB                             0x24000

#ifndef __ASSEMBLY__

#define cpu_read_reg(base, offset) 			((base)[(offset)])
#define cpu_write_reg(base, offset, value)  ((base)[(offset)] = (value))
/* Stub these out for now... */

static inline unsigned int xlr_revision(void)
{
    return chip_id() & 0xff00ff;
}

static __inline__ int chip_processor_id_xls(void) 
{
    return 0;
}

static __inline__ int chip_processor_id_xls4xx(void) 
{
    return 0;
}

static __inline__ int chip_processor_id_xls2xx(void) 
{
    return 0;
}

static __inline__ int chip_processor_id_xls1xx(void) 
{
    return 0;
}

static __inline__ int chip_pid_xls_b0(void) 
{
    return 0;
}

static __inline__ int is_xlr_c_series(void)
{
    return 0;
}
#endif				/* __ASSEMBLY__ */
#endif
