/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/
#include <stdio.h>
#include <stdlib.h>
#include <iomap.h>
#ifdef XLP_SIM
#include "nlm_hal_pic.h"
#else
#include "pic.h"
#endif
#include "irq.h"
#include <stddef.h>
#include <mips-exts.h>
#include "spinlock.h"
#include "xlr_boot_lib.h"
#include "handlers.h"
#include "sys_entry.h"
#include "irqparam.h"

extern int gdb_connection_mode;
extern irqreturn_t gdb_continue_ipi_handler(int irq, struct pt_regs *ptregs);
extern irqreturn_t gdb_stop_ipi_handler(int irq, struct pt_regs *ptregs);
void (*netos_shutdown_func[NCPU]) (void *);
void *netos_shutdown_args[NCPU];
void (*netos_join_func[NCPU]) (void *);
void *netos_join_args[NCPU];
extern int nlm_sys_call(int sysno, void *arg);


int request_irq(unsigned int irq,
		irqreturn_t (*handler)(int, struct pt_regs *),
		unsigned long irqflags,
		unsigned int mask)
{
	struct irq_param s;
	s.cmd = NLM_REQUEST_IRQ;
	s.irq = irq;
	s.handler = (unsigned long)handler;
	s.irqflags = irqflags;
	s.mask = mask;
	nlm_sys_call(NLM_IRQ_SYS_CALL, (void *)&s);
	return s.rv;
}

void free_irq(unsigned int irq)
{
	struct irq_param s;
	s.cmd = NLM_FREE_IRQ;
	nlm_sys_call(NLM_IRQ_SYS_CALL, (void *)&s);
	return;

}

void set_irqops(int irq, int option, void *function)
{
	struct irq_param s;
	s.cmd = NLM_SET_IRQOPS;
	s.options = option;
	s.handler = (long)function;
	nlm_sys_call(NLM_IRQ_SYS_CALL, (void *)&s);
	return;

}

int register_shutdown_handler(int cpu_id, void (*func)(void *), void *args)
{
	if (cpu_id > NCPU)
		return -1;

	netos_shutdown_func[cpu_id] = func;
	netos_shutdown_args[cpu_id] = args;

	return 0;
}

void free_shutdown_handler(int cpu_id)
{
	if (cpu_id > NCPU)
		return ;

	netos_shutdown_func[cpu_id] = netos_shutdown_args[cpu_id] = NULL;
}

int register_join_handler(int cpu_id, void (*func)(void *), void *args)
{
	if (cpu_id > NCPU)
		return -1;

	netos_join_func[cpu_id] = func;
	netos_join_args[cpu_id] = args;

	return 0;
}

void free_join_handler(int cpu_id)
{
	if (cpu_id > NCPU)
		return ;

	netos_join_func[cpu_id] = netos_join_args[cpu_id] = NULL;
}

int init_irq(void)
{
	int retval;
	retval = request_irq(IRQ_STOP_CPU_IPI, stop_cpu_ipi_handler,
			IRQ_FLAGS_RESERVED, 1 << processor_id());

	if (retval)
		goto ret;

	retval = request_irq(IRQ_WAKEUP_CPU_IPI, wakeup_cpu_ipi_handler,
			IRQ_FLAGS_RESERVED, 1 << processor_id());
	if (retval)
		goto ret;

	retval = request_irq(IRQ_JOIN_CPU_IPI, join_cpu_ipi_handler,
			IRQ_FLAGS_RESERVED, 1 << processor_id());
	if (retval)
		goto ret;

	if (gdb_connection_mode >= 0)
	{
		retval = request_irq(IRQ_STOP_GDB_IPI, gdb_stop_ipi_handler,
				IRQ_FLAGS_RESERVED, 1 << processor_id());
		if (retval)
			goto ret;

		retval = request_irq(IRQ_BA_GDB_IPI, gdb_stop_ipi_handler,
				IRQ_FLAGS_RESERVED, 1 << processor_id());
		if (retval)
			goto ret;
	}

	return 0;

ret:
	return -1;
}


void trap_init(void)
{

}


int set_except_vector (int n, void *addr)
{
	struct except_param s;
	s.cmd = NLM_SET_EXCEPT_VECTOR;
	s.index = n;
	s.handler = (long)addr;
	nlm_sys_call(NLM_EXCEPT_SYS_CALL, (void *)&s);
	return s.rv;
}

#if defined(_MIPS_ARCH_XLP)
/* Some routines Imported from elfwb
 */
uint64_t read_timer_cycles(void)
{
        uint64_t ret;

        __asm__ volatile (
                ".set push \n"
                ".set arch=xlp\n"
                "dmfur %0, $16\n"
                ".set pop\n"
                :"=r"(ret)
                );

        return ret;
}

double read_timer_period(void)
{
	const double period = 7.5 * 1.0e-9; /* 133.33 MHz */
	return period;
}

double read_timer_seconds(void)
{
	uint64_t cycles = read_timer_cycles();
	const double period = read_timer_period();
	return (double)cycles * period;
}

void setup_timer(int cpu)
{
}
#else
uint64_t read_timer_cycles(void)
{
        return 0; /* TODO */
}

double read_timer_period(void)
{
        return 0; /* TODO */
}

double read_timer_seconds(void)
{
        return 0; /* TODO */
}

void setup_timer(int cpu)
{
}
#endif /* _MIPS_ARCH_XLP */
