/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/
#include <stdio.h>
#include <stdlib.h>

#include <assert.h>

#include "iomap.h"
#ifdef XLP_SIM
#include "nlm_hal_pic.h"
#else
#include "pic.h"
#endif
#include "timer.h"
#include "mips-exts.h"
#include "pt_regs.h"
#include "irq.h"
#include "system.h" /* for variable 'current' */
#include "irqparam.h" /* for variable 'current' */

void (*perfctr_sample_thread)(void *) = NULL;
void (**perfctr_ihandler)(unsigned long) = NULL;

int perfctr_enabled = 0;
int perfctr_interrupt_enabled = 0;

unsigned long long current_timer_tick[NCPU] = {0};

void pic_timer_ack (int irq)
{
#ifdef XLP_SIM

#if 0
	nlm_hal_ack_pic(nlm_hal_irq_to_irt(PIC_TIMER_7_IRQ));
#endif

#else

     phoenix_write_reg(phoenix_io_mmio(PHOENIX_IO_PIC_OFFSET),
     	PIC_INT_ACK, (1 << (PIC_TIMER_7_IRQ - PIC_IRQ_BASE)));
#endif
}

int cpu0_timer_startup(int irq)
{
#ifdef XLP_SIM

#if 0
	nlm_hal_write_pic_reg((pic_reg_t *) XLP_IO_PIC_OFFSET, PIC_TIMER_7_MAXVAL, 
		      PIC_CLKS_PER_TIMER_TICK);

	/* Todo: Need to change phnx_cpu_id() to xlp_cpu_id() */
	nlm_hal_pic_write_irt_direct(PIC_IRT_TIMER_7, IRT_VALID, 0,
			     LOCAL_SCHEDULING, PIC_TIMER_7_IRQ, processor_id());

	nlm_hal_pic_update_control (1 << (10 + 7));
#endif

#else
	phoenix_write_reg(phoenix_io_mmio(PHOENIX_IO_PIC_OFFSET), 
					  PIC_TIMER_7_MAXVAL_0, (PIC_CLKS_PER_TIMER_TICK & 0xffffffff));
	phoenix_write_reg(phoenix_io_mmio(PHOENIX_IO_PIC_OFFSET), 
					  PIC_TIMER_7_MAXVAL_1, (PIC_CLKS_PER_TIMER_TICK >> 32) & 0xffffffff);
	
	phoenix_write_reg(phoenix_io_mmio(PHOENIX_IO_PIC_OFFSET), 
					  PIC_IRT_0_TIMER_7, 
					  (1 << ((phnx_cpu_id() << 2) | phnx_thr_id())));
	phoenix_write_reg(phoenix_io_mmio(PHOENIX_IO_PIC_OFFSET), 
					  PIC_IRT_1_TIMER_7, 
					  (IRT_VALID << 31) | (EDGE_TRIGGER << 30) | 
					  (LOCAL_SCHEDULING << 6) | PIC_TIMER_7_IRQ);
	
	pic_update_control (1 << (8 + 7));
#endif // ifdef XLP_SIM

	return 0;
}

int phoenix_timer_setup(void)
{
	int retval;

    if (processor_id() == 0) {
		set_irqops(PIC_TIMER_7_IRQ, NLM_SET_STARTUP, (void *) cpu0_timer_startup);
		set_irqops(PIC_TIMER_7_IRQ, NLM_SET_PROACK, (void *)pic_timer_ack);
				retval = request_irq(PIC_TIMER_7_IRQ, phoenix_timer_interrupt, EMPTY_IRQ_FLAGS,
					(1 << processor_id()));
    }
	else {
		/* we could move the following *1* line to startup routine */
		write_c0_compare(read_c0_count() + CYCLES_PER_JIFFY);
		retval = request_irq(IRQ_TIMER, phoenix_timer_interrupt, EMPTY_IRQ_FLAGS,
					(1 << processor_id()));
	}

	return retval;
}

void save_epc(unsigned long *epc)
{
    __asm__ __volatile__ (
		".set push\n" 
        ".set noreorder\n"
        "mfc0 %0, $14\n"
        ".set pop\n"
        : "=r" (*epc) 
	);
}

#define is_timer_enabled() (0)
#define nlm_timer_run()

irqreturn_t phoenix_timer_interrupt(int irq, struct pt_regs *regs)
{
    if ((irq != PIC_TIMER_7_IRQ) && (irq != IRQ_TIMER)) {
        printf("cpu_%d: bad timer irq = %x\n", processor_id(), irq);
        return -1;
    }

	if (perfctr_interrupt_enabled) {
		assert((current != NULL) && (perfctr_ihandler != NULL));
		(*perfctr_ihandler)(instruction_pointer(regs));
	}

    if (processor_id() == 0) {
        if (irq == PIC_TIMER_7_IRQ) {

			++current_timer_tick[processor_id()];
			
			if (perfctr_enabled) {
				assert((current != NULL) && (perfctr_sample_thread != NULL));
				perfctr_sample_thread(current);
			}

			if (is_timer_enabled() )
				nlm_timer_run();
            return 0; 
        }       
        else {
            /* 
			 * assumes that count-compare interrupt 
			 * is not active in cpu0
			 */
            write_c0_compare(0);
        }
    }
    else {
		++current_timer_tick[processor_id()];
        write_c0_compare(read_c0_count() + CYCLES_PER_JIFFY);

		if (perfctr_enabled) {
			assert((current != NULL) && (perfctr_sample_thread != NULL));
			perfctr_sample_thread(current);
		}

		if (is_timer_enabled() )
			nlm_timer_run();
    }

	return 0;
}

unsigned long long get_current_timer_tick()
{
    return current_timer_tick[processor_id()];
}
