/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include "psb.h"
#include "xlr_boot_lib.h"
#include "tlb.h"

static spinlock_t default_print_lock = {
	.lock = 0
};

static volatile spinlock_t *print_lock = &default_print_lock;

void set_print_lock(spinlock_t *lock)
{
	if (lock) print_lock = lock;
}

void release_recursive_print_lock(void)
{
	uint32_t *lock= (uint32_t *)print_lock;

	/* release if same cpu currently holds the lock */
	if(*lock == processor_id() + 1)
		*lock = 0;
}

unsigned int acquire_print_lock(void)
{
	unsigned int flags = 0;

	release_recursive_print_lock();
	lib_spin_lock_irqsave(print_lock, flags);

	return flags;
}

void release_print_lock(unsigned int flags)
{
	lib_spin_unlock_irqrestore(print_lock, flags);
}

