/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include "analyzer.h"
#include "pkt_gen.h"

//******************************************************************************** 
//* Function   : 
//* Desription : Check if the length matches the expected length
//* Return     : 0 for match, 1 for no match 
//* Arguments  : 
//******************************************************************************** 

int pkt_chk_check_length (p_packet pbuf, int t_length) 
{
	return ( !( pbuf->len == t_length ) ) ;
}


//******************************************************************************** 
//* Function   : 
//* Desription : Verify packet FCS
//* Return     : 0 for correct FCS, 1 for wrong FCS
//* Arguments  : 
//******************************************************************************** 

int pkt_chk_check_crc ( p_packet pbuf ) 
{
	unsigned int pktCrc_computed ;
	unsigned int pktCrc_actual ;

	pktCrc_computed = ~crc32(pbuf->data,(pbuf->len)-4);

	pktCrc_actual   = ( pbuf->data[(pbuf->len)-1] << 24 ) |
		          ( pbuf->data[(pbuf->len)-2] << 16 ) |
		          ( pbuf->data[(pbuf->len)-3] << 8  ) |
		          ( pbuf->data[(pbuf->len)-4] << 0  ) ;
	return ( !( pktCrc_computed == pktCrc_actual ) ) ;
	
}

//******************************************************************************** 
//* Function   : 
//* Desription : Compare two packets and check if they are the same
//* Return     : 0 for match 
//             : 1 for length mismatch 
//             : 2 for data mismatch
//* Arguments  : 
//******************************************************************************** 

int pkt_chk_compare_packet ( p_packet pbuf1, p_packet pbuf2 )
{
	int i ;
	int mismatch ;

	mismatch = 0 ;

	if ( pbuf1->len != pbuf2->len ) {
		mismatch = 1 ;
	} else {
		for ( i = 0 ; i < pbuf1->len ; i ++ ) {
			if ( pbuf1->data[i] != pbuf2->data[i] ) {
				mismatch = 1 ;
			}
		}
	}

	return ( mismatch ) ;
}

//******************************************************************************** 
//* Function   : 
//* Desription : Extract the sequence number from the packet
//* Return     : sequence number - 32 bits
//             : returns 0xffffffff if sequence pattern is not found
//* Arguments  : 
//******************************************************************************** 

int pkt_chk_get_seq_num ( p_packet pbuf )
{
	int i ;
	uint32_t seq_num ;
	uint32_t temp_array ;

	seq_num = 0xffffffff ;
	temp_array = 0 ;

	for ( i = 0 ; i < pbuf->len ; i ++ ) {
		temp_array = ( temp_array << 8 ) | pbuf->data[i] ;
		if ( temp_array == SEQ_SIGNATURE ) {
			seq_num = pbuf->data[i+1] << 24 |
				  pbuf->data[i+2] << 16 |
				  pbuf->data[i+3] << 8  |
				  pbuf->data[i+4] ;
			break ;
		}
	}	
	printf ("Sequence num = %d \n",seq_num);
	return (seq_num);
}


//******************************************************************************** 
//* Function   : 
//* Desription : Extract the sequence number from the packet and 
//             : compare expected sequence number
//* Return     : 0 for match, 
//             : 1 for no match
//* Arguments  : 
//******************************************************************************** 

int pkt_chk_sequence_num (p_packet pbuf, int t_seq) 
{
	int seq_num_match ;

	seq_num_match = 1 ;

	if ( pkt_chk_get_seq_num(pbuf) == t_seq ) {
		seq_num_match = 0 ;
	} else {
		seq_num_match = 1 ;
	}

	return ( seq_num_match ) ;

}

