/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include <stdio.h>
#include <stdlib.h>

#include "asmmacros.h"
#include "psb.h"
#include "shared_structs.h"
#include "xlr_boot_lib.h"
#include "iomap.h"
#ifdef XLP_SIM
#include "nlm_hal_pic.h"
#else
#include "pic.h"
#endif
#include "cpu.h"
#include "string.h"


extern volatile struct xlr_vcpu_wakeup_info *launcher_vcpu_info_ptr;
volatile void *netos_thr_wakeup_pointer[32] __shared_memory__;
volatile void *netos_thr_stack_pointer[32] __shared_memory__;
extern volatile int master_cpu;
extern struct exception_region *__exception_vectors;
extern int find_and_set_wired_tlbs();
extern void per_cpu_arch_init(void);
extern void per_cpu_boot_init(void);
long symtab_cpu_halt = (long)(void *)cpu_halt;

void netos_exec (void (*func_ptr)(void *), void *arg, unsigned int mask)
{
	if ( !((1 << processor_id()) & mask) ) {
		return;
	}

	(*func_ptr)(arg);
	
	return;
}

static void wakeup_cpu(int cpu, unsigned long fn, unsigned long args)
{
	volatile struct xlr_vcpu_wakeup_info *t;
	int ipi;
	int pid = cpu >> 2;
	int tid = cpu % 4;
	t = launcher_vcpu_info_ptr;
	t += cpu;

	t->func = fn;
	t->args = args;
	ipi = (pid << 20) | (tid << 16) | IRQ_WAKEUP_CPU_IPI ;
	/*Set a bit in global_wakeup_cpu_mask for this vcpu - vcpu 't' will 
	reset this.*/
	*(int *)(long)(t->global_wakeup_mask) = *(int *)(long)(t->global_wakeup_mask) | (1UL<<cpu);
#ifdef XLP_SIM
	// XLP_TODO : use proper node id based on a macro
	nlm_hal_pic_send_ipi(0, IRQ_WAKEUP_CPU_IPI, 0, cpu); 
#else
	{
	phoenix_reg_t *mmio = phoenix_io_mmio(PHOENIX_IO_PIC_OFFSET);
	phoenix_write_reg(mmio, PIC_IPI, ipi);
	}
#endif
}

void netos_init_secondary_thread(void *arg0, void *arg1, void *arg2, void *arg3)
{
	int id = processor_id();
	volatile void *fn = NULL;
	uint64_t gp,sp;

	fn = netos_thr_wakeup_pointer[id];
	
	if(!fn){
		printf("\nno function to jump!!");
		printf("\nhalting cpu %d",id);
		cpu_halt();
	}

	/* per cpu boot init */
	per_cpu_boot_init();
	
	/* intialize the cache vars */
	per_cpu_arch_init();

	/*setup exception handler for this cpu*/
	trap_init();

	netos_thr_wakeup_pointer[id] = NULL;

	gp = (long)malloc(NETOS_GUARD_PAGE_SIZE);
	if(gp == 0ULL) {
		printf("Malloc failed in alloc gp\n");
		cpu_halt();
	}
	sp = (long)netos_thr_stack_pointer[id];

	/*Fill 4k guard page with 0xdeaddead pattern*/
	memset((unsigned char *)(unsigned long)gp, 0xa5, NETOS_GUARD_PAGE_SIZE);

	/* Enabling interrepts by default to support tlb flush ipi */
	sti();

	__asm__ volatile(
			".set push\n"
			".set noreorder\n"
			"sync\n"
			"move $31,%3\n"
			"move $28, %0\n"
			"move $29, %1\n"
			"move $4, %4\n"
			"jr %2\n"
			"nop\n"			
			/*should never come here*/
			"1:b 1b\n"
			"nop\n"
			".set pop\n"
			:
			:"r"(gp),"r"(sp),"r"((unsigned long)fn),"r"((unsigned long)(cpu_halt)), "r"((unsigned long)arg3)
			:"$4","$29","$31"
		);
	// should have clobbered $28 as well, but gcc errors out
}

void psb_wakeup_cpu(void *fn, void *args, unsigned int mask)
{
	int i=0;
	volatile struct xlr_vcpu_wakeup_info *t, *b;
	unsigned long mflags=0;

	mask = mask & ~(1UL<<processor_id());
	t = launcher_vcpu_info_ptr;
	t = t + processor_id();
	for(i=0;i<32;i++){
		if(mask & (1<<i)){
			/*We will allow a buddy cpu to wakeup another buddy cpu.*/
			if(!(t->buddy_mask & (1<<i))){
				printf("\nCant wakeup VCPU %#x, This is not a " 
					"buddy cpu.\n",i);
				printf("buddy mask is %d ",t->buddy_mask);
				continue;
			}
			b = launcher_vcpu_info_ptr + i;
			if(b->cpu_status == CPU_RUNNING){
//				printf("\nCant wakeup vcpu %#x, it is not in park mode.\n",i);
				continue;
			}
			netos_thr_wakeup_pointer[i] = fn;
			spin_lock_irqsave((spinlock_t *)(long)t->loader_lock,mflags);
			wakeup_cpu(i, 
				(unsigned long) netos_init_secondary_thread, 
				(unsigned long)args);
			spin_unlock_irqrestore((spinlock_t *)(long)t->loader_lock,mflags);
		}
	}
}
