/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "psb.h"
#include "iomap.h" 

#define MAX_SAMPLES             1
#define TRACE_SIZE              255

#define PHOENIX_TB_REQ_MATCH1   0
#define PHOENIX_TB_REQ_MATCH2   1
#define PHOENIX_TB_REQ_ADDRLO   2
#define PHOENIX_TB_REQ_ADDRHI   3
#define PHOENIX_TB_CTRL_REG     4
#define PHOENIX_TB_INIT_REG     5
#define PHOENIX_TB_ACCESS_REG   6
#define PHOENIX_TB_RDDATA_REG0  7
#define PHOENIX_TB_RDDATA_REG1  8
#define PHOENIX_TB_RDDATA_REG2  9
#define PHOENIX_TB_RDDATA_REG3  10
#define PHOENIX_TB_STATUS_REG   15

struct entry {
    unsigned int e0;
    unsigned int e1;
    unsigned int e2;
    unsigned int e3;
    unsigned int status;
};
 
// static struct entry entries[MAX_SAMPLES * (TRACE_SIZE + 1)];
static struct entry *entries;
 
/* these are bitmasks */

#define SNOOP_RES_CPUSHR 0x1
#define SNOOP_RES_CPUMOD 0x2
#define SNOOP_RES_L2SHR  0x4
#define SNOOP_RES_L2MOD  0x8
 
/* Nodes : [0..7]   : cpus
 *         [8..16]  : L2
 *         [16...]  : IO
 */
 
int trace(int max_samples, int trace_size)
{
    int    i, j;
    int    count = 0;
    volatile unsigned int *mmio;
    volatile unsigned int temp=0;

    mmio = (volatile unsigned int *)0xbef1c000;

    max_samples  = max_samples ? max_samples : MAX_SAMPLES;
    trace_size   = trace_size ? trace_size : TRACE_SIZE;

    printf ("max_samples: %d\n", max_samples);
    printf ("trace_size : %d\n", trace_size);

    entries = (struct entry *) 
        malloc (sizeof (struct entry) * max_samples * (trace_size + 1));
    if (!entries) {
        printf ("%s(): Unable to allocate memory for 'struct entries'\n", __FUNCTION__);
        return -1;
    }
    memset((void *)entries, 0x00, sizeof(struct entry));

    printf ("Collecting %d sample(s):\n", max_samples);

    for (i = 0; i < max_samples; i++) {
        
        /* TB is enabled at reset; Disable it here... */
        temp = phoenix_read_reg(mmio,PHOENIX_TB_CTRL_REG);
        phoenix_write_reg( mmio,PHOENIX_TB_CTRL_REG,( temp | 0x01000000)); 

        /* Program the size of each sample */
        phoenix_write_reg( mmio,PHOENIX_TB_CTRL_REG,(trace_size << 16));
        
        /* Create a MATCH condition here, if desired */

        /* Re-enable the disabled TB Logic... */
        temp = phoenix_read_reg(mmio,PHOENIX_TB_CTRL_REG);
        phoenix_write_reg(mmio,PHOENIX_TB_CTRL_REG,( temp & 0xfeffffff));

        /* Start the trace trigger by toggling
         * the INIT register from Low to High
         */
        phoenix_write_reg(mmio,PHOENIX_TB_INIT_REG,0);                              
        phoenix_write_reg(mmio,PHOENIX_TB_INIT_REG,1);                               
        
        /* Poll till the TB is full */
        while ((phoenix_read_reg(mmio,PHOENIX_TB_STATUS_REG) & 0x40) == 0);
        
        for (j=0; j<=trace_size; j++, count++) {
            /* pop the entry */
            phoenix_write_reg( mmio,PHOENIX_TB_ACCESS_REG, 0x0);                         
            /* read the entry */
            entries[count].e0     = phoenix_read_reg( mmio,PHOENIX_TB_RDDATA_REG0);           
            entries[count].e1     = phoenix_read_reg( mmio,PHOENIX_TB_RDDATA_REG1);
            entries[count].e2     = phoenix_read_reg( mmio,PHOENIX_TB_RDDATA_REG2);
            entries[count].e3     = phoenix_read_reg( mmio,PHOENIX_TB_RDDATA_REG3);
            entries[count].status = phoenix_read_reg( mmio,PHOENIX_TB_STATUS_REG);
        }
    }

    printf ("Samples (%d):\n", count);

    for (i = 0; i < count; i++) {
        unsigned long long addr = (((unsigned long long) entries[i].e1 << 8) | 
                                  (((unsigned long long) entries[i].e0 >> 29) << 5));
        unsigned int node_req   = (entries[i].e0 >> 16) & 0x1f;
        unsigned int snoop_res  = (entries[i].e0 >> 12) & 0x0f;
        printf ("addr=0x%010llx, node_req=0x%x, snoop_res=0x%x\n", addr, node_req, snoop_res);
    }

    free (entries);
    return 0;
}
