/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <error.h>
#include <string.h>
#include <sched.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <nlm_hal.h>
#include <signal.h>

unsigned int cpu_online_map;

extern int dom_id;

extern int fdt_get_cpu_online_map(void *fdt, unsigned int *map);
extern int fdt_get_shared_mem(void *fdt, unsigned long long *off, 
		unsigned long long *sz);
extern void *read_dtb_file(char *dtbfname);
extern int contig_memory_init(void *shmaddr, unsigned long long paddr, unsigned long long size);
extern int init_nae(void *fdt);
extern int process_fdt(void *fdt);
extern int nlm_fuse_init(void);
extern void nlm_dealloc_shmem(void);
void linuxu_app_preinit(void) __attribute__ ((constructor));
#define MAX_CPUS 32
#define BUFLEN  256

static int phys_to_log_map[MAX_CPUS];
	
static void set_phys_to_log_map(unsigned int map)
{
	int i, log_cpu = 0;
	for(i = 0; i < 32; i++) {
		if((1 << i) & map) {
			phys_to_log_map[i] = log_cpu;
			log_cpu++;
		}
	}
}
static int get_phys_to_log_map(int physcpu)
{
	return phys_to_log_map[physcpu];
}

static int app_set_affinity(int cpu)
{
	cpu_set_t mask;

	CPU_ZERO(&mask);
	CPU_SET(cpu, &mask);
	if(sched_setaffinity(0, sizeof(mask), &mask) != 0) {
		fprintf(stderr, "Set affinity failed for cpu %d\n", cpu);
		return -1;
	}
	return 0;
}

void linuxu_app_postexit(void)
{
	printf("pid@%d exiting\n", getpid());
	//kill(0, SIGKILL);
}

void linuxu_app_preinit(void)
{
	char *buf, *fname;
	unsigned int cpu_possible_map = 0, onlinemask = 0;
	int i, pid, memfd;
	void *fdt, *addr;
	unsigned long long shoff, shsize;
	int logical_cpu = 0;

	/* Register an atexit handler */
	if (atexit(linuxu_app_postexit)) {
		fprintf(stderr, "Error: Unable to register atexit hook![%s]\n", strerror(errno));
		goto err;
	}

	/* set default dom_id to 1, Linux is dom_id=0 */
	dom_id = 1;
	if((buf = getenv("lddomain")) == NULL) {
		fprintf(stderr, "Error : lddomain getenv failed\n");
		goto err;
	}
	dom_id = strtoul(buf, NULL, 10);
	printf("[ldapps]: domain id = %d\n", dom_id);

	/* Get the shared mem info */
	if((fdt = read_dtb_file(getenv("lddtbfname"))) == NULL) {
		fprintf(stderr, "Error : FDT file read failed\n");
		goto err;
	}

	/* Read cpu_online_map from fdt in corresponding domain node */
	if (fdt_get_cpu_online_map(fdt, &cpu_online_map) < 0) {
		fprintf(stderr, "Unable to parse onlinemask for domain@%d\n", dom_id);
		goto err;
	}

	if((buf = getenv("ldcpupossiblemap")) == NULL) {
		fprintf(stderr, "ldcpumap getenv failed\n");
		goto err;
	}
	cpu_possible_map = strtoul(buf, NULL, 16);
	set_phys_to_log_map(cpu_possible_map);

	/* Adjust cpu_online_map to cpu_possible_map */
	cpu_online_map = cpu_online_map & cpu_possible_map;

	onlinemask = cpu_online_map;

	printf("[ldapps]: cpu_possible_map = 0x%x, cpu_online_map = 0x%x, onlinemask=0x%x\n",
	       cpu_possible_map, cpu_online_map, onlinemask);
	if (!onlinemask) {
		fprintf(stderr, "invalid onlinemask=%x\n", onlinemask);
		goto err;
	}

	/* Set affinity of master/first/parent process to cpu@0
	 */
	for (i = 0; i < MAX_CPUS; i++) {
		if (onlinemask & (1 << i)) break;
	}
	logical_cpu = get_phys_to_log_map(i);
	if(app_set_affinity(logical_cpu) < 0)
		goto err;

	/* clear the bit corresponding to master cpu */
	onlinemask = onlinemask & ~(1 << i);

	if(fdt_get_shared_mem(fdt, &shoff, &shsize) != 0) {
		fprintf(stderr, "Shared mem params not defined, mempool does not work \n");
		goto err;
	} else {
		fprintf(stderr, "Shared mem params: shoff 0x%llx shsize 0x%llx\n", shoff, shsize);
        }
	

	if(shsize) {
		memfd = open("/dev/mem", O_RDWR);
		if (memfd == -1) {
			fprintf(stderr, "Error: cannot open /dev/mem - %s, mempool does not work\n",
					strerror(errno));
			goto err;
		} else {
			if((addr = mmap(NULL, shsize, PROT_READ | PROT_WRITE, MAP_SHARED, memfd, shoff)) == MAP_FAILED) {
				fprintf(stderr, "Error : mmap failed for the shared mem, mempool does not work : %s\n", strerror(errno));
				goto err;
			}
			else if(contig_memory_init(addr, shoff, shsize) != 0) {
				fprintf(stderr, "Error : Mempool init failed for the shared mem, mempool does not work \n");
				goto err;
			}
		}
	}

	if(process_fdt(fdt) < 0)
		goto err;

	/*Initialize HAL*/
	nlm_hal_init();

	/* Dealloc any shared mem of the prev run */
	nlm_dealloc_shmem();

	/* initialize the nae */
	if(init_nae(fdt) < 0)
		goto err;

	/* fuse deomon will run as child , All common initialization 
	 should be done before this */
	if (nlm_fuse_init() < 0)
		goto err;

	if((fname = getenv("ldfname")) == NULL) {
		fprintf(stderr, "ldfname getenv failed\n");
		goto err;
	}
	printf("[ldapps]: ldfname = %s\n", fname);

	if((buf = getenv("ldmode")) == NULL) {
		fprintf(stderr, "Error : ldmode getenv failed\n");
		goto err;
	}
	/* in smp mode, clone does the job */
	if(strcmp(buf, "smp") == 0) {
		printf("[ldapps]: Detected smpmode, skipping launching of children\n");
		goto done;
	}

	/* launch the children */
	printf("[ldapps]: Launching 0x%x children\n", onlinemask);
	for(i = 0; i < MAX_CPUS; i++) {

		if ((onlinemask & (1 << i)) == 0) continue;
		logical_cpu = get_phys_to_log_map(i);
		/* find the next available cpu */
		pid = fork();
		if (pid == 0) {
			/* Child */
			app_set_affinity(logical_cpu);
			printf("[ldapps]: cpu@%d, pid=%d running\n", i, (int)getpid());
			goto done;
		}
	}
done:
	return;
	//exit(0);
err:
	exit(-1);
}
