/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _NLM_UACCESS_H
#define _NLM_UACCESS_H

#include <nlm_hal_fmn.h>
#include <stdint.h>
#include <string.h>

#ifndef THREAD_SIZE
#define THREAD_SIZE (2048 * sizeof(long))
#endif

static inline int xlp_message_send_1(uint32_t dst, uint32_t  code, uint64_t data);
static inline int xlp_message_send_2(uint32_t dst, uint32_t  code, uint64_t data0, uint64_t data1);
static inline int xlp_message_send_3(uint32_t dst, uint32_t  code, uint64_t data0, uint64_t data1, uint64_t data2);
static inline int xlp_message_receive_2(uint32_t vc, uint32_t *src_id, uint32_t *size, uint32_t *code, uint64_t *msg0, uint64_t *msg1);
static inline int xlp_message_receive_1(uint32_t vc, uint32_t *src_id, uint32_t *size, uint32_t *code,	uint64_t *msg0);

static inline int xlp_message_pop_2(uint32_t vc, uint32_t src_id, uint32_t code, uint64_t data0, uint64_t data1);
static inline int xlp_message_pop_1(uint32_t vc, uint32_t src_id, uint32_t code, uint64_t data0);

static inline unsigned int nlm_uaccess_processor_id(void) 
{
	unsigned int val;
	__asm__ volatile(
			"syscall 14\n" 
			"move %0, $8\n"    /* val = t0 */
			:"=r"(val)
			:
			:"$8");

	return val;
}

static __inline__ uint32_t nlm_uaccess_mem_read32(uint64_t phys)
{
	uint32_t lsb, msb;
	uint32_t value;

	lsb = phys & (0xffffffffULL);
	msb = phys >> 32;

	__asm__ volatile(
		".set push\n"
		".set arch=xlp\n"
		".set noreorder\n"
		"move $8, %2\n"
		"move $9, %1\n"
		"syscall 21 \n"
		"move %0, $10\n"
		".set pop\n"
		:"=r"(value)
		:"r"(lsb),"r"(msb)
		:"$8", "$9", "$10"
	);
	return value;
}

static __inline__ void nlm_uaccess_mem_write32(uint64_t phys, uint32_t value)
{
	uint32_t lsb, msb;

	lsb = phys & (0xffffffffULL);
	msb = phys >> 32;

	__asm__ volatile(
		".set push\n"
		".set arch=xlp\n"
		".set noreorder\n"
		"move $8, %1\n"
		"move $9, %0\n"
		"move $10, %2\n"
		"syscall 22 \n"
		".set pop\n"
		:
		:"r"(lsb),"r"(msb), "r"(value)
		:"$8", "$9", "$10"
	);
	return;
}

static __inline__ uint64_t nlm_uaccess_mem_read64(uint64_t phys)
{
	uint32_t lsb, msb;
	uint64_t value;
	uint64_t lsb_val, msb_val;

	lsb = phys & (0xffffffffULL);
	msb = phys >> 32;

	__asm__ volatile(
		".set push\n"
		".set arch=xlp\n"
		".set noreorder\n"
		"move $8, %3\n"
		"move $9, %2\n"
		"syscall 19 \n"
		"move %0, $11\n"
		"move %1, $10\n"
		".set pop\n"
		:"=r"(lsb_val), "=r"(msb_val)
		:"r"(lsb),"r"(msb)
		:"$8", "$9", "$10", "$11"
	);
	value = ((uint64_t)msb_val << 32) | ((uint64_t)lsb_val);
	return value;
}


static __inline__ void nlm_uaccess_mem_write64(uint64_t phys, uint64_t value)
{
	uint32_t lsb, msb;
	uint32_t lsb_val, msb_val;

	lsb = phys & (0xffffffffULL);
	msb = phys >> 32;
	lsb_val = value & (0xffffffffULL);
	msb_val = value >> 32;
	__asm__ volatile(
		".set push\n"
		".set arch=xlp\n"
		".set noreorder\n"
		"move $8, %1\n"
		"move $9, %0\n"
		"move $10, %3\n"
		"move $11, %2\n"
		"syscall 20 \n"
		".set pop\n"
		:
		:"r"(lsb),"r"(msb), "r"(lsb_val), "r"(msb_val)
		:"$8", "$9", "$10", "$11"
	);
	return;
}

static inline int nlm_uaccess_msgsnd_3(uint32_t code, uint32_t dst, uint64_t data0, uint64_t data1, uint64_t data2)
{
	return xlp_message_send_3(dst, code, data0, data1, data2);
}

static inline int nlm_uaccess_msgsnd_2(uint32_t code, uint32_t dst, uint64_t data0, uint64_t data1)
{
	return xlp_message_send_2(dst, code, data0, data1);
}

static inline int nlm_uaccess_msgsnd_1(uint32_t code, uint32_t dst, uint64_t data0)
{
	return xlp_message_send_1(dst, code, data0);
}

static __inline__ int nlm_uaccess_msgrcv_2(uint32_t vc, 
		uint32_t *src_id, uint32_t *size, uint32_t *code,
		uint64_t *data0, uint64_t *data1)
{
  return xlp_message_receive_2(vc, src_id, size, code, data0, data1);
}

static inline int nlm_uaccess_msgrcv_1(uint32_t vc, uint32_t *src_id, uint32_t *size, 
		uint32_t *code,	uint64_t *data0)
{
	return xlp_message_receive_1(vc, src_id, size, code, data0);
}

static inline int nlm_pop_msg_2(uint32_t dst, uint32_t src, uint32_t code, uint64_t data0, uint64_t data1)
{
	return xlp_message_pop_2(dst, src, code, data0, data1);
}

static inline int nlm_pop_msg_1(uint32_t dst, uint32_t src, uint32_t code, uint64_t data0)
{
	return xlp_message_pop_1(dst, src, code, data0);
}
#endif
