/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/


/*

PKT_POOL_HEAD Node       pkt_list        pkt_list         pkt_list 
----------------|       |--------|      |--------|       |--------|
| first_free_pkt|-------|next_pkt|----->|next_pkt|------>|next_pkt|
|---------------|       |--------|      |--------|       }--------|
| last_free_pkt	|-----|                                       ^	
|---------------|     |                                       |	
                      |---------------------------------------|




*/
#include <stdio.h>
#include <stdlib.h>

#include "psb.h"
#include "packet.h"
// #include "cache.h"
#include "spinlock.h"
#include "system.h"
// #include "mips-exts.h"
#include "compiler.h"
#include "shmvars.h"


static struct misc_shm_vars *m = NULL;
typedef struct pkt_list PKT_LIST;
typedef struct pkt_pool_header PKT_POOL_HEAD;


struct pkt_list {
PKT_LIST * next_pkt;
};


struct  pkt_pool_header{
PKT_LIST * first_free_pkt;
PKT_LIST * last_free_pkt;
};

/*  ________________________________
    |				   |
    |		   pBuff    		   \/	
PKT_POOL_HEAD Node       pkt_list       pkt_list         pkt_list
-----------------       |--------|      |--------|       |--------|
| first_free_pkt|-\-\-\-|next_pkt|----->|next_pkt|------>|next_pkt|     |
|---------------|       |--------|      |--------|       }--------|
| last_free_pkt |-----|                                       ^
|---------------|     |                                       |
                      |---------------------------------------|


*/
	
struct packet *alloc_pbuf (void) 
{
	struct packet * pkt;

	if(m == NULL)
		return NULL;

	if ( m->pkt_pool_spinlock ==  NULL ){
		return NULL;
	}

	spin_lock(m->pkt_pool_spinlock);
	pkt = ( struct packet *)((PKT_POOL_HEAD *)m->pool_header)->first_free_pkt;

	if ( pkt ==  NULL ) {
		spin_unlock(m->pkt_pool_spinlock);
		return NULL;
	}
	
	((PKT_POOL_HEAD *)m->pool_header)->first_free_pkt = ((PKT_POOL_HEAD *)m->pool_header)->first_free_pkt->next_pkt;
	spin_unlock(m->pkt_pool_spinlock);
	return pkt;

}

/*
	 pBuf
        |--------|
     -->|next_pkt|-----
    |   |--------|    |
    |                 \/
 PKT_POOL_HEAD Node      pkt_list       pkt_list         pkt_list
----------------|       |--------|      |--------|       |--------|
| first_free_pkt|-\-\-\-|next_pkt|----->|next_pkt|------>|next_pkt|     |
|---------------|       |--------|      |--------|       }--------|
| last_free_pkt |-----|                                       ^
|---------------|     |                                       |
                      |---------------------------------------|


*/	
void free_pbuf (struct packet *pbuf)
{

	PKT_LIST * temp;

	if(m == NULL)
		return;

	spin_lock(m->pkt_pool_spinlock);

	if ( pbuf == NULL){
		spin_unlock(m->pkt_pool_spinlock);
		return;
	}

	/* Append the Free Buffer to head of the free pool list */ 
        temp = (PKT_LIST *)pbuf;
        temp->next_pkt = ((PKT_POOL_HEAD *)m->pool_header)->first_free_pkt;
        ((PKT_POOL_HEAD *)m->pool_header)->first_free_pkt = temp;

	spin_unlock(m->pkt_pool_spinlock);
	return;
}
void  packet_deinit( void )
{
	if(m == NULL)
		return;
	*(m->pkt_pool_inited) =  !0xdeadbeef; // reset the magic number
        m->pkt_pool_inited    = (int *)0xdeadbeef;  // reset the memory
	return ;	

}

int packet_init ( char * addr, uint64_t size )
{
	int    i;
	PKT_LIST * temp;
	int nchunks = 0;
	unsigned long int   address = (unsigned long int) (addr);
	unsigned long __flags = 0;

	if((m = nlm_misc_shm_init()) == NULL)
		return -1;

	if ( (addr == NULL) || (size == 0) ) {
		printf ("packet_init():Address Passed is invalid\n");
		return 0; 
	}

	__flags = acquire_print_lock();

	if(address & 0x1f){
		/* The address passed is not cachline aligned, make it cacheline aligned */
		address = (((address >> 5 ) << 5 ) + 32 );
		printf("The Address is not cacheline aligned.Defalting to the nearest cacheline address\n");
		addr =(char *)address;
	}

	if (sizeof(struct packet) & 0x1f) {
		printf ("size of struct packet (0x%x) is not a multiple of cacheline size\n", (unsigned int)sizeof(struct packet));
	}
	
	m->pkt_pool_inited = (int *) addr ;
	m->pkt_pool_spinlock  = ( spinlock_t *)( addr + 32 );
	m->pool_header = ( PKT_POOL_HEAD *) ( addr + 64 ) ;

	if (*(m->pkt_pool_inited) != 0xdeadbeef) {
		*(m->pkt_pool_inited) =  0xdeadbeef;
		spin_lock_init(m->pkt_pool_spinlock);
		spin_lock(m->pkt_pool_spinlock);

		release_print_lock(__flags);

		addr = (char *) (addr + 96 );
		size = size - 96 ; 

		if ( (unsigned long int )addr & 0x7ff )	{
			address = (unsigned long int) (addr);
			address = ((( address >> 11 ) << 11 )+ ( 2 << 10 ));
			size = size - ( address - (unsigned long int )addr );	
			addr = (char *)address;
		}

		nchunks = size/sizeof(struct packet);
		if ( nchunks > MAX_PACKETS )
			nchunks =  MAX_PACKETS;

		((PKT_POOL_HEAD *)m->pool_header)->first_free_pkt = ( PKT_LIST *)addr; 		
		temp = ((PKT_POOL_HEAD *)m->pool_header)->first_free_pkt;

		for (i = 0; i < nchunks; i++) {
			temp->next_pkt = ( PKT_LIST *) ((char *) temp  + sizeof( struct packet ) ); 
			temp = temp->next_pkt;
		}

		temp->next_pkt = NULL;
		((PKT_POOL_HEAD *)m->pool_header)->last_free_pkt =  temp;
		spin_unlock(m->pkt_pool_spinlock);
	}
	else	{

		release_print_lock(__flags);
	}
		printf("The number of packets  allocated is %d \n",nchunks);

	return nchunks;
}
