/***********************************************************************

Copyright 2003-2006 Raza Microelectronics, Inc.(RMI). All rights
reserved.

Use of this software shall be governed in all respects by the terms and
conditions of the RMI software license agreement ("SLA") that was
accepted by the user as a condition to opening the attached files. 

Without limiting the foregoing, use of this software in source and
binary code forms, with or without modification, and subject to all
other SLA terms and conditions, is permitted.

Any transfer or redistribution of the source code, with or without
modification, IS PROHIBITED, unless specifically allowed by the SLA.

Any transfer or redistribution of the binary code, with or without
modification, is permitted, provided that the following condition is
met:

Redistributions in binary form must reproduce the above copyright
notice, the SLA, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the distribution:


THIS SOFTWARE IS PROVIDED BY Raza Microelectronics, Inc. `AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL RMI OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

*****************************#RMI_3#***********************************/

ENTRY(_start)
STARTUP(crt0.o)
OUTPUT_FORMAT(OBJ_FORMAT)

#ifndef VMLINUX_SYMBOL
#define VMLINUX_SYMBOL(_sym_) _sym_
#endif		
		
#define NOTES								\
	.notes : AT(ADDR(.notes) - 0) {			\
		VMLINUX_SYMBOL(__start_notes) = .;			\
		*(.note.*)						\
		VMLINUX_SYMBOL(__stop_notes) = .;			\
	}
		
/* GROUP(-lc -lphoenix -lgcc) */
/* SEARCH_DIR(.) */

__DYNAMIC  =  0;

/*
 * Allocate the stack to be at the top of memory, since the stack
 * grows down
 */

/* PROVIDE (__stack = 0); */
/* PROVIDE (__global = 0); */

/*
 * Initalize some symbols to be zero so we can reference them in the
 * crt0 without core dumping. These functions are all optional, but
 * we do this so we can have our crt0 always use them if they exist. 
 * This is so BSPs work better when using the crt0 installed with gcc.
 * We have to initalize them twice, so we multiple object file
 * formats, as some prepend an underscore.
 */

PROVIDE (hardware_exit_hook = 0);
PROVIDE (hardware_hazard_hook = 0);
PROVIDE (hardware_init_hook = 0);
PROVIDE (software_init_hook = 0);
PROVIDE (bucket_sizes = 0)	; 
PROVIDE ( cc_table_cpu_0 = 0) 	; 
PROVIDE ( cc_table_cpu_1 = 0) 	; 
PROVIDE ( cc_table_cpu_2 = 0) 	; 
PROVIDE ( cc_table_cpu_3 = 0)	; 
PROVIDE ( cc_table_cpu_4 = 0) 	; 
PROVIDE ( cc_table_cpu_5 = 0)	; 
PROVIDE ( cc_table_cpu_6 = 0)	; 
PROVIDE ( cc_table_cpu_7 = 0)	; 
PROVIDE ( cc_table_xgs_0 = 0) ; 
PROVIDE ( cc_table_xgs_1 = 0) ; 
PROVIDE ( cc_table_gmac = 0) ; 
PROVIDE ( cc_table_dma = 0) ; 
PROVIDE ( cc_table_sec = 0) ; 

PROVIDE ( xls_bucket_sizes = 0)	; 
PROVIDE ( xls_cc_table_cpu_0 = 0) ; 
PROVIDE ( xls_cc_table_cpu_1 = 0) ; 
PROVIDE ( xls_cc_table_xgs_0 = 0) ; 
PROVIDE ( xls_cc_table_xgs_1 = 0) ; 
PROVIDE ( xls_cc_table_gmac0 = 0) ; 
PROVIDE ( xls_cc_table_gmac1 = 0) ; 
PROVIDE ( xls_cc_table_cmp = 0) ; 
PROVIDE ( xls_cc_table_dma = 0) ; 
PROVIDE ( xls_cc_table_sec = 0) ; 
PROVIDE ( xls_cc_table_pcie = 0) ; 

PHDRS
{
    headers PT_PHDR PHDRS ;
    text PT_LOAD FILEHDR PHDRS ;
    data PT_LOAD ;
	note PT_NOTE FLAGS(4);  /* R__ */
}
		
SECTIONS
{
    . = LOADADDR + SIZEOF_HEADERS;
    _loadaddr = .; 

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //                     Start of Read-Only Segment
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    .init : 
    {
	   PROVIDE(_start = .);
        *(.init)
    } :text

    .text : 
    {
        _ftext = . ;
        PROVIDE (eprol = .);
        _shim_reg = . ;
        *(.text)
        *(.text.*)
        *(.gnu.linkonce.t*)
        *(.mips16.fn.*)
        *(.mips16.call.*)
    }

    .fini : 
    {
        *(.fini)
    }

    PROVIDE (__etext = .);
    PROVIDE (_etext = .);
    PROVIDE (etext = .);

    .rel.sdata :
    {
        PROVIDE (__runtime_reloc_start = .);
        *(.rel.sdata)
        PROVIDE (__runtime_reloc_stop = .);
    }

	NOTES :text :note
	.dummy : { *(.dummy) } :text

    .rodata : 
    {
        *(.rdata)
        *(.rodata)
        *(.rodata.*)
        *(.gnu.linkonce.r*)
		PROVIDE (__modules_start = .);
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //       End of Read-Only Segment and Start of Data Segment
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    .data BLOCK(DATA_SEG_ALIGNMENT) : 
    {
    	_fdata = ABSOLUTE(.);

        *(.data)
        *(.data.*)
        *(.gnu.linkonce.d*)
        . = ALIGN(32);
        _gzip_start = ABSOLUTE(.);
        *(.gzip)
        . = ALIGN(4);
        _gzip_end = ABSOLUTE(.);

        PROVIDE(netos_symtab = .);
	PROVIDE(netos_symtab_string_t = .);

    } :data

    .eh_frame : 
    { 
        KEEP (*(.eh_frame)) 
    }

    .ctors :
    {
        /* gcc uses crtbegin.o to find the start of
           the constructors, so we make sure it is
           first.  Because this is a wildcard, it
           doesn't matter if the user does not
           actually link against crtbegin.o; the
           linker won't look for a file to match a
           wildcard.  The wildcard also means that it
           doesn't matter which directory crtbegin.o
           is in.  */

        KEEP (*crtbegin.o(.ctors))

        /* We don't want to include the .ctor section from
           from the crtend.o file until after the sorted ctors.
           The .ctor section from the crtend file contains the
           end of ctors marker and it must be last */

        KEEP (*(EXCLUDE_FILE (*crtend.o) .ctors))
        KEEP (*(SORT(.ctors.*)))
        KEEP (*(.ctors))
    }

    .dtors    :
    {
        KEEP (*crtbegin.o(.dtors))
        KEEP (*(EXCLUDE_FILE (*crtend.o) .dtors))
        KEEP (*(SORT(.dtors.*)))
        KEEP (*(.dtors))
    }

    /* GP register for Master CPU */
    // _gp = . + 0x8000; 
    _gp = ALIGN(16) + 0x7ff0; 
    __global = _gp;

    .sdata : 
    {
        *(.sdata)
        *(.sdata.*)
        *(.gnu.linkonce.s*)
    }

    . = ALIGN (8);

    .lit8 : 
    { 
        *(.lit8) 
    }
    .lit4 : 
    { 
        *(.lit4) 
    }

    . = ALIGN(4);

     _edata  =  .;
     PROVIDE (edata  =  .);

    /* will be freed after init */
//    . = ALIGN(PAGE_SIZE);         /* Init code and data */
    . = ALIGN(8);         /* Init code and data */
    __init_begin = .;
    .init.text : {
        _sinittext = .;
        *(.init.text)
        _einittext = .;
    }
    .init.data : { *(.init.data) }
    . = ALIGN(16);
    __setup_start = .;
    .init.setup : { *(.init.setup) }
    __setup_end = .;
		
     _fbss = .;
    .sbss : 
    {
        *(.sbss)
        *(.scommon)
    }
    .bss : 
    {
        _bss_start = . ;
        *(.bss)
        *(COMMON)
        . = ALIGN (32 / 8);

        /* Stack for the Master */
	
        . = ALIGN(8192); 
	
	
        PROVIDE(__stack = . + 12K - 64);

        . = . + 12K;

        __stack_base = .;

        /*Allocate stack (size 12K) for each cpu*/
        . = . + 12K*32; 

        __stack_end = .;
	
        . = . + 64;

        *(.thread.info)
        . = ALIGN (32);

        _end = .;
        PROVIDE (end = .);
        . = . + HEAPSIZE;
        PROVIDE(_endaddr = ABSOLUTE(.));
    }
		
    PROVIDE(_program_size = .);
		
    . = ALIGN(DATA_SEG_ALIGNMENT);

    /*  DWARF debug sections.
        Symbols in the DWARF debugging sections are relative to 
        the beginning of the section so we begin them at 0.  */

    /* DWARF 1 */
    .debug          0 : { *(.debug) }
    .line           0 : { *(.line) }

    /* GNU DWARF 1 extensions */
    .debug_srcinfo  0 : { *(.debug_srcinfo) }
    .debug_sfnames  0 : { *(.debug_sfnames) }

    /* DWARF 1.1 and DWARF 2 */
    .debug_aranges  0 : { *(.debug_aranges) }
    .debug_pubnames 0 : { *(.debug_pubnames) }

    /* DWARF 2 */
    .debug_info     0 : { *(.debug_info) }
    .debug_abbrev   0 : { *(.debug_abbrev) }
    .debug_line     0 : { *(.debug_line) }
    .debug_frame    0 : { *(.debug_frame) }
    .debug_str      0 : { *(.debug_str) }
    .debug_loc      0 : { *(.debug_loc) }
    .debug_macinfo  0 : { *(.debug_macinfo) }

    /* SGI/MIPS DWARF 2 extensions */
    .debug_weaknames 0 : { *(.debug_weaknames) }
    .debug_funcnames 0 : { *(.debug_funcnames) }
    .debug_typenames 0 : { *(.debug_typenames) }
    .debug_varnames  0 : { *(.debug_varnames) }
}
