#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static int str_size;
#define STR_SIZE_INC  100000
#define STR_TABLE_INC 10000

main(int argc,char *argv[])
{
	FILE *fin, *fout;

	if (argc < 3) {
		printf("Usage : %s <symbol-file> <output-c-file>\n",argv[0]);
		return (0);
	}

	fin = fopen(argv[1], "r");

	if (!fin) {
		printf("%s: input filename [%s] incorrect\n",argv[0], argv[1]);
		return (-1);
	}

	fout = fopen(argv[2], "w");
	if (!fout) {
		printf("%s: cannot open output file %s\n",argv[0], argv[2]);
		fclose(fin);
		return (-1);
	}

	read_and_generate_symtab(fin, fout);
	fclose(fin);
	fclose(fout);
	return 0;
}

struct symtab_entry { 
	unsigned long long pos;
	int off;
};
char *sym_string;

read_and_generate_symtab(FILE *fin, FILE *fout)
{
	char sym_str[1000];
	char sym_type[100];
	unsigned long long sym_addr;
	int count,len,i;
	struct symtab_entry *sym_table;
	int str_index = 0, index = 0;

	str_size = STR_SIZE_INC;

	sym_table = (struct symtab_entry *)malloc(STR_TABLE_INC*sizeof(struct
							       symtab_entry));
	sym_string = (char *)malloc(str_size);
	if (!sym_string || !sym_string) {
		printf("Cannot allocate memory\n");
		return;
	}

	while (1) { 

		count = fscanf(fin, "%llx %c %999s\n", &sym_addr, &sym_type, 
			       sym_str);
		if (count == EOF)
			break;
		if (count != 3) {
			if (count != EOF) {
				fgets(sym_str, 1000, fin);
			}
			continue;
		}
		if (sym_str[0] == '$')
			continue;

		len = strlen(sym_str);
		if (len > 1000)
			len = 1000;
		strncpy(&sym_string[str_index], sym_str, len);
		sym_string[str_index + len] = 0;
		sym_table[index].pos= sym_addr;
		sym_table[index++].off= str_index;
		str_index += len + 1;
		if (index % STR_TABLE_INC) {
			sym_table = realloc(sym_table, (STR_TABLE_INC + index) * 
					    sizeof(struct symtab_entry));
			if (!sym_table) {
				printf("Cannot allocate memory\n");
				return;
			}
		}
		if (str_index > str_size - 1000){
			sym_string = (char *)realloc(sym_string, STR_SIZE_INC +
						     str_index);
			if (!str_index) {
				printf("Cannot allocate memory\n");
				return;
			}
			str_size += STR_SIZE_INC;
		}
	}

	fprintf(fout,"struct netos_symtab_t {\n");
	fprintf(fout,"unsigned long long pos;\n");
	fprintf(fout,"unsigned int off;\n } netos_symtab[] = { \n");
	for (i = 0 ; i < index; i++) {
		fprintf(fout, "{0x%llxULL , %ld },\n", sym_table[i].pos, sym_table[i].off);
	}
	fprintf(fout,"};\n");

	fprintf(fout,"char netos_symtab_string_t[] ={\n");
	fprintf(fout,"\"");
	for (i = 0 ; i < index; i++) {
		fprintf(fout,"%s ",&sym_string[sym_table[i].off]);
	}
	fprintf(fout,"\"");
	fprintf(fout,"\n};\n");

}
