/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

/**************************************************************************/
/* script for parsing tracebuffer data generated by calling the function  */
/*                      trace(max_samples, trace_size)                    */
/**************************************************************************/
 
 #! /usr/bin/perl -w
 
 use FileHandle ;
 
 # Main
 {
    my $file = new FileHandle("$ARGV[0]") ;
 
    while (<$file>) {
         if ($_ =~ /\<(.*)\>/) {
             my @vals = split(/, /,$1) ;
             #print "$vals[0] $vals[1] $vals[2] $vals[3]" ;
             #print "\n" ;
             my $cmd     = ( (hex($vals[0]) >> 7) & 0x7) ;
             my $address = ( ( (hex($vals[3]) >> 29) |
                       (hex($vals[2]) << 3) ) << 5 ) ;
             
             my $snprslt  = ( (hex($vals[3]) >> 12) & 0xf) ;
             my $reqid    = ( (hex($vals[3]) & 0x1f) ) ;
             my $l2alloc  = ( (hex($vals[0]) >> 2) & 0x1) ;
             my $retry    = ( (hex($vals[3]) >> 9) & 0x07);
     
             if ($cmd == 1) {
                 $cmd = "WR" ;
             } elsif ($cmd == 2) {
                 $cmd = "RD" ;
             } elsif ($cmd == 3) {
                 $cmd = "RDEX" ;
             } elsif ($cmd == 4) {
                 $cmd = "UPG" ;
             } else {
                 $cmd = "????:".$cmd ;
             }
            
             if ($cmd ne "WR") {
                 if ($snprslt == 0x2) {
                     $snprslt = "CPU MOD" ;
                 } elsif ($snprslt == 0x3) {
                     $snprslt = "CPU MOD/SHR (OWN)" ;
                 } elsif ($snprslt == 0x1) {
                     if ($retry == 0) {
                         $snprslt = "CPU SHR" ;
                     }
                     else {
                         $snprslt = "RETRY" ;
                     }
                 } elsif ($snprslt == 8) {
                     $snprslt = "L2 MOD" ;
                 } elsif ($snprslt == 9) {
                     $snprslt = "L2 MOD/CPU SHR" ;
                 } elsif ($snprslt == 0) {
                     if ($retry == 0)  {
                         $snprslt = "DRAM/IO" ;
                     }
                     else {
                         $snprslt = "RETRY" ;
                     }
                 } elsif ($snprslt == 4) {
                     $snprslt = "L2 SHR" ;
                 } elsif ($snprslt == 5) {
                     $snprslt = "L2 SHR/CPU SHR" ;
                 } else {
                     $snprslt = "????:".$snprslt ;
                 }
             } 
            else {
                 if ($retry == 0)  {
                     if ($l2alloc == 1) {
                         $snprslt = "L2 WRITE" ;
                     } else {
                         $snprslt = "DRAM WRITE" ;
                     }
                 }
                 else {
                     $snprslt = "RETRY";
                 }
             }
            
             if ($reqid == 0) {
                 $reqid = "CPU_0";
             } elsif ($reqid == 1) {
                 $reqid = "CPU_1";
             } elsif ($reqid == 2) {
                 $reqid = "CPU_2";
             } elsif ($reqid == 3) {
                 $reqid = "CPU_3";
             } elsif ($reqid == 19) {
                 $reqid = "GMAC";
             } else {
                 $reqid = "????:".$reqid;
             }
    
             printf("node: %10s cmd: %4s snp: %20s, address: 0x%08x\n", \
                $reqid,$cmd,$snprslt,$address) ;
         }
    }
 }
