#ifndef _XLX_H
#define _XLX_H

#include <inttypes.h>
#include <sgidefs.h>

#ifdef NLM_ARCH_XLP
#define CRC_POLY_REG_0_SEL 0
#define CRC_POLY_REG_1_SEL 1
#define CRC_POLY_REG_2_SEL 2
#define CRC_POLY_REG_3_SEL 3

#define CRC_ENDIAN_BIT 5
#define CRC_FLIPBITS_BIT 4
#define CRC_DESTINATION_BIT 6

#define CRC_32_INIT_VALUE 0xffffffff
#define CRC_16_INIT_VALUE 0

typedef enum crc_type {
	NLM_CRC_32 = 0,
	NLM_CRC_16 = 16,
	NLM_CRC_7 = 25
} crc_type_t;

#define REG_STR(x) "$" #x

#undef USE_64BIT_CRC
#ifdef __KERNEL__
#define USE_64BIT_CRC
#else
#if _MIPS_SIM != _MIPS_SIM_ABI32 && !defined(ABI_32)
#define USE_64BIT_CRC
#endif
#endif /* __KERNEL__ */

#ifdef USE_64BIT_CRC
typedef uint64_t u_data;
#else
typedef uint32_t u_data;
#endif

#define INIT_CRC_POLY(poly_reg, poly_type, crc_poly) \
({                                                            \
    asm volatile (                                            \
        ".set push\n"                                         \
        ".set noat\n"                                         \
	"dmtur %0, " REG_STR(poly_reg) "\n"                   \
        ".set pop\n"                                          \
	: : [poly] "r"((crc_poly << poly_type) & 0x00000000ffffffff)  \
    );                                                        \
})

#ifdef USE_64BIT_CRC
static inline unsigned int
nlm_crc32_generic(uint64_t data, unsigned int flags_len, unsigned int crc)
{
	unsigned int ret;
	asm volatile (
		".set push\n"
		".set noreorder\n"
		"addiu $9, %[flags_len], 0\n"
		"dcrc   %[result], %[input], $9\n"
		".set pop\n"
		: [result] "=r"(ret)
		: [input] "r"(data), [flags_len] "r"(flags_len), "0"(crc)
		: "$9"
	);
	return ret;
}
#else
static inline unsigned int
nlm_crc32_generic(unsigned int data, unsigned int flags_len, unsigned int crc)
{
	unsigned int ret;
	asm volatile (
		".set push\n"
		".set noreorder\n"
		"addiu $9, %[flags_len], 0\n"
		"crc   %[result], %[input], $9\n"
		".set pop\n"
		: [result] "=r"(ret)
		: [input] "r"(data), [flags_len] "r"(flags_len), "0"(crc)
		: "$9"
	);
	return ret;
}
#endif

static inline uint32_t
bit_flip(uint32_t t)
{
	unsigned int ret = 0, x = t;

    asm volatile (
	    ".set push\n"
	    ".set mips32\n"
	    ".set noreorder\n"
	    "addiu   %[ret], $0, 0\n"
	 "1: addiu   $9, $0, 32\n"
	    "clz     $8, %[val]\n"
	    "beq     $8, $9, 3f\n"

	    // set the bit in the correct location of result
	    "addiu   $9, $0, 1\n"
	    "sllv    $9, $9, $8\n"
	    "or      %[ret], %[ret], $9\n"

	    // reset the bit in the data
	    "addiu   $9, $0, 31\n"
	    "subu    $9, $9, $8\n"
	    "addiu   $8, $0, 1\n"
	    "sllv    $9, $8, $9\n"
	    "xor     %[val], %[val], $9\n"
	    "b       1b\n"
	    ".set pop\n"
	    "3: nop\n"
	    : [ret] "=&r"(ret) : [val] "r"(x)
	    :"$8", "$9"
	    );
    return ret;
}

static inline unsigned int
nlm_crc32_word(int crc_reg, u_data data, unsigned int len,
	   unsigned int crc_init)
{
	/* flip bit */
	unsigned int flags_len = (1 << CRC_FLIPBITS_BIT) | (1 << CRC_DESTINATION_BIT);

	/* set length and CRC poly reg */
	flags_len |= ((len & 0x7) |
		      ((crc_reg & 0x3) << 8));

	return nlm_crc32_generic(data, flags_len, crc_init);
}

static inline uint32_t
nlm_crc16_ibm(int crc_reg, u_data data, unsigned int len,
	  unsigned short crc_init)
{
	/* flip bit */
	unsigned int flags_len = (1 << CRC_FLIPBITS_BIT) | (1 << CRC_DESTINATION_BIT);

	/* set length and CRC poly reg */
	flags_len |= ((len & 0x7) |
		      ((crc_reg & 0x3) << 8));

	return  nlm_crc32_generic(data, flags_len, crc_init);
}

static inline uint32_t
nlm_crc7_word(int crc_reg, u_data data, unsigned int len, unsigned short crc_init)
{
	/* flip bit */
	unsigned int flags_len = (1 << CRC_FLIPBITS_BIT) | (1 << CRC_DESTINATION_BIT);

	/* set length and CRC poly reg */
	flags_len |= ((len & 0x7) |
		      ((crc_reg & 0x3) << 8));

	return  nlm_crc32_generic(data, flags_len, crc_init);
}

/**
 * This macro loops on a string of data.  It is kept this way to
 * adjust to endianess and use of 64bit instruction, so that each
 * CRC function does not require ifdef.
 */
#ifdef USE_64BIT_CRC
#define LOOP_ON_DATA(c, init, b, len, f) \
({                                                                           \
        uint64_t data;                                                       \
	uint32_t i, rem = len, __ret = init, l;                              \
	for(i = 0; rem > 0;) {                                               \
                data = ((uint64_t *)buf)[i];                                 \
      	        i += 8;                                                      \
	        if(rem >= 8) {                                               \
		        rem -= 8;                                            \
		        l = 7;                                               \
		} else if (rem == 7) {                                       \
			rem -= 7;                                            \
			l = 6;                                               \
		} else if (rem == 6) {                                       \
			rem -= 6;                                            \
			l = 5;                                               \
		} else if (rem == 5) {                                       \
			rem -= 5;                                            \
			l = 4;                                               \
		} else if (rem == 4) {                                       \
			rem -= 4;                                            \
			l = 3;                                               \
		} else if (rem == 3) {                                       \
			rem -= 3;                                            \
			l = 2;                                               \
		} else if (rem == 2) {                                       \
			rem -= 2;                                            \
			l = 1;                                               \
		} else if (rem == 1) {                                       \
			rem -= 1;                                            \
			l = 0;                                               \
		}                                                            \
		__ret = f(crc_reg, data, l, __ret);                          \
	}                                                                    \
        __ret;                                                               \
})
#else
#define LOOP_ON_DATA(c, init, b, len, f) \
({                                                                           \
        uint32_t data;                                                       \
	uint32_t i, rem = len, __ret = init, l;                              \
	for(i = 0; rem > 0;) {                                               \
		data = ((uint32_t *)buf)[i];                                 \
                i += 4;                                                      \
		if(rem >= 4) {                                               \
			rem -= 4;                                            \
			l = 3;                                               \
		} else if (rem == 3) {                                       \
			rem -= 3;                                            \
			l = 2;                                               \
		} else if (rem == 2) {                                       \
			rem -= 2;                                            \
			l = 1;                                               \
		} else if (rem == 1) {                                       \
			rem -= 1;                                            \
			l = 0;                                               \
		}                                                            \
		__ret = f(crc_reg, data, l, __ret);                          \
	}                                                                    \
        __ret;                                                               \
})
#endif

static inline uint32_t
nlm_crc32(int crc_reg, const unsigned char *buf, unsigned int len, unsigned int crc)
{
	return LOOP_ON_DATA(crc_reg, crc,
			    buf, len, nlm_crc32_word) ^ 0xffffffff;
}

static inline uint16_t
nlm_crc16(int crc_reg, const unsigned char *buf, unsigned int len, unsigned short crc)
{
	return (uint16_t)(LOOP_ON_DATA(crc_reg, crc,
				       buf, len, nlm_crc16_ibm) & 0xffff);
}

static inline unsigned char
nlm_crc7(int crc_reg, const unsigned char *buf, unsigned int len, unsigned char crc)
{
	return (unsigned char)(LOOP_ON_DATA(crc_reg, crc, buf,
					    len, nlm_crc7_word) & 0x3f);
}

#endif /* NLM_ARCH_XLP */

#endif /* _XLX_H */
