/***********************************************************************
Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/

#include <stdio.h>

#include "fast_syscall.h"

/*
 * Get the status of the message ring and the c0 counter
 */
void fs_c0_count(unsigned int *counter)
{
        __asm__ volatile (
		"syscall "STR(FS_C0_COUNT)"\n"
		"move %0, $8\n"      /* *counter = t0 */
		: "=r"(*counter)
		: 
		:"$8"
		);
}
/*
 * Read from the i/o memory
 */
unsigned int fs_read(unsigned int addr)
{
	unsigned int val;
	__asm__ volatile(
		"move $8, %1\n"    /* t0 = addr */
		"syscall "STR(FS_MEM_READ)"\n"
		"move %0, $9\n"           /* val = t1 */
		:"=r"(val)
		:"r"(addr)
		:"$8", "$9");

	return val;
}
/*
 * Write to the i/o memory
 */
void fs_write(unsigned int addr, unsigned int val)
{
	__asm__ volatile(
		"move $8, %1\n"    /* t0 = addr */
		"move $9, %0\n"           /* t1 = val */
		"syscall "STR(FS_MEM_WRITE)"\n"
		::"r"(val), "r"(addr)
		:"$8", "$9");
}

/*
 * Disable the message ring interrupts and enable 64 bits operations
 */
void fs_msg_write(void)
{
	__asm__ volatile (
		"syscall "STR(FS_MSGINT)"\n"
		::);
}

/*
 * Read cop0 and cop2 registers
 */
void fs_msg_read(unsigned int *c0_status, unsigned int *c2_status_0,
			       unsigned int *c2_status_1, unsigned int *c2_config_0,
			       unsigned int *c2_config_1)
{
	__asm__ volatile (
		"syscall "STR(FS_READ_COP)"\n"
		"move %0, $8\n"           /* *c0_status = t0 */
		"move %1, $9\n"           /* *c2_status_0 = t1 */
		"move %2, $10\n"          /* *c2_status_1 = t2 */
		"move %3, $11\n"          /* *c2_config_0 = t3 */
		"move %4, $12\n"          /* *c2_config_1 = t4 */
		:"=r" (*c0_status), "=r"(*c2_status_0), "=r"(*c2_status_1),
		 "=r"(*c2_config_0), "=r"(*c2_config_1)
		::"$8", "$9", "$10", "$11", "$12");
}

void fs_perf_ctr_start(int ctr1, int ctr2, int thread, int global)
{
	unsigned int val0, val1;

	val0 = ( ((ctr1) << 5) | 0x0000000f);
	val1 = ( ((ctr2) << 5) | 0x0000000f);

#if defined(NLM_ARCH_XLP)
	/* Enable 64-bit counters */
	val0 = ( val0 | (1 << 30) | ((global & 0x01) << 25));
	val1 = ( val1 | (1 << 30) | ((global & 0x01) << 25));
#else
	val0 = ( val0 | (1 << 30) | ((global & 0x01) << 13) | ((thread & 3) << 11) );
	val1 = ( val1 | (1 << 30) | ((global & 0x01) << 13) | ((thread & 3) << 11) );
#endif

	__asm__ volatile (
		"move $8, %0\n"       /* t0 = val0 */
		"move $9, %1\n"       /* t1 = val1 */
		"syscall "STR(FS_PERFCTR_START)"\n"
		:
		: "r"(val0), "r"(val1)
		: "$8", "$9"
		);
}

/*
 * Stop the performance counters
 */
void fs_perf_ctr_stop(uint64_t *val0, uint64_t *val1)
{
	unsigned int val;
	unsigned int e0_hi, e0_lo, e1_hi, e1_lo;

	val = 0x80002000;
	__asm__ volatile (
		"move $8, %4\n"       /* t0 = 0x80002000 */
		"syscall "STR(FS_PERFCTR_STOP)"\n"
		"move %0, $8\n"
		"move %1, $9\n"
		"move %2, $10\n"
		"move %3, $11\n"
		: "=r"(e0_lo), "=r"(e0_hi), "=r"(e1_lo), "=r"(e1_hi)
		:  "r"(val)
		:"$8", "$9", "$10", "$11"
		);

#if defined(NLM_ARCH_XLR)
	/* XLR doesn't have 64-bit counters */
	e0_hi = 0;
	e1_hi = 0;
#endif

	*val0 = ((uint64_t)e0_hi << 32) | ((uint64_t)e0_lo);
	*val1 = ((uint64_t)e1_hi << 32) | ((uint64_t)e1_lo);
}

void fs_get_cpumasks(struct cpumasks *_cpumasks)
{
	unsigned int cpumask_phy;
	unsigned int cpuid_hard;
	int i, num_cpus;

	__asm__ volatile (
		"syscall "STR(FS_READ_CPUMASKS)"\n"
		"move %0, $8\n"
		"move %1, $9\n"
		: "=r"(cpuid_hard), "=r"(cpumask_phy)
		:
		: "$8", "$9"
		);

	_cpumasks->cpumask_phy = cpumask_phy;
	_cpumasks->cpuid_hard = cpuid_hard;

	for(i = num_cpus = 0; i < NR_CPUS; i++) {
		if ((1 << i) & cpumask_phy) {
			_cpumasks->cpu_number_map[i] = num_cpus;
			_cpumasks->cpu_logical_map[num_cpus] = i;
			++num_cpus;
		}
	}

	_cpumasks->cpuid_smp = _cpumasks->cpu_number_map[cpuid_hard];

	return;
}

unsigned int fs_processorId(void)
{
	unsigned int val;
	__asm__ volatile(
		"syscall "STR(FS_READ_PROCID)"\n"
		"move %0, $9\n"    /* val = t1 */
		:"=r"(val)
		:
		:"$9");

	return val;
}

int fs_is_big_endian(void)
{
	int big_endian = 0;
	__asm__ volatile (
		"syscall "STR(FS_ENDIANESS)"\n"
		"addiu %[be], $8, 0\n"
		:[be]"=r"(big_endian)
		:
		:"$8"
		);
	return !!big_endian;
}

int fs_is_endian_reversed(void)
{
	int endian_reversed = 0;
	__asm__ volatile (
		"syscall "STR(FS_REVERSE_ENDIANESS)"\n"
		"addiu %[er], $8, 0\n"
		:[er]"=r"(endian_reversed)
		:
		:"$8"
		);
	return !!endian_reversed;
}

int fs_uspace_64bit_ins_enabled(void)
{
	int sb_en = 0;
	__asm__ volatile (
		"syscall "STR(FS_USPACE_64BIT_INS)"\n"
		"addiu %[sb], $8, 0\n"
		:[sb]"=r"(sb_en)
		:
		:"$8"
		);
	return !!sb_en;
}

uint64_t fs_cpu_max_freq(void)
{
	uint32_t hi, lo;
	__asm__ volatile (
		"syscall "STR(FS_CPU_MAX_FREQ)"\n"
		"move %[hi], $9\n"
		"move %[lo], $8\n"
		:[hi]"=r"(hi), [lo]"=r"(lo)
		:
		:"$8", "$9"
		);
	return ((((uint64_t)hi) << 32) | (lo & 0xffffffff));

}

uint64_t fs_read_timer(void)
{
	uint32_t hi = 0, lo = 0;
	uint64_t counter = 0;
	const uint64_t maxval = 0xffffffffffffffffULL;

	__asm__ volatile (
		"syscall "STR(FS_READ_TIMER)"\n"
		"move %0, $8\n"
		"move %1, $9\n"
		: "=r"(lo), "=r"(hi)
		:
		: "$8", "$9"
		);

	counter = ((uint64_t)hi << 32) | ((uint64_t)lo & 0xffffffff);
/* 	printf("[%s]: (hi=0x%08x, lo=0x%08x), counter = 0x%016llx, elapsed cycles=0x%016llx\n", */
/* 	       __FUNCTION__, */
/* 	       hi, lo, counter, maxval - counter); */

	return maxval - counter;
}

int fs_hard_cpuid(void)
{
	unsigned int cpuid = 0;

	__asm__ volatile(
		"syscall "STR(FS_HARD_CPUID)"\n"
		"move %0, $8\n"    /* val = t0 */
		:"=r"(cpuid)
		:
		:"$8");

	return cpuid;
}

#if defined(NLM_ARCH_XLP)

uint32_t fs_msgsnd3(uint32_t size,
		    uint32_t code,
		    uint32_t dest,
		    uint64_t msg0,
		    uint64_t msg1,
		    uint64_t msg2)
{
	uint32_t pass, arg0, arg1, arg2, arg3, arg4, arg5;

	/* printf("size=%d code=%d dest=%d\n\tmsg0=0x%016llx msg1=0x%016llx\n",
	   size, code, dest, msg0, msg1); */

	arg0 = (uint32_t)(msg0 >> 32);
	arg1 = (uint32_t)(msg0);
	arg2 = (uint32_t)(msg1 >> 32);
	arg3 = (uint32_t)(msg1);
	arg4 = (uint32_t)(msg2 >> 32);
	arg5 = (uint32_t) (msg2);

	__asm__ volatile (
		"move $8, %1\n"  /* t0 = ((code << 24) | (size-1) << 16) | dest */
		"move $9, %2\n"  /* t1 = arg0 */
		"move $10, %3\n" /* t2 = arg1 */
		"move $11, %4\n" /* t3 = arg2 */
		"move $12, %5\n" /* t4 = arg3 */
		"move $13, %6\n" /* t3 = arg4 */
		"move $14, %7\n" /* t4 = arg5 */
		"syscall "STR(FS_MSGSND3)"\n"
		"move %0, $9\n"  /* pass = t1 */
		: "=r"(pass)
		: "r"(((code << 24) | (size-1) << 16 | dest)), "r"(arg0), "r"(arg1), "r"(arg2), "r"(arg3),"r"(arg4),"r"(arg5)
		: "$8", "$9", "$10", "$11", "$12", "$13","$14"
		);

	return !pass;
}


uint32_t fs_msgsnd(uint32_t size,
		   uint32_t code,
		   uint32_t dest,
		   uint32_t vc,
		   uint64_t msg0,
		   uint64_t msg1)
{
	uint32_t pass, arg0, arg1, arg2, arg3;

	/* printf("size=%d code=%d dest=%d\n\tmsg0=0x%016llx msg1=0x%016llx\n", */
	/*         size, code, dest, msg0, msg1); */

	arg0 = (uint32_t)(msg0 >> 32);
	arg1 = (uint32_t)(msg0);
	arg2 = (uint32_t)(msg1 >> 32);
	arg3 = (uint32_t)(msg1);

	__asm__ volatile (
		"move $8, %1\n"  /* t0 = ((code << 24) | (vc << 19) | (size-1) << 16) | dest */
		"move $9, %2\n"  /* t1 = arg0 */
		"move $10, %3\n" /* t2 = arg1 */
		"move $11, %4\n" /* t3 = arg2 */
		"move $12, %5\n" /* t4 = arg3 */
		"syscall "STR(FS_MSGSND)"\n"
		"move %0, $9\n"  /* pass = t1 */
		: "=r"(pass)
		: "r"(((code << 24) | (vc << 19)| (size-1) << 16 | dest)), "r"(arg0), "r"(arg1), "r"(arg2), "r"(arg3)
		: "$8", "$9", "$10", "$11", "$12"
		);

	return !pass;
}

uint32_t fs_msgrcv(uint32_t vc,
		   uint32_t *size,
		   uint32_t *code,
		   uint32_t *src,
		   uint64_t *msg0,
		   uint64_t *msg1)
{
	uint32_t pass, status, arg0, arg1, arg2, arg3;

	/* printf("vc=%d\n", vc); */

	__asm__ volatile (
		"move $8, %6\n"  /* t0 = vc */
		"syscall "STR(FS_MSGRCV)"\n"
		"move %0, $8\n"  /* pass = t0 */
		"move %1, $9\n"  /* arg0 = t1 */
		"move %2, $10\n" /* arg1 = t2 */
		"move %3, $11\n" /* status = t3 */
		"move %4, $12\n" /* arg2 = t4 */
		"move %5, $13\n" /* arg3 = t5 */
		: "=r"(pass), "=r"(arg0), "=r"(arg1), "=r"(status), "=r"(arg2), "=r"(arg3)
		: "r"(vc)
		: "$8", "$9", "$10", "$11", "$12", "$13"
		);

	if (!pass) {
		return 1;
	}

	*size = ((status >> 26) & 0x3) + 1;
	*code = (status >> 18) & 0xff;
	*src  = (status >> 4) & 0xfff;
	*msg0 = (((uint64_t)arg0) << 32) | arg1;
	*msg1 = (((uint64_t)arg2) << 32) | arg3;

	return 0;
}

uint32_t fs_msgrcv1(uint32_t vc,
		    uint32_t *size,
		    uint32_t *code,
		    uint32_t *src,
		    uint64_t *msg0)
{
	uint32_t pass, status, arg0, arg1;

	/* printf("vc=%d\n", vc); */

	__asm__ volatile (
		"move $8, %4\n"  /* t0 = vc */
		"syscall "STR(FS_MSGRCV1)"\n"
		"move %0, $8\n"  /* pass = t0 */
		"move %1, $9\n"  /* arg0 = t1 */
		"move %2, $10\n" /* arg1 = t2 */
		"move %3, $11\n" /* status = t3 */
		: "=r"(pass), "=r"(arg0), "=r"(arg1), "=r"(status) 
		: "r"(vc)
		: "$8", "$9", "$10", "$11" 
		);


	if (!pass) {
		return !pass;
	}

	/* printf("arg0 %x arg1 %x\n", arg0, arg1); */

	*size = ((status >> 26) & 0x3) + 1;
	*code = (status >> 18) & 0xff;
	*src  = (status >> 4) & 0xfff;
	*msg0 = (((uint64_t)arg0) << 32) | arg1;

	return 0;
}

#else /* NLM_ARCH_XLP */

/*
 * Send to the message ring
 */
unsigned int fs_msgsnd(unsigned int size, unsigned int code,
		       unsigned int stid, unsigned long long msg0,
		       unsigned long long msg1)
{
	unsigned int status,
		arg0,
		arg1,
		arg2,
		arg3;;
	arg0 = (unsigned int)(msg0>>32);
	arg1 = (unsigned int)(msg0);
	arg2 = (unsigned int)(msg1>>32);
	arg3 = (unsigned int)(msg1);
	__asm__ volatile (
		"move $8, %1\n"       /* t0 = (((size-1)<<16)|(code<<8)|stid */
		"move $9, %2\n"       /* t1 = arg0 */
		"move $10, %3\n"      /* t2 = arg1 */
		"move $11, %4\n"      /* t3 = arg2 */
		"move $12, %5\n"      /* t4 = arg3 */
		"syscall "STR(FS_MSGSND)"\n"
		"move %0, $9\n"       /* status = t1 */
		: "=r"(status)
		: "r"(((size-1)<<16)|(code<<8)|stid), "r"(arg0), "r"(arg1), "r"(arg2), "r"(arg3)
		:"$8", "$9", "$10", "$11", "$12"
		);
	return status & 0x6;
}
/*
 * Receive from the message ring
 */
int fs_msgrcv(int pri, int *size, int *code, int *src_id,
			    unsigned long long *msg0)
{
	unsigned int result,
		status,
		arg0,
		arg1,
		arg2,
		arg3;
	__asm__ volatile (
		"move $8, %6\n"       /* t0 = pri */
		"syscall "STR(FS_MSGRCV)"\n"
		"move %0, $8\n"       /* result = t0 */
		"move %1, $9\n"       /* arg0 = t1 */
		"move %2, $10\n"      /* arg1 = t2 */
		"move %3, $12\n"      /* arg2 = t4 */
		"move %4, $13\n"      /* arg3 = t5 */
		"move %5, $11\n"      /* status = t3 */
		: "=r"(result), "=r"(arg0), "=r"(arg1), "=r"(arg2), "=r"(arg3), "=r"(status)
		: "r"(pri)
		:"$8", "$9", "$10", "$11", "$12", "$13"
		);
	if (!result) {
		*size=((status & 0xc0)>>6)+1;
		*code=(status & 0xff00)>>8;
		*src_id=(status & 0x7f0000)>>16;
		*msg0=(((unsigned long long)arg0)<<32)|arg1;
		if (*size > 1)
			*msg0=(((unsigned long long)arg2)<<32)|arg3;
		return 0;
	}
	return result;
}

#endif /* NLM_ARCH_XLP */

uint64_t fs_mem_read64(uint64_t addr)
{
	uint32_t addr0, addr1, val0, val1;

	addr0 = (uint32_t)(addr >> 32);
	addr1 = (uint32_t)(addr);

	/* printf("addr=0x%016llx (%08x,%08x)\n", addr, addr0, addr1); */

	__asm__ volatile (
		"move $8, %2\n"  /* t0 = addr0 */
		"move $9, %3\n"  /* t1 = addr1 */
		"syscall "STR(FS_MEM_READ64)"\n"
		"move %0, $10\n" /* val0 = t2 */
		"move %1, $11\n" /* val1 = t3 */
		: "=r"(val0), "=r"(val1)
		: "r"(addr0), "r"(addr1)
		: "$8", "$9", "$10", "$11"
		);

	return (((uint64_t)val0) << 32) | val1;
}

void fs_mem_write64(uint64_t addr, uint64_t val)
{
	uint32_t addr0, addr1, val0, val1;

	addr0 = (uint32_t)(addr >> 32);
	addr1 = (uint32_t)(addr);
	val0 = (uint32_t)(val >> 32);
	val1 = (uint32_t)(val);

	/* printf("addr=0x%016llx (%08x,%08x)\n\tval=0x%016llx (%08x,%08x)\n", */
	/*         addr, addr0, addr1, val, val0, val1); */

	__asm__ volatile (
		"move $8, %2\n"  /* t0 = addr0 */
		"move $9, %3\n"  /* t1 = addr1 */
		"move $10, %0\n" /* t2 = val0 */
		"move $11, %1\n" /* t3 = val1 */
		"syscall "STR(FS_MEM_WRITE64)"\n"
		:
		: "r"(val0), "r"(val1), "r"(addr0), "r"(addr1)
		: "$8", "$9", "$10", "$11"
		);
}

uint32_t fs_mem_read32(uint64_t addr)
{
	uint32_t addr0, addr1, val;

	addr0 = (uint32_t)(addr >> 32);
	addr1 = (uint32_t)(addr);

	/* printf("addr=0x%016llx (%08x,%08x)\n", addr, addr0, addr1); */

	__asm__ volatile (
		"move $8, %1\n"  /* t0 = addr0 */
		"move $9, %2\n"  /* t1 = addr1 */
		"syscall "STR(FS_MEM_READ32)"\n"
		"move %0, $10\n" /* val = t2 */
		: "=r"(val)
		: "r"(addr0), "r"(addr1)
		: "$8", "$9", "$10"
		);

	return val;
}

void fs_mem_write32(uint64_t addr, uint32_t val)
{
	uint32_t addr0, addr1;

	addr0 = (uint32_t)(addr >> 32);
	addr1 = (uint32_t)(addr);

	/* printf("addr=0x%016llx (%08x,%08x)\n\tval=0x%08x\n", */
	/*        addr, addr0, addr1, val); */

	__asm__ volatile (
		"move $8, %1\n"  /* t0 = addr0 */
		"move $9, %2\n"  /* t1 = addr1 */
		"move $10, %0\n" /* t2 = val */
		"syscall "STR(FS_MEM_WRITE32)"\n"
		:
		: "r"(val), "r"(addr0), "r"(addr1)
		: "$8", "$9", "$10"
		);
}
