/***********************************************************************
Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/

#ifndef _FAST_SYSCALL_H
#define _FAST_SYSCALL_H

#include <inttypes.h>

#ifndef __STR
#define __STR(x) #x
#endif
#ifndef STR
#define STR(x) __STR(x)
#endif

#define FS_UNUSED             0
#define FS_MSGSND             1
#define FS_MSGRCV             2
#define FS_C0_COUNT           3
#define FS_MEM_READ           4
#define FS_MEM_WRITE          5
#define FS_MSGINT             6
#define FS_READ_COP           7
#define FS_PERFCTR_START      8
#define FS_PERFCTR_STOP       9
#define FS_READ_CPUMASKS      10
#define FS_READ_PROCID        11
#define FS_PROMINFO           12
#define FS_READ_TIMER         13
#define FS_HARD_CPUID         14
#define FS_ENDIANESS          15
#define FS_REVERSE_ENDIANESS  16
#define FS_USPACE_64BIT_INS   17
#define FS_CPU_MAX_FREQ       18
#define FS_MEM_READ64         19
#define FS_MEM_WRITE64        20
#define FS_MEM_READ32         21
#define FS_MEM_WRITE32        22
#define FS_MSGSND3            23
#define FS_MSGRCV1            24

extern unsigned int fs_processorId(void);

extern void fs_perf_ctr_start(int ctr1, int ctr2, int thread, int global);
extern void fs_perf_ctr_stop(uint64_t *val0, uint64_t *val1);

extern int fs_is_big_endian(void);
extern int fs_is_endian_reversed(void);
extern int fs_uspace_64bit_ins_enabled(void);

extern uint64_t fs_cpu_max_freq(void);
extern uint64_t fs_read_timer(void);

#define NR_CPUS 32
struct cpumasks {
    unsigned int cpumask_phy;
    unsigned int cpu_number_map[NR_CPUS];
    unsigned int cpu_logical_map[NR_CPUS];
    unsigned int cpuid_hard;
    unsigned int cpuid_smp;
};
extern void fs_get_cpumasks(struct cpumasks *_cpumasks);

#endif /* _FAST_SYSCALL_H */
