#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sched.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <sgidefs.h>

#include "nlm_soclib.h"
#include "fast_syscall.h"
#include "perfctr.h"

static struct nlm_soclib_perfctr env_perfctr = {
	.ctr[0] = PERFCTR_INSN,
	.ctr[1] = PERFCTR_CYC,
	.thread = 0,
	.global = PERFCTR_GLOBAL,
	.ctr_val[0] = 0,
	.ctr_val[1] = 0
};

static const char *env_perfctr_strings[] = {
	[0] = "NLM_PERFCTR_CTR0",
	[1] = "NLM_PERFCTR_CTR1",
	[2] = "NLM_PERFCTR_THREAD",
	[3] = "NLM_PERFCTR_GLOBAL",
	[4] = "NLM_PERFCTR"
};
#define NUM_ENV_PERFCTR_STRINGS (sizeof(env_perfctr_strings)/sizeof(env_perfctr_strings[0]))

void perfctr_env_init(void)
{
	int i = 0;
	const char *name, *value;

	for (i=0;i<NUM_ENV_PERFCTR_STRINGS;i++) {
		name = env_perfctr_strings[i];
		value = getenv(name);
		if (!value) {
			continue;
		}
		if (strcasecmp(name, "NLM_PERFCTR_CTR0") == 0)
			env_perfctr.ctr[0] = strtoul(value, 0, 10);
		else if (strcasecmp(name, "NLM_PERFCTR_CTR1") == 0)
			env_perfctr.ctr[1] = strtoul(value, 0, 10);
		else if (strcasecmp(name, "NLM_PERFCTR_THREAD") == 0)
			env_perfctr.thread = strtoul(value, 0, 10);
		else if (strcasecmp(name, "NLM_PERFCTR_GLOBAL") == 0)
			env_perfctr.global = strtoul(value, 0, 10);
		else if (strcasecmp(name, "NLM_PERFCTR") == 0) {
			value = getenv(name);
			if (value && strcasecmp(value,"ipc")==0) {
				env_perfctr.ctr[0] = PERFCTR_INSN;
				env_perfctr.ctr[1] = PERFCTR_CYC;
			}
			else if (value && strcasecmp(value,"brmiss")==0) {
				env_perfctr.ctr[0] = PERFCTR_BR_MISS;
				env_perfctr.ctr[1] = PERFCTR_BR;
			}
			else if (value && strcasecmp(value,"l1hit")==0) {
				env_perfctr.ctr[0] = PERFCTR_L1_HIT;
				env_perfctr.ctr[1] = PERFCTR_L1_REF;
			}
			else if (value && strcasecmp(value,"excrate")==0) {
				env_perfctr.ctr[0] = PERFCTR_CYC;
				env_perfctr.ctr[1] = PERFCTR_EXC;
			}
			else if (value && strcasecmp(value,"intrate")==0) {
				env_perfctr.ctr[0] = PERFCTR_CYC;
				env_perfctr.ctr[1] = PERFCTR_INT;
			}
			else {
				fprintf(stderr, "[%s]: unknown NLM_PERFCTR=%s\n", __FUNCTION__,
					value);
			}
		}
		else {
			fprintf(stderr, "[%s]: Unknown env name(%s)\n", __FUNCTION__, name);
		}
	}

	env_perfctr.ctr_val[0] = env_perfctr.ctr_val[1] = 0;
	printf("Using ctr0=%u, ctr1=%u, thread=%d, global=%d\n",
	       env_perfctr.ctr[0], env_perfctr.ctr[1],
	       env_perfctr.thread, env_perfctr.global);
}

void nlm_soclib_perfctr_init(struct nlm_soclib_perfctr *pc)
{
	*pc = env_perfctr;
}

void nlm_soclib_perfctr_start(struct nlm_soclib_perfctr *perfctr)
{
	fs_perf_ctr_start(perfctr->ctr[0], perfctr->ctr[1],
			  perfctr->thread, perfctr->global);
}

void nlm_soclib_perfctr_stop_accum(struct nlm_soclib_perfctr *perfctr)
{
	uint64_t val0, val1;

	fs_perf_ctr_stop(&val0, &val1);

	perfctr->ctr_val[0] += val0;
	perfctr->ctr_val[1] += val1;
}

void nlm_soclib_perfctr_print(struct nlm_soclib_perfctr *perfctr)
{
	uint64_t val0 = perfctr->ctr_val[0];
	uint64_t val1 = perfctr->ctr_val[1];

	printf("ctr0 = 0x%016llx, ctr1 = 0x%016llx, ratio=%g\n",
	       (unsigned long long)val0, (unsigned long long)val1,
	       (double)val0 / (val1?:1));
}

