/***********************************************************************
Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/

#ifndef _PROCESSOR_H
#define _PROCESSOR_H

#include <inttypes.h>

#define CHIP_PROCESSOR_ID_XLR_C4    0x00
#define CHIP_PROCESSOR_ID_XLR_B2    0x0b

#define CHIP_PROCESSOR_ID_XLS_608_STEP_A   0x80
#define CHIP_PROCESSOR_ID_XLS_408_STEP_A   0x88
#define CHIP_PROCESSOR_ID_XLS_404_STEP_A   0x8c
#define CHIP_PROCESSOR_ID_XLS_208_STEP_A   0x8e
#define CHIP_PROCESSOR_ID_XLS_204_STEP_A   0x8f
#define CHIP_PROCESSOR_ID_XLS_108_STEP_A   0xce
#define CHIP_PROCESSOR_ID_XLS_104_STEP_A   0xcf

#define CHIP_PROCESSOR_ID_XLS_616_STEP_B 0x40
#define CHIP_PROCESSOR_ID_XLS_416_STEP_B 0x44
#define CHIP_PROCESSOR_ID_XLS_608_STEP_B 0x4a
#define CHIP_PROCESSOR_ID_XLS_412_STEP_B 0x4c
#define CHIP_PROCESSOR_ID_XLS_408_STEP_B 0x4e
#define CHIP_PROCESSOR_ID_XLS_404_STEP_B 0x4f

#define CHIP_PROCESSOR_ID_XLR_308_STEP_B 0x6
#define CHIP_PROCESSOR_ID_XLR_508_STEP_B 0x7
#define CHIP_PROCESSOR_ID_XLR_516_STEP_B 0x8
#define CHIP_PROCESSOR_ID_XLR_532_STEP_B 0x9
#define CHIP_PROCESSOR_ID_XLR_716_STEP_B 0xA
#define CHIP_PROCESSOR_ID_XLR_732_STEP_B 0xB

#define CHIP_PROCESSOR_ID_XLR_308_STEP_C 0xF
#define CHIP_PROCESSOR_ID_XLR_508_STEP_C 0xB
#define CHIP_PROCESSOR_ID_XLR_516_STEP_C 0xA
#define CHIP_PROCESSOR_ID_XLR_532_STEP_C 0x8
#define CHIP_PROCESSOR_ID_XLR_716_STEP_C 0x2
#define CHIP_PROCESSOR_ID_XLR_732_STEP_C 0x0

#define NLM_REVISION_A0 0xc0000
#define NLM_REVISION_A1 0xc0001
#define NLM_REVISION_B0 0xc0002
#define NLM_REVISION_B1 0xc0003
#define NLM_REVISION_B2 0xc0004
#define NLM_REVISION_C0 0xc0005
#define NLM_REVISION_C1 0xc0006
#define NLM_REVISION_C2 0xc0007
#define NLM_REVISION_C3 0xc0008
#define NLM_REVISION_C4 0xc0009

enum CPU_REVISION {
	REV_A0 = 0,
	REV_A1,
	REV_B0,
	REV_B1,
	REV_B2,
	REV_C0,
	REV_C1,
	REV_C2,
	REV_C3,
	REV_C4
};

enum PROCESSOR_FAMILY {
	PROC_ID_XLR,
	PROC_ID_XLS,
	PROC_ID_XLP,
	MAX_PROC_FAMILY
};

enum PROC_STEPPING {
	PROC_ID_STEP_A,
	PROC_ID_STEP_B,
	PROC_ID_STEP_C,
	MAX_STEPPING
};

#define is_stepping_a() is_step_a
#define is_stepping_b() is_step_b
#define is_stepping_c() is_step_c

extern int processor_id;
extern int is_step_a, is_step_b, is_step_c;
extern int xlr_step_a, xlr_step_b, xlr_step_c;
extern int xls_step_a, xls_step_b;
extern int on_xlr, xlr_7xx, xlr_5xx, xlr_3xx;
extern int on_xls, xls_6xx, xls_608, xls_4xx, xls_2xx, xls_1xx;
extern int on_xlp;
extern int big_endian;

#define is_xlr() on_xlr
#define is_xlr_step_a() xlr_step_a
#define is_xlr_step_b() xlr_step_b
#define is_xlr_step_c() xlr_step_c

#define is_xls() on_xls
#define is_xls_step_a() xls_step_a
#define is_xls_step_b() xls_step_b

#define is_xlr_7xx() xlr_7xx
#define is_xlr_5xx() xlr_5xx
#define is_xlr_3xx() xlr_3xx

#define is_xls_2xx() xls_6xx
#define is_xls_4xx() xls_4xx
#define is_xls_6xx() xls_2xx
#define is_xls_1xx() xls_1xx

#define is_xls_608() xls_608

#define is_xlp() on_xlp

extern void init_cached_proc_id(void);

#endif /* _PROCESSOR_H */
