/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright

    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _NLM_DEV_H
#define _NLM_DEV_H

/* Generic API */
extern int soc_recv(int vc, unsigned int *intf, uint64_t *paddr, unsigned int *flags);

/* NAE API */
enum nlm_nae_tx_flags {
	NLM_NAE_TX_NONE = 0,
	NLM_NAE_TX_FREE_IN = 0x1,
};

enum nlm_nae_rx_flags {
	NLM_NAE_RX_NONE = 0,
	NLM_NAE_RX_ERR = 0x1,
	NLM_NAE_RX_TXC = 0x2,
	NLM_NAE_RX_TXERR = 0x4,
};

#define poe_class(hdr) (((hdr) >> 53) & 0x7)
enum poe_fwd_mode {
	POE_DROP_PKT_IN_NAE = 0,
	POE_FWD_DEST = 1,
	POE_REENQ_DV = 2,
	POE_REENQ_DEST = 3,
	POE_FWD_DV = 4,
	POE_DROP_PKT = 5,
	POE_REENQ_DV_SERIAL = 6,
	POE_REENQ_DEST_SERIAL = 7,
};


extern int nae_set_poe_distvec(int vec, uint32_t cm0, uint32_t cm1, uint32_t cm2, uint32_t cm3, uint32_t vcmask);
extern int nae_recv(uint64_t *paddr, unsigned int *cntl);
extern int nae_recv_poe(uint64_t *poe_hdr, uint64_t *paddr, int *cntl);

extern int nae_send(int dest, int fbid, uint64_t paddr, int len, unsigned int flags);
extern int nae_send_poe(int fwd_mode, uint64_t poe_hdr, int poe_class, int fbid, uint64_t paddr, int len);


uint16_t nae_get_hwport(uint32_t context);
/* DTR API */
int nlm_dtr_transfer (const char *in, uint32_t size, char *out);
int nlm_dtr_multi_transfer (const char **in, const unsigned int *len, char **out, unsigned int arr_size);
int nlm_dtr_raid (uint32_t disks, uint32_t segments_per_disk, char ***data,
                  const uint32_t *segment_length, uint32_t operation, uint32_t raidtype,
                  uint32_t p_device_id, uint32_t q_device_id);

/* KBP API  */

/* KBP request response type */
typedef enum kbp_rqt_rsp_type_t
{
	KBP_DIR_REQ_DIR_RSP_TYPE,
	KBP_DIR_REQ_MEM_RSP_TYPE,
	KBP_MEM_REQ_TYPE
}kbp_rqt_rsp_type_t;


/* KBP request data structure */
typedef struct kbp_rqt_data_t
{
	kbp_rqt_rsp_type_t	req_rsp_type;
	uint8_t			valid_ln;
	uint8_t			rsp_len; 
	uint8_t 		req_len;
	uint8_t			empty_byte;

	uint32_t		req_header;
	uint8_t			req_data_len;
	uint64_t		req_data[11];   					
	
}kbp_rqt_data_t;

/* KBP response data structure */
typedef struct  kbp_rsp_data_t {
	uint8_t rsp_type;
	uint8_t rsp_len;
	uint16_t req_id; 
	uint32_t req_header; 

	uint64_t recv_msg[4];
	uint64_t recv_msg_size;
}kbp_rsp_data_t;



/* Initializes the XLP KBP data structures and if init_link is requested then 
it also initializes the KBP link. */
int32_t kbp_init(uint32_t vc, uint32_t init_link);


/* Sends the request to the KBP using FMN. */
int32_t kbp_send(uint32_t vc, 
	kbp_rqt_data_t *req_p);


/* Recieve response from KBP for the previous request associated with the given vc. */
int32_t kbp_recv(int cpu_vc, 
	kbp_rsp_data_t* recv_rsp);

/* Free the XLP KBP Data structures and destroy. */
void kbp_destroy(uint32_t vc_id,
	uint32_t destroy_link);
#endif

