#include <stdio.h>
#include <inttypes.h>
#include <time.h>
#include <reent.h>
#include <errno.h>

extern double read_timer_seconds(void);

int clock_gettime(clockid_t clock_id, struct timespec *ts)
{
	struct _reent *reent = __getreent();
	int result = 0;

	switch(clock_id) {

		case CLOCK_REALTIME: {

			 double tot_time = read_timer_seconds();
			
			 uint64_t secs = (uint64_t)tot_time; 				/* mantissa */
			 uint64_t nsecs = (tot_time-secs) * 1000000000; 	/* fraction */
			
			 if (!ts) {
				 reent->_errno = EFAULT;
				 result = -1;
			 }
			 else {
				 ts->tv_sec = secs;
				 ts->tv_nsec = nsecs;
				 result = 0;
			 }
		 }
		 break;

		default:
			 reent->_errno = EINVAL;
			 result = -1;
			 break;
	}
	return result;
}

int clock_settime (clockid_t clock_id, const struct timespec *ts)
{
	struct _reent *reent = __getreent();

	reent->_errno = EPERM;
	return -1;
}

/**
* @defgroup smp Symmetric Multiprocessing API
* @brief Description about the symmetric multiprocessing apis supported in NETOS
*/


/**
* @brief The clone function starts a new process(child) on the next availabe cpu
* @param[in] fn The child process start executing fn(arg)
* @param[in] child_stack The child_stack arguement specifies the location of the stack used by the child process. 
* Remember that the stack grows from upper to bottom. So the address given should be the endaddress.
* @param[in] flags The bitwise ored value of several of the following constant. As we support clone() only in 
* the smp mode, the flags should be 
* - CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND | CLONE_THREAD | CLONE_SYSVSEM : 
*   The code and data segments, semaphores etc are shared with the parent process
* @param[in] arg Argument to the given funcation.
*
* @return 
*  - On sucess,  returns the cpuid of the child process. 
*  - On failure, returns -1. errno is set to indicate the cause of the error.
*	  
* @ingroup smp
*/

int clone(int (*fn) (void *), void *child_stack, int flags, void *arg, ...)
{
        extern int _clone(int (*fn) (void *), void *child_stack, int flags, void *arg, int *err);
	struct _reent *reent = __getreent();
        return _clone(fn, child_stack, flags, arg, &reent->_errno);
}

