#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "spinlock.h"
#include "devices.h"

static spinlock_t fdlock;
uint64_t *fdmap = NULL;
uint32_t nfds = 0;

struct device_ops *dev_ops[MAX_DEVICES] = {
	[0] = &dev_stdio_in,
	[1] = &dev_stdio_out,
	[2] = &dev_stdio_err,
	[3] = &dev_tmpfs,
	[4] = &dev_mem
};

static int ffc_quad(uint64_t b)
{
        int i;
        /* modify this to clz */
        for(i = 0; i < 64; i++) {
                if(!((1ULL << i) & b))
                        break;
        }
        return i;
}

int fdget(void)
{
        int off = 0, bcnt, bit;
	uint64_t *map = fdmap;

        spin_lock(&fdlock);
        bcnt = nfds * sizeof(uint64_t);
        while(bcnt) {
                bit = ffc_quad(*map);
                if(bit != 64 && (bit < (bcnt * 8))) {
                        *map = *map | ( 1ULL << bit);
                        spin_unlock(&fdlock);
                        return bit + off;
                }
                bcnt -= 8;
                map++;
                off += 64;
        }

        if((fdmap = realloc(fdmap,
                (nfds + 1) * sizeof(uint64_t))) == NULL) {
                spin_unlock(&fdlock);
                return -1;
        }
        map = (uint64_t *)&fdmap[nfds];
        nfds++;
        *map =  1ULL << 0;
        spin_unlock(&fdlock);
        return off;
}

void fdput(int fd)
{
        uint64_t *map;

        spin_lock(&fdlock);
        map = (uint64_t *)&fdmap[fd / 64];
        *map = *map & (~(1ULL << (fd % 64)));
        spin_unlock(&fdlock);
}

void fdset(int fd)
{
        uint64_t *map;

        spin_lock(&fdlock);

	if(fdmap == NULL) {
		fdmap = calloc(1, sizeof(uint64_t));
         	nfds++;
	}

        map = (uint64_t *)&fdmap[fd / 64];
        *map = *map | ((1ULL << (fd % 64)));
        spin_unlock(&fdlock);
}


void syscall_init(int cpu)
{
	int i = 0;
	extern void setup_timer(int);
	for (i = 0; i < MAX_DEVICES; i++) {
		if(dev_ops[i]) {
			dev_ops[i]->init();
			fdset(i);
		}
	}

	setup_timer(cpu);
}

int dev_add(struct device_ops *dev, int fd)
{
	if(fd >= MAX_DEVICES)
		return -1;
	dev_ops[fd] = dev;
	return 0;
}

struct device_ops *dev_get(int fd)
{
	if(fd >= MAX_DEVICES)
		return NULL;
	return dev_ops[fd];
}


