#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <reent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <setjmp.h>
#include <system.h>
#include <mips-exts.h>	/* for cpu id */

#include <devices.h>

int errno;

extern double read_timer_seconds(void);

extern struct device_ops *dev_ops[];

int _fstat(int fd, struct stat *buf)
{
	buf->st_mode = S_IFCHR;	/* Always pretend to be a tty */
	buf->st_blksize = 0;

	return (0);
}

/* getpid -- only one process, so just return 1.
 */
int _getpid(void)
{
	return processor_id();
}

/*
 * isatty -- returns 1 if connected to a terminal device,
 *           returns 0 if not. Since we're hooked up to a
 *           serial port, we'll say yes _AND return a 1.
 */
int _isatty(int fd)
{
	return 1;
}

/*
 * kill -- go out via exit...
 */
int _kill(int pid, int sig)
{
#if 0
	if(pid == __MYPID) 
		_exit(sig); 
#endif
	errno = EINVAL;
	return -1;
}

int _link(const char *old, const char *new)
{
	errno = EMLINK;
	return -1;
}

/*
 * lseek --  Since a serial port is non-seekable, we return an error.
 */
_off_t _lseek(int fd, _off_t pos, int whence)
{
	errno = ESPIPE;
	return ((off_t)-1);
}


int _execve(const char *name, char * const *argv, char * const *env)
{
	errno = ENOMEM;

	return -1;
}

int _fcntl(int fd, int cmd, int arg)
{
	int ret = -1;

	return ret;
}

/*
 * stat -- Since we have no file system, we just return an error.
 */
int _stat(const char *file, struct stat *pstat)
{
	errno = EIO;
	return (-1);
}

_CLOCK_T_ _times(struct tms *ptms)
{
	clock_t ret = -1;

	return ret;
}

/*
 * unlink -- since we have no file system, 
 *           we just return an error.
 */
int _unlink(const char *file)
{
	errno = ENOENT;
	return -1;
}

int _wait(int *status)
{
	errno = ECHILD;
	return -1;
}

int _gettimeofday(struct timeval *__tp, void *__tzp)
{
	double tot_time = read_timer_seconds();

	uint64_t secs = (uint64_t)tot_time; /* mantissa */
	uint64_t usecs = (tot_time - secs) * 1000000; /* fraction */

	__tp->tv_sec = secs;
	__tp->tv_usec = usecs;

	return 0;
}

int _open(const char *file, int flags, int mode)
{
	int fd = -1;

	for (fd = 0; fd < MAX_DEVICES; fd++) {
		if (dev_ops[fd] && (strcmp(dev_ops[fd]->name, file) == 0)) {
			/* found the device */
			if (dev_ops[fd]->open_r == 0) {
				return -1;
			}
			fd =  dev_ops[fd]->open_r(0, file, flags, mode);
			break;
		}
	}

	if(fd >= MAX_DEVICES)
		return -1;

	return fd;
}

int _close(int fd)
{
	int rv = -1;
	if (fd >= 0 && fd < MAX_DEVICES) {
		if(dev_ops[fd] && dev_ops[fd]->close_r)
			rv = dev_ops[fd]->close_r(0, fd);
		if(fd >= 3)
			fdput(fd); /* 3 are reserved for stdin, out and err */
	}
	return rv;
}

static struct _reent _rmi_impure_ptrs[XLx_NUM_CPUS] = {
	[0] = _REENT_INIT(_rmi_impure_ptrs[0]),
	[1] = _REENT_INIT(_rmi_impure_ptrs[1]),
	[2] = _REENT_INIT(_rmi_impure_ptrs[2]),
	[3] = _REENT_INIT(_rmi_impure_ptrs[3]),
	[4] = _REENT_INIT(_rmi_impure_ptrs[4]),
	[5] = _REENT_INIT(_rmi_impure_ptrs[5]),
	[6] = _REENT_INIT(_rmi_impure_ptrs[6]),
	[7] = _REENT_INIT(_rmi_impure_ptrs[7]),
	[8] = _REENT_INIT(_rmi_impure_ptrs[8]),
	[9] = _REENT_INIT(_rmi_impure_ptrs[9]),
	[10] = _REENT_INIT(_rmi_impure_ptrs[10]),
	[11] = _REENT_INIT(_rmi_impure_ptrs[11]),
	[12] = _REENT_INIT(_rmi_impure_ptrs[12]),
	[13] = _REENT_INIT(_rmi_impure_ptrs[13]),
	[14] = _REENT_INIT(_rmi_impure_ptrs[14]),
	[15] = _REENT_INIT(_rmi_impure_ptrs[15]),
	[16] = _REENT_INIT(_rmi_impure_ptrs[16]),
	[17] = _REENT_INIT(_rmi_impure_ptrs[17]),
	[18] = _REENT_INIT(_rmi_impure_ptrs[18]),
	[19] = _REENT_INIT(_rmi_impure_ptrs[19]),
	[20] = _REENT_INIT(_rmi_impure_ptrs[20]),
	[21] = _REENT_INIT(_rmi_impure_ptrs[21]),
	[22] = _REENT_INIT(_rmi_impure_ptrs[22]),
	[23] = _REENT_INIT(_rmi_impure_ptrs[23]),
	[24] = _REENT_INIT(_rmi_impure_ptrs[24]),
	[25] = _REENT_INIT(_rmi_impure_ptrs[25]),
	[26] = _REENT_INIT(_rmi_impure_ptrs[26]),
	[27] = _REENT_INIT(_rmi_impure_ptrs[27]),
	[28] = _REENT_INIT(_rmi_impure_ptrs[28]),
	[29] = _REENT_INIT(_rmi_impure_ptrs[29]),
	[30] = _REENT_INIT(_rmi_impure_ptrs[30]),
	[31] = _REENT_INIT(_rmi_impure_ptrs[31]),
};

struct _reent *__getreent(void)
{
	return &_rmi_impure_ptrs[processor_id()];
}

#if defined(NEWLIB_VER) && (NEWLIB_VER < 117)
int _execve_r (struct _reent *ptr, char *name, char **argv, char **env)
#else
int _execve_r(struct _reent *ptr, const char *name, char * const *argv, char * const *env)
#endif
{
	int ret;

	errno = 0;
	if ((ret = _execve (name, argv, env)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

int _fcntl_r(struct _reent *ptr, int fd, int cmd, int arg)
{
	int ret;

	errno = 0;
	if ((ret = _fcntl (fd, cmd, arg)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

int _fstat_r(struct _reent *ptr, int fd, struct stat *pstat)
{
	int ret;

	errno = 0;
	if ((ret = _fstat (fd, pstat)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

int _getpid_r(struct _reent *ptr)
{
	int ret;
	ret = _getpid ();
	return ret;
}

int _isatty_r(struct _reent *ptr, int fd)
{
	int ret;

	errno = 0;
	if ((ret = _isatty (fd)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

int _kill_r(struct _reent *ptr, int pid, int sig)
{
	int ret;

	errno = 0;
	if ((ret = _kill (pid, sig)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

int _link_r(struct _reent *ptr, const char *old, const char *new)
{
	int ret;

	errno = 0;
	if ((ret = _link (old, new)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

_off_t _lseek_r(struct _reent *ptr, int fd, _off_t pos, int whence)
{
	_off_t ret;

	errno = 0;
	if ((ret = _lseek (fd, pos, whence)) == (_off_t) -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

int _open_r(struct _reent *ptr, const char *file, int flags, int mode)
{
	int fd = -1;
	if ((fd = _open(file, flags, mode)) == -1) {
		/* device not found */
		ptr->_errno = ENODEV;
	}
	return fd;
}

int _close_r(struct _reent *ptr, int fd)
{
	int rv = -1;
	if ((rv =_close(fd)) == -1) {
		ptr->_errno = ENODEV;
	}
	return rv;
}

_ssize_t _read_r(struct _reent *ptr, int fd, void *buf, size_t cnt)
{
	if (fd >= 0 && fd < MAX_DEVICES)
		return dev_ops[fd]->read_r(ptr, fd, buf, cnt);

	ptr->_errno = ENODEV;
	return -1;
}

_ssize_t _write_r(struct _reent *ptr, int fd, const void *buf, size_t cnt)
{
	if (fd >= 0 && fd < MAX_DEVICES)
		return dev_ops[fd]->write_r(ptr, fd, buf, cnt);

	ptr->_errno = ENODEV;
	return -1;
}

/**
* @brief The mmap function maps 'len' bytes starting at 'off' from the file descriptor 'fd' into memory at address 'addr'.
* @param[in] addr Address where the contents of the file should be mapped. Should be NULL in our case.
* @param[in] length Number of bytes to be mapped
* @param[in] prot Memory protection flags. Valid values are
*  - PROT_READ | PROT_WRITE  : Read and Write to this memory is allowed
* @param[in] flags Type of the mapped object. Valid values are 
*  - MAP_SHARED : Share this mapping with all other threads.
* @param[in] fildis File descriptor opened by shm_open or open
* @param[in] offset Mapping offset from the beginning of the file. Should be zero in our case.
*
* @return 
*  - On sucess,  returns a pointer to the mapped area. 
*  - On failure, returns MAP_FAILED(i,e. (void *)-1). errno is set to indicate the cause of the error.
*	  
* @sa    munmap
* @ingroup mem
*/
void *mmap(void *addr, size_t len, int prot, int flags, int fildis, off_t off)
{
	int fd = fildis;
	struct _reent *ptr = __getreent();
	if(fd >= 0 && fd < MAX_DEVICES && dev_ops[fd] && dev_ops[fd]->mmap) 
		return dev_ops[fd]->mmap(ptr, fildis, addr, len, prot, flags, off);
	ptr->_errno = ENODEV;
	return (void *)-1;
}

/**
* @brief The munmap function delets the mapping for 'len' bytes at address 'addr'.
* @param[in] addr Address returned by mmap. 
* @param[in] length Number of bytes to be unmapped. Partial unmapping is not supported, i.e, 
* the length should be same as the length given during the mmap call.
*
* @return 
*  - On sucess,  returns zero. 
*  - On failure, returns -1. errno is set to indicate the cause of the error.
*	  
* @sa    munmap
* @ingroup mem
*/
int munmap(void *addr, size_t len)
{
	extern int shm_munmap(struct _reent *reent, void *addr, size_t len);
	extern int pmem_munmap(struct _reent *reent, void *addr, size_t len);
	extern unsigned long devmem_vaddr_start;
	if((unsigned long)addr >= devmem_vaddr_start)
		return pmem_munmap(__getreent(), addr, len);
	else
		return shm_munmap(__getreent(), addr, len);

}

int ftruncate(int fildes, off_t len)
{
	int fd = fildes;
	struct _reent *ptr = __getreent();
	if(fd >= 0 && fd < MAX_DEVICES && dev_ops[fd] && dev_ops[fd]->ftrunc) 
		return dev_ops[fd]->ftrunc(ptr, fildes, len);
	ptr->_errno = ENODEV;
	return -1;

}

extern void *_sbrk(ptrdiff_t nbytes);
void *_sbrk_r(struct _reent *ptr, ptrdiff_t incr)
{
	char *ret;
	void *_sbrk(ptrdiff_t);

	errno = 0;
	if ((ret = (char *)(_sbrk (incr))) == (void *) -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

int _stat_r(struct _reent *ptr, const char *file, struct stat *pstat)
{
	int ret;

	errno = 0;
	if ((ret = _stat (file, pstat)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

_CLOCK_T_ _times_r(struct _reent *ptr, struct tms *ptms)
{
	clock_t ret;

	ret = _times (ptms);
	return ret;
}

int _unlink_r(struct _reent *ptr, const char *file)
{
	int ret;

	errno = 0;
	if ((ret = _unlink (file)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

int _wait_r(struct _reent *ptr, int *status)
{
	int ret;

	errno = 0;
	if ((ret = _wait (status)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}

/* Not guaranteed to be available on all targets.  
 */
int _gettimeofday_r(struct _reent *ptr, struct timeval *ptimeval, void *ptimezone)
{
	int ret;

	errno = 0;
	if ((ret = _gettimeofday (ptimeval, ptimezone)) == -1 && errno != 0)
		ptr->_errno = errno;
	return ret;
}
