/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#!/bin/sh -e

pkgroot=`pwd`
installdir="$pkgroot/install.$$"
cygwindir=/
builddir=winbuild.$$
buildpath="$pkgroot/$builddir"

starttime=`date`

#
# This script runs in $builddir
#
mkdir "$installdir" "$builddir"
cd "$builddir"  || exit 1

#
# 1. Check out the sources, we can do this only on a unix machine
#    because there is no pserver installed.

#ssh jayachandranc@nlm2-india  "(rm -fr $builddir; mkdir $builddir; cd $builddir\
#   && cvs -d /proj/aps/sw/cvsroot co netos_lib netos_apps netos_gdb toolchains\
#   )>buildwin.log; \
#   if [ $? -eq 0 ]; then cd $builddir && tar cvzf - .; fi"  | tar xvzf -

tar xvzf $HOME/nlmwinpkg.tar.gz

#
# 2. Build and install the stuff
#

#
# 2a. Build binutils
#
(mkdir toolchains/build-binutils; \
  cd toolchains/build-binutils && \
  ../src/binutils-2.15/configure --target=mipsisa64r2xlp-elf --enable-cxx-flags=-G0 \
 --disable-nls --enable-languages=c,c++ --prefix=$installdir/nlmtools \
 --disable-shared --with-newlib \
 --with-headers="$buildpath/netos_lib/srcs/libs/newlib/mipsisa64r2xlp-elf/include" \
 --disable-threads --with-gnu-as --with-gnu-ld --with-arch=xlr --with-float=soft \
 --with-tune=xlr && make && make install )

PATH="$installdir/nlmtools/bin":$PATH

#
# 2b. Build GCC
#
(mkdir toolchains/build-gcc; \
  cd toolchains/build-gcc && \
  ../src/gcc-3.4.3/configure --target=mipsisa64r2xlp-elf --enable-cxx-flags=-G0 \
 --disable-nls --enable-languages=c,c++ --prefix=$installdir/nlmtools \
 --disable-shared --with-newlib \
 --with-headers="$buildpath/netos_lib/srcs/libs/newlib/mipsisa64r2xlp-elf/include" \
 --disable-threads --with-gnu-as --with-gnu-ld --with-arch=xlr --with-float=soft \
 --with-tune=xlr && make && make install )

#
# 2c. Build insight
#

(mkdir netos_gdb/build-insight; \
  cd netos_gdb/build-insight && \
  ../insight-6.4/configure --target=mips64\
  --prefix=$installdir/nlmtools --disable-nls && make && make install)

#
# 3. Put in the sources
#
mkdir $installdir/sources && \
 tar --exclude CVS --exclude .cvsignore -cf - netos_lib netos_apps  | \
          (cd $installdir/sources &&  tar xvf -)

#
# 4. Build netos_lib
#

(cd netos_lib && \
   make REBUILD_SCRIPTS=true \
   NETOS_INSTALL_DIR=$installdir/netos_lib all install)

# 5. Put in the documentation
#
mkdir $installdir/docs && \
  tar --exclude CVS -C netos_win32 -cvf - docs | (cd "$installdir" && tar xvf -)

#
# 6. put in the required cygwin tools & libs
#
(cd /bin && cp -f cp.exe find.exe grep.exe make.exe pwd.exe rm.exe sort.exe \
               cygiconv-2.dll cygintl-2.dll cygwin1.dll "$installdir/nlmtools/bin")
cp -f "$buildpath/netos_win32/nlmlaunch.tcl" "$installdir/nlmtools/bin"

#
# 7. Create the package
#
(cd "$installdir" && cp -f "$buildpath/netos_win32/nlmwin32.iss" . && \
      /cygdrive/c/Program\ Files/Inno\ Setup\ 5/iscc /O.. nlmwin32.iss)

#
# That's all folks!
#
echo "Build started at $starttime, ended at `date`"
echo "Installation package in $pkgroot"
