#define _GNU_SOURCE
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <sys/nlm_crypto.h>
#include <sys/nlm_crypto_api.h>

#include <mempool.h>
#include "test_data.h"

//#define DEBUG  

//#define Message(a, b...) printf(a, ##b)
#define Message(a, b...) 


struct nlm_frags{
        unsigned char *buf;
        unsigned int len;
};

void nlm_crypto_print_error_msg(int status)
{
	if (!status)
		return;
	printf("Error ...\n");
	if (status & NLM_ERR_CIPHER_ALG_INVALID)
		printf("cipher alg invalid");
	if (status & NLM_ERR_CIPHER_ALG_MODE_INVALID)
		printf("\tcipher alg mode invalid\n");
	if (status & NLM_ERR_AUTH_ALG_INVALID)
		printf("\tauth alg invalid\n");
	if (status & NLM_ERR_AUTH_ALG_MODE_INVALID)
		printf("\tauth alg mode invalide");
	if (status & NLM_ERR_CIPHER_IV_LEN_INVALID)
		printf("\tcipher iv len invalid");
	if (status & NLM_ERR_CIPHER_KEY_LEN_INVALID)
		printf("\tcipher key len invalid");
	if (status & NLM_ERR_CIPHER_KEY_NULL)
		printf("\tcipher key is null\n");
	if (status & NLM_ERR_AUTH_KEY_NULL)
		printf("\tauth key is null\n");
	if (status & NLM_ERR_AUTH_DST_NULL)
		printf("\tauth destination is null\n");
	if (status & NLM_ERR_TAG_LEN)
		printf("\ttag length incorrect\n");
}
mpoolptr_t pool_iovec_mem, pool_hash_dst_mem, pool_pkt_mem;
static void free_digest_text(struct crypto_param *sd)
{
	if (sd->hash_dst_address);
	{
	//	free(sd->hash_dst_address);
		mpfree(pool_hash_dst_mem, sd->hash_dst_address);
		mpdestroy(pool_hash_dst_mem);
	}
	if (sd->src && sd->dst)
	{
		if(sd->src->buf && sd->dst->buf)
		{
			mpfree(pool_pkt_mem, sd->src->buf);
			mpfree(pool_pkt_mem, sd->dst->buf);
			mpdestroy(pool_pkt_mem);
		}
		mpfree(pool_iovec_mem, sd->src);
		mpfree(pool_iovec_mem, sd->dst);
		mpdestroy(pool_iovec_mem);
	}
	if (sd->iv)
		free(sd->iv);
}

static void alloc_digest_text(struct crypto_param *sd, struct test_params *t)
{
	pool_iovec_mem = mpcreate("sae_iovec_mem", 2, sizeof(struct crypto_iovec), 64, 0);
	if(pool_iovec_mem == NULL) 
	{
		printf("\n Memory allocation failed for iovec");
		//return NLM_ERR_NO_MEM;
	}
	sd->src = mpalloc(pool_iovec_mem, 0);
	sd->dst = mpalloc(pool_iovec_mem, 0);
	
	pool_hash_dst_mem = mpcreate("sae_hash_dst_mem", 2, sizeof(struct crypto_iovec), 64, 0);
	if(pool_iovec_mem == NULL) 
	{
		printf("\n Memory allocation failed for hash destination");
		//return NLM_ERR_NO_MEM;
	}
	sd->hash_dst_address = mpalloc(pool_hash_dst_mem, 0);

	pool_pkt_mem = mpcreate("sae_pkt_mem", 2, (TXT_SIZE + 64), 64, 0);
	if(pool_iovec_mem == NULL) 
	{
		printf("\n Memory allocation failed for iovec");
		//return NLM_ERR_NO_MEM;
	}
	sd->src->buf = mpalloc(pool_pkt_mem, 0);
	sd->dst->buf = mpalloc(pool_pkt_mem, 0);
#if 0
	sd->src = malloc(sizeof(struct crypto_iovec));
	sd->dst = malloc(sizeof(struct crypto_iovec));
	sd->hash_dst_address = malloc(128);
	sd->src->buf = malloc(TXT_SIZE + 64);
	sd->dst->buf = malloc(TXT_SIZE + 64);
#endif
}

static __inline__ unsigned char getnibble(unsigned char a)
{       
    if (a >= 'a' && a <= 'f')
        return a - 'a' + 10;
    if (a >= 'A' && a <= 'F')
        return a - 'A' + 10;
    return a - '0';
}               
        
static void hex2bin(unsigned char *dst, unsigned char *src, unsigned int len)
{       
    int i;
    for (i = 0; i < len*2; i=i+2) {
        dst[i/2] = (getnibble(src[i]) << 4 ) | (getnibble(src[i + 1])) ;
    }
}

#ifdef DEBUG
static void print_text(unsigned char *buf, int len) 
{
	int i;
	unsigned char *ptr;

	ptr = buf;

	if (!buf) {
		printf("Oops... passed null ptr to %s\n",__FUNCTION__);
		return;
	}

	printf("\n");
	for (i = 0; i < len; i++) {
		printf("%02x",ptr[i]);
		if (i % 16 == 15)
			printf("\n");
	}
	printf("\n");
}
#endif

static int check_cipher_output_frag(struct crypto_param *sd, struct test_data *t)
{
	unsigned char *dst_bin;
	int cmp=0,done=0;
	int start,i;
	struct nlm_frags *f;

	dst_bin = malloc(t->len);
	hex2bin(dst_bin, t->out, t->len);

	f = (struct nlm_frags *)sd->dst;
	for (start = 0, i = 0; i < sd->nr_frags; i++) {
		cmp |= memcmp(f->buf , dst_bin + done, f->len);
		Message("in [%s]... cmp = %d, f->len = %d, done = %d\n",
			__FUNCTION__, cmp, f->len,done);
		done += f->len;
		f++;
	}

	free(dst_bin);

	return cmp;
}

static int check_cipher_output(unsigned char *dst, struct test_data *t, int offset)
{
	unsigned char *dst_bin = NULL;
	int cmp = 0;

	dst_bin = (unsigned char*)malloc(t->len);

        if(dst_bin == NULL)
		printf("\n ERROR- MAlloc failed");

	hex2bin(dst_bin, t->out, t->len);
	printf("\n");
#ifdef DEBUG
	printf("\n cipher:\nexpected output:");
	print_text(dst_bin, t->len);
	printf("\n actual output:");
	print_text(dst+offset, t->len);
	printf("\n IV + actual output :");
	print_text(dst, t->len);
#endif
	cmp = memcmp(dst + offset, dst_bin, t->len);
	free(dst_bin);

	return cmp;
}

static int check_auth_output(unsigned char *auth_dst, struct test_data *t)
{
	unsigned char *dst_bin;
	int cmp;

	dst_bin = malloc(t->digest_len);
	hex2bin(dst_bin, t->digest, t->digest_len);

	printf("\n");
#ifdef DEBUG
	printf("\n auth:\n expected output:");
	print_text(dst_bin, t->digest_len);
	printf("\n actual output:");
	print_text(auth_dst, t->digest_len);
#endif
	cmp = memcmp(auth_dst, dst_bin, t->digest_len);
    
	free(dst_bin);

	return cmp;
}

#define CIPHER_FAILED 1
#define AUTH_FAILED 2
#if 0
static int check_cipher_iv_frag_output(struct crypto_param *sd, struct test_data *t)
{
	int cmp=0, ret=0;

	if (sd->session->cipher_alg != NLM_CIPHER_BYPASS)
		cmp = check_cipher_output_frag(sd, t);
	if (cmp)
		ret |= CIPHER_FAILED;

	cmp = 0;
	if (sd->session->auth_alg != NLM_AUTH_BYPASS)
		cmp = check_auth_output(sd->auth_dst, t);
	if (cmp)
		ret |= AUTH_FAILED;
	return ret;
}
static int check_cipher_iv_output(struct crypto_param *sd, struct test_data *t)
{
	int cmp=0, ret=0;

	if (sd->session->cipher_alg != NLM_CIPHER_BYPASS)
		cmp = check_cipher_output(sd->dst, t, 0);
	if (cmp)
		ret |= CIPHER_FAILED;

	cmp = 0;
	if (sd->session->auth_alg != NLM_AUTH_BYPASS)
		cmp = check_auth_output(sd->auth_dst, t);
	if (cmp)
		ret |= AUTH_FAILED;
	return ret;
}

#endif
static int check_output(struct crypto_session *session, struct crypto_param *sd, struct test_data *t)
{
	int cmp=0, ret=0;

	if (session->cip && (session->cip->cipher_alg != NLM_CIPHER_BYPASS))
		cmp = check_cipher_output(sd->dst->buf, t, sd->iv_len);
	if (cmp)
		ret |= CIPHER_FAILED;

	cmp = 0;
	if (session->aip && (session->aip->auth_alg != NLM_AUTH_BYPASS))
		cmp = check_auth_output(sd->hash_dst_address, t);
	if (cmp)
		ret |= AUTH_FAILED;
	return ret;
}

static void print_test_info(struct crypto_session *session)
{
	printf("*********** ");
	if (session->cip && session->cip->cipher_alg != NLM_CIPHER_BYPASS) {

		if (nlm_crypto_cipher_alg_has_mode(session->cip->cipher_alg))
			printf("Cipher = (%d:%d) ",session->cip->cipher_alg,session->cip->cipher_mode);
		else
			printf("Cipher = (%d) ",session->cip->cipher_alg);
	}

	if (session->aip && session->aip->auth_alg != NLM_AUTH_BYPASS) {
		if (nlm_crypto_auth_alg_has_mode(session->aip->auth_alg))
			printf("Auth = (%d:%d) ", session->aip->auth_alg, session->aip->auth_mode);
		else
			printf("Auth = (%d) ", session->aip->auth_alg);
	}
	if (session->aip && ((session->aip->auth_alg == NLM_AUTH_MD5) || (session->aip->auth_alg == 
						      NLM_AUTH_SHA)) && session->aip->hmac)
		printf("(hmac) ");

	printf("****************\n");
}


static int convert_fill_keys(unsigned char *cipher_key, unsigned char *auth_key, struct test_params *tp, 
		       struct test_data *t, int test_case)
{
	if ((tp->cipher_alg != NLM_CIPHER_BYPASS) && (cipher_key))
	{
		hex2bin(cipher_key, t->cipher_key, tp->cipher_key_len);
	}
	if ((tp->auth_alg != NLM_AUTH_BYPASS) && (auth_key)){
		// Zero out the key
		memset(auth_key, 0, 128);
		if (t->auth_key_len > 128) {
			printf("Auth key too long\n");
			printf("Skipping test %d \n",test_case);
			return -1;
		}
		hex2bin(auth_key, t->auth_key, t->auth_key_len);
	}
	return 0;
}

//ret: response of check_output funcion
//test_case: test case number
static void print_output_response(int ret, int test_case, char *success)
{
	if (ret) {
		if ((ret & (AUTH_FAILED | CIPHER_FAILED)) == 
		    (AUTH_FAILED | CIPHER_FAILED)) 
			printf("Error...!!!!! Both cipher and auth tests failed for test [%d]\n", test_case);
		else if (ret & AUTH_FAILED)
			printf("Error...!!!!! Auth failed for test [%d]\n",test_case);
		else if (ret & CIPHER_FAILED)
			printf("Error...!!!!! Cipher failed for test [%d]\n",test_case);
		else printf("Unknown error for test [%d]\n",test_case);
	} else {
		if (!success)
			printf("Great!!! test [%d] passed \n", test_case);
		else
			printf("%s",success);
	}
}

static int cipher_mode_iv_len[NLM_CIPHER_MAX][NLM_CIPHER_MODE_MAX] = {
/*		       ECB  CBC   CFB   OFB   CTR  AESF8    GCM  CCM    8   9  LRW   XTS */
/* BYPASS */       {   0,    0,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* DES */          {   0,    8,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* 3DES */         {   0,    8,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* AES128 */       {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,},
/* AES192 */       {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,},
/* AES256 */       {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,},
/* ARC4 */         {   0,    0,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* KASUMI F8 */    {   0,    0,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* SNOW3G F8 */    {   0,    0,    0,    0,    0,    0,      0,    0,   0,  0,  0,    0,},
/* CAMELLIA128 */  {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,},
/* CAMELLIA192 */  {   0,    16,   16,   16,   16,   16,     -1,   0,   0,  0,  16,   16,}, 
/* CAMELLIA256 */  {   0,    16,   16,   16,   16,   16      -1,   0,   0,  0,  16,   16,},
};



static void fill_offsets_data(struct crypto_session *session, struct crypto_param *sd, struct test_data *t)
{
	unsigned char *tmp;

	tmp = sd->src->buf;

	sd->iv_offset = 0;
	sd->auth_offset = 0;
	sd->cipher_offset = 0;
	if (session->cip)
	{
		if(cipher_mode_iv_len[session->cip->cipher_alg][session->cip->cipher_mode]) {
		hex2bin(tmp, t->iv, sd->iv_len);
		tmp += sd->iv_len;
		sd->cipher_offset= sd->iv_len;
		sd->src_len += sd->iv_len;
		}
	} /*else {
		sd->session->cipher_iv_len = 0;
	}*/
	hex2bin(tmp, t->in, t->len);
	Message("cipher iv length = %d, t->len = %d\n",
		sd->iv_len, t->len);
	sd->src->iov_len = sd->dst->iov_len = t->len + sd->iv_len;
}

void test_cipher_auth(struct test_params *tp, struct test_data
		 *td, unsigned int nr_tests)
{
	struct test_data *t = td;
	int i,ret;
	struct crypto_param *sd = NULL;

	struct crypto_session *session = NULL;
	uint8_t cipher_key[128], auth_key[128] ;
	uint8_t *tmp_key = NULL;
	if (!tp) return;
	if (!t) return;
	session = malloc(sizeof(struct crypto_session));
	sd = malloc(sizeof(struct crypto_param));
	if (!sd || !session) {
		printf("Ran out of memory\n");
		printf("returning\n");
		return;
	}
	memset(session, 0, sizeof(struct crypto_session));
	memset(sd, 0, sizeof(struct crypto_param));
	sd->tag_len = t->digest_len;

	alloc_digest_text(sd, tp);
	sd->iv = NULL;
	convert_fill_keys(cipher_key, auth_key, tp, t, 0);

	hex2bin(cipher_key, t->cipher_key, tp->cipher_key_len);
	tmp_key = auth_key;
	if (t->auth_key_len)
		hex2bin(auth_key, t->auth_key, t->auth_key_len);
	else 
		tmp_key = NULL;
	if(tp->cipher_alg != NLM_CIPHER_BYPASS)
	{
		session->cip = (struct crypto_cipher_init_param*)malloc(sizeof(struct crypto_cipher_init_param));
		if(session->cip == NULL)
			printf("\n couldn't allocte memory for session->cip");
		memset(session->cip, 0, sizeof(struct crypto_cipher_init_param));
		session->cip->cipher_alg = tp->cipher_alg;
                session->cip->cipher_mode = tp->cipher_mode;

                {
                        session->cip->cipher_key = malloc(sizeof(struct crypto_iovec));
			if(session->cip->cipher_key == NULL)
				printf("\n couldn't allocte memory for session->cip->cipher_key");
			memset(session->cip->cipher_key, 0, sizeof(struct crypto_iovec));
                        session->cip->cipher_key->buf = malloc(tp->cipher_key_len+1);
			if(session->cip->cipher_key->buf == NULL)
				printf("\n couldn't allocte memory for session->cip->cipher_key->buf");
                        memcpy(session->cip->cipher_key->buf, cipher_key, tp->cipher_key_len);
                        session->cip->cipher_key->iov_len = tp->cipher_key_len;
                }
                session->cip->arc4_cipher_key_len = 0;
                session->cip->arc4_key_init = 0;
                session->cip->cfb_mask = 0;
		sd->iv_len = cipher_mode_iv_len[tp->cipher_alg][tp->cipher_mode];

	}
	if(tp->auth_alg != NLM_AUTH_BYPASS)
	{
		session->aip = (struct crypto_auth_init_param*)malloc(sizeof(struct crypto_auth_init_param));
		if(session->aip == NULL)
			printf("\n couldn't allocte memory for session->aip");
		memset(session->aip, 0, sizeof(struct crypto_auth_init_param));
                session->aip->auth_alg = tp->auth_alg;
                session->aip->auth_mode = tp->auth_mode;
	}
	ret = crypto_setup_cipher_auth_session(session);
	if (ret) {
		printf("Error in session setup\n");
			nlm_crypto_print_error_msg(ret);
			printf("returning\n");
			return;
	}
	print_test_info(session);
	Message(" session setup successful. \n");

	for (i = 0; i < nr_tests; i++) {

		if (convert_fill_keys(cipher_key, auth_key, tp, t, i))
			continue;

		if (t->auth_key_len)
		{
			tmp_key = auth_key;
			session->aip->auth_key = malloc(sizeof(struct crypto_iovec));
                        if(session->aip->auth_key == NULL)
                                printf("\n couldn't allocte memory for session->aip->auth_key");
                        memset(session->aip->auth_key, 0, sizeof(struct crypto_iovec));
                        session->aip->auth_key->buf = malloc(t->auth_key_len +1);
                        if(session->aip->auth_key->buf == NULL)
                                printf("\n couldn't allocte memory for session->aip->auth_key->buf");
                        memcpy(session->aip->auth_key->buf,auth_key, t->auth_key_len);
                        session->aip->auth_key->iov_len = t->auth_key_len;
                        session->aip->hmac = 1;

			sd->auth_key_len = session->aip->auth_key->iov_len;//t->auth_key_len;
		}
		else
			tmp_key = NULL;

		crypto_session_reset_key(session, cipher_key,
					     tmp_key);
		sd->nr_frags = 1;
		sd->src_len = t->len;
		sd->enc = t->enc;
		sd->hash_source = t->enc;
		sd->cipher_len = t->len;
     		sd->cipher_key_len = tp->cipher_key_len;
		if (session->aip && (session->aip->auth_alg != NLM_AUTH_BYPASS)) {
			sd->auth_len = t->auth_len;
			sd->tag_len = t->digest_len;
		} else
			sd->auth_len = 0;
		fill_offsets_data(session, sd, t);

		sd->hash_dst_address = (unsigned char*)mpgetphys(pool_hash_dst_mem, sd->hash_dst_address);
		sd->src->buf = (unsigned char*)mpgetphys(pool_pkt_mem, sd->src->buf);
		sd->dst->buf = (unsigned char*)mpgetphys(pool_pkt_mem, sd->dst->buf);
		ret = crypto_cipher_auth_op(session, sd);
		sd->hash_dst_address = (unsigned char*)mpgetvirt(pool_hash_dst_mem, (unsigned long)sd->hash_dst_address);
		sd->src->buf = (unsigned char*)mpgetvirt(pool_pkt_mem, (unsigned long)sd->src->buf);
		sd->dst->buf = (unsigned char *)mpgetvirt(pool_pkt_mem, (unsigned long)sd->dst->buf);
		if(ret){
		printf("Error in cipher auth operation for test [%d]\n", i);
			printf("Check input provided to nlm_crypto_cipher_auth function\n");
			continue;
		}

		ret = check_output(session, sd, t);
		print_output_response(ret, i, NULL);

		if(session->aip)
		if(session->aip->auth_key)
                {
                        if(session->aip->auth_key->buf)
                                free(session->aip->auth_key->buf);
                        free(session->aip->auth_key);
			session->aip->auth_key = NULL;
                }

		t++;
	}
	free_digest_text(sd);
	crypto_cleanup_session(session);
	if(session->cip)
	{
		if (session->cip->cipher_key)
		{
			if(session->cip->cipher_key->buf)
				free(session->cip->cipher_key->buf);
			free(session->cip->cipher_key);
		}
		free(session->cip);
	}
	if(session->aip)
	{
		free(session->aip);
	}

	free(session);
	free(sd);
}

#define CALLBACK_VAL 0x12345678
#if 0
static void async_api_test_callback(void *callback_arg)
{
	struct crypto_param *sd;

	sd = (struct crypto_param *)callback_arg;

	if (!sd) {
		printf("Return message from engine corrupted\n");
		return;
	}
	Message("callback received.\n");

	sd->state = CALLBACK_VAL;

}
#endif
static struct test_params api_test_params = {
	.cipher_alg = NLM_CIPHER_AES128,
	.cipher_mode = NLM_CIPHER_MODE_CBC, 
	.auth_alg = NLM_AUTH_SHA,
	.auth_mode = NLM_AUTH_MODE_SHA1,
	.cipher_iv_len = NLM_AES_IV_LEN,
	.cipher_key_len = NLM_AES128_KEY_LEN, 
//	.max_auth_key_len = NLM_SHA1_KEY_LEN, 
	.designer_desc_valid = 0, 
	.hmac = 1,
};

static struct test_data api_test_data = {
        .enc = NLM_ENCRYPT,
        .len = 32,
	.auth_len = 48,
	.auth_key_len = 64,
	.digest_len = 20,
        .cipher_key="c286696d887c9aa0611bbb3e2025a45a",
	.auth_key = "31323334353637383931323334353637383931323334353637383931323334353637383931323334353637383931323334353637383931323334353637383930",
        .iv = "562e17996d093d28ddb3ba695a2e6f58",
        .in = "000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f",
        .out = "d296cd94c2cccf8a3a863028b5e1dc0a7586602d253cfff91b8266bea6d61ab1",
	.digest ="e937d67f9f7ca953f6f9c209d6e03f26c389872e",

};

static struct test_data aad_b4_iv_test_data = {
        .enc = NLM_ENCRYPT,
        .len = 16,
	.auth_len = 32,
	.aad_len = 16,
	.auth_key_len = 64,
	.digest_len = 20,
        .cipher_key="c286696d887c9aa0611bbb3e2025a45a",
	.auth_key = "31323334353637383931323334353637383931323334353637383931323334353637383931323334353637383931323334353637383931323334353637383930",
        .iv = "d296cd94c2cccf8a3a863028b5e1dc0a",
        .in = "101112131415161718191a1b1c1d1e1f",
        .out = "7586602d253cfff91b8266bea6d61ab1",
        .aad = "562e17996d093d28ddb3ba695a2e6f58",
	.digest ="e937d67f9f7ca953f6f9c209d6e03f26c389872e",

};
static struct test_data aad_after_iv_test_data = {
        .enc = NLM_ENCRYPT,
        .len = 32,
	.auth_len = 48,
	.aad_len = 16,
	.auth_key_len = 64,
	.digest_len = 20,
        .cipher_key="c286696d887c9aa0611bbb3e2025a45a",
	.auth_key = "31323334353637383931323334353637383931323334353637383931323334353637383931323334353637383931323334353637383931323334353637383930",
        .iv = "562e17996d093d28ddb3ba695a2e6f58",
        .aad = "aadaadaadaadaadaadaadaadaadaadaa",
        .in = "000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f",
        .out = "d296cd94c2cccf8a3a863028b5e1dc0a7586602d253cfff91b8266bea6d61ab1",
	.digest ="1bb901d9b1f7542030e32d975222e2cb6f65d5d1", //if iv is included
//	.digest = "090c97c6a640535ce7e5c9010cba9c1990ab8145",//if iv is excluded

};

static struct test_params auth_api_test_params = {
	.cipher_alg = NLM_CIPHER_BYPASS,
	.cipher_mode = 0, 
	.auth_alg = NLM_AUTH_SHA,
	.auth_mode = NLM_AUTH_MODE_SHA1,
	.cipher_iv_len = 0,
	.cipher_key_len = 0,
	.designer_desc_valid = 0, 
	.hmac = 1,
};

static struct test_data auth_api_test_data = {
	.len = 16,
	.auth_len = 16,
	.auth_key_len = 64,
	.auth_key = "31323334353637383931323334353637383931323334353637383931323334353637383931323334353637383931323334353637383931323334353637383930",
	.in= "dc95c078a2408989ad48a21492842087",
	.digest = "f530bb1d956e1a680b031f23572afef33a5bb8ae",
	.digest_len = 20,
};

#if 0
void test_async_api()
{
	struct test_data *t;
	struct test_params *tp;
	int ret;
	struct crypto_param *sd;
	struct nlm_crypto_session *sess;
	uint8_t cipher_key[128], auth_key[128];

	tp = &api_test_params;
	t = &api_test_data;

	if (!tp) return;
	if (!t) return;

	sess = malloc(sizeof(struct nlm_crypto_session));
	sd = malloc(sizeof(struct crypto_param));
	if (!sd || !sess) {
		printf("Ran out of memory\n");
		printf("returning\n");
		return;
	}
	memset(sd, 0, sizeof(sd));
	sd->session = sess;

	sess->tag_len = t->digest_len;

	alloc_digest_text(sd, tp);
	sd->cipher_iv = malloc(64);

	printf("%s",tp->comment);
	hex2bin(cipher_key, t->cipher_key, tp->cipher_key_len);
	hex2bin(auth_key, t->auth_key, t->auth_key_len);

	ret = nlm_crypto_setup_cipher_auth_session(sess, tp->cipher_alg, tp->cipher_mode,
				   cipher_key, NULL, tp->auth_alg, tp->auth_mode, 
				   auth_key, t->digest_len);
	if (ret) {
		printf("Error in session setup\n");
			nlm_crypto_print_error_msg(ret);
			printf("returning\n");
			return;
	}
	Message(" session setup successful. \n");

	sd->nr_frags = 0;
	sd->len = t->len;
	sd->enc = t->enc;
	sd->cipher_len = t->len;
	if (sd->session->auth_alg != NLM_AUTH_BYPASS) {
		sd->auth_len = t->auth_len;
		sess->tag_len = t->digest_len;
	} else
		sd->auth_len = 0;

	hex2bin(sd->src, t->in, t->len);
	hex2bin(sd->cipher_iv, t->iv, sd->session->cipher_iv_len);
	sd->aad_len = 0;
	sd->aad = NULL;
	sd->callback = async_api_test_callback;
	sd->callback_arg = sd;

	sd->flags = NLM_CRYPTO_FLAG_ASYNC_OP;
	Message("going for nlm_crypto_cipher_auth_op...\n");
	ret = nlm_crypto_cipher_auth_op(sess, sd);

	if ((ret) && (ret != NLM_ERR_IN_PROGRESS)) {
		printf("Error in cipher auth operation for test [%d]\n", 0);
		printf("Check input provided to nlm_crypto_cipher_auth function\n");
	}

	while (((volatile struct crypto_param *)sd)->state != CALLBACK_VAL) {
		nlm_crypto_process_cms_msg();
	}

	Message("Async op completed. \n");

	ret = check_cipher_iv_output(sd, t);
	print_output_response(ret, 0, "Async API test passed\n");
#ifdef DEBUG
	printf("src is: \n");
	print_text(sd->src, t->len);
//	printf("dst = %p, virt_to_phys(dst) = %llx, dst is: \n",&sd->dst,
//	       (unsigned long long)virt_to_phys(sd->dst) );
	print_text(sd->dst, t->len);
	printf("iv is: \n");
	print_text(sd->cipher_iv, sd->session->cipher_iv_len);

	Message("Testing testing 6 \n");
#endif

	free_digest_text(sd);
	nlm_crypto_cleanup_session(sd->session);
	free(sd->session);
	free(sd);
}

void test_frags()
{
	struct test_data *t;
	struct test_params *tp;
	int i,ret;
	struct crypto_param *sd;
	struct nlm_crypto_session *sess;
	struct nlm_frags src_frags[10], dst_frags[10];
	uint8_t *src, *dst;
	int rem;
	uint8_t cipher_key[128], auth_key[128];

	tp = &api_test_params;
	t = &api_test_data;


	if (!tp) return;
	if (!t) return;

	sess = malloc(sizeof(struct nlm_crypto_session));
	sd = malloc(sizeof(struct crypto_param));
	if (!sd || !sess) {
		printf("Ran out of memory\n");
		printf("returning\n");
		return;
	}
	memset(sd, 0, sizeof(struct crypto_param));
	memset(sess, 0, sizeof(struct nlm_crypto_session));
	sd->session = sess;

	sess->tag_len = t->digest_len;

	alloc_digest_text(sd, tp);
	src = sd->src;
	dst = sd->dst;
	
	sd->src = (unsigned char *)src_frags;
	sd->dst = (unsigned char *)dst_frags;
	sd->cipher_iv = malloc(64);

	printf("%s",tp->comment);
	
	hex2bin(cipher_key, t->cipher_key, tp->cipher_key_len);
	hex2bin(auth_key, t->auth_key, t->auth_key_len);
	Message("going for nlm_crypto_setup_cipher_auth_session\n");

	ret = nlm_crypto_setup_cipher_auth_session(sess, tp->cipher_alg, tp->cipher_mode,
				   cipher_key, NULL, tp->auth_alg, tp->auth_mode, 
				   auth_key, t->digest_len);
	if (ret) {
		printf("Error in session setup\n");
			nlm_crypto_print_error_msg(ret);
			printf("returning\n");
			return;
	}
	Message(" session setup successful. \n");

	sd->nr_frags = 0;
	sd->len = t->len;
	sd->enc = t->enc;
	sd->cipher_len = t->len;
	if (sd->session->auth_alg != NLM_AUTH_BYPASS) {
		sd->auth_len = t->auth_len;
		sess->tag_len = t->digest_len;
	} else
		sd->auth_len = 0;

	memset(src, 0, t->len * 2);
	memset(dst, 0, t->len * 2);
	for (i = 0, rem = t->len; rem > 0; i++, rem -= 10) {
		if (rem >= 10) {
			hex2bin(src + i * 20, t->in + i * 20, 10);
			src_frags[i].buf = src + i * 20;
			dst_frags[i].buf = dst + i * 20;
			src_frags[i].len = 10;
			dst_frags[i].len = 10;
		} else {
			hex2bin(src + i * 20, t->in + i * 20, rem);
			src_frags[i].buf = src + i * 20;
			dst_frags[i].buf = dst + i * 20;
			src_frags[i].len = rem;
			dst_frags[i].len = rem;
		}
	}

	sd->nr_frags = i;

	hex2bin(sd->cipher_iv, t->iv, sd->session->cipher_iv_len);

	sd->aad_len = 0;
	sd->aad = NULL;
	sd->callback = async_api_test_callback;
	sd->callback_arg = sd;

	sd->flags = NLM_CRYPTO_FLAG_SRC_DST_NLM_FRAGS;
	ret = nlm_crypto_cipher_auth_op(sess, sd);

	ret = check_cipher_iv_frag_output(sd, t);
	print_output_response(ret, 0, "Frags test passed\n");
#ifdef DEBUG
	Message("src is: \n");
	print_text(src, t->len * 2);
	Message("dst is: \n");
	print_text(dst, t->len * 2);
	Message("iv is: \n");
	print_text(sd->cipher_iv, sd->session->cipher_iv_len);

	Message("Testing testing 6 \n");
#endif

	sd->src = src;
	sd->dst = dst;
	free_digest_text(sd);
	nlm_crypto_cleanup_session(sd->session);
	free(sd->session);
	free(sd);
}
void test_aad_frags(int flag)
{
	struct test_data *t;
	struct test_params *tp;
	int i,ret;
	struct crypto_param *sd;
	struct nlm_crypto_session *sess;
	struct nlm_frags src_frags[10], dst_frags[10], aad_frags[10];
	uint8_t *src, *dst, *aad;
	uint8_t cipher_key[128], auth_key[128];
	int rem;

	sess = malloc(sizeof(struct nlm_crypto_session));
	sd = malloc(sizeof(struct crypto_param));
	if (!sd || !sess) {
		printf("Ran out of memory\n");
		printf("returning\n");
		return;
	}
	memset(sd, 0, sizeof(sd));
	sd->session = sess;

	tp = &api_test_params;

	if (flag & NLM_CRYPTO_FLAG_AAD_AFTER_IV){
		sd->flags = NLM_CRYPTO_FLAG_SRC_DST_NLM_FRAGS |
			NLM_CRYPTO_FLAG_AAD_NLM_FRAGS|
			NLM_CRYPTO_FLAG_AAD_AFTER_IV;
		t = &aad_after_iv_test_data;

		printf("aad after iv ");//no \n


	} else {
		sd->flags = NLM_CRYPTO_FLAG_SRC_DST_NLM_FRAGS |
			NLM_CRYPTO_FLAG_AAD_NLM_FRAGS;
		t = &aad_b4_iv_test_data;
		printf("aad before iv "); //no \n
	}

	if (!tp) return;
	if (!t) return;

	sess->tag_len = t->digest_len;

	alloc_digest_text(sd, tp);
	src = sd->src;
	dst = sd->dst;
	aad = malloc(200);
	
	sd->src = (unsigned char *)src_frags;
	sd->dst = (unsigned char *)dst_frags;
	sd->cipher_iv = malloc(64);
	sd->aad = (unsigned char *)aad_frags;
	sd->aad_len = t->aad_len;

//	print_test_info(sess);
	Message("\n%s",tp->comment);

	hex2bin(cipher_key, t->cipher_key, tp->cipher_key_len);
	hex2bin(auth_key, t->auth_key, t->auth_key_len);
	ret = nlm_crypto_setup_cipher_auth_session(sess, tp->cipher_alg, tp->cipher_mode,
				   cipher_key, NULL, tp->auth_alg, tp->auth_mode, 
				   auth_key, t->digest_len);
	if (ret) {
		printf("Error in session setup\n");
			nlm_crypto_print_error_msg(ret);
			printf("returning\n");
			return;
	}
	Message(" session setup successful. \n");

	sd->nr_frags = 0;
	sd->len = t->len;
	sd->enc = t->enc;
	sd->cipher_len = t->len;
	if (sd->session->auth_alg != NLM_AUTH_BYPASS) {
		sd->auth_len = t->auth_len;
		sess->tag_len = t->digest_len;
	} else
		sd->auth_len = 0;

	memset(src, 0, t->len * 2);
	memset(dst, 0, t->len * 2);
	for (i = 0, rem = t->len; rem > 0; i++, rem -= 10) {
		if (rem >= 10) {
			hex2bin(src + i * 20, t->in + i * 20, 10);
			src_frags[i].buf = src + i * 20;
			dst_frags[i].buf = dst + i * 20;
			src_frags[i].len = 10;
			dst_frags[i].len = 10;
		} else {
			hex2bin(src + i * 20, t->in + i * 20, rem);
			src_frags[i].buf = src + i * 20;
			dst_frags[i].buf = dst + i * 20;
			src_frags[i].len = rem;
			dst_frags[i].len = rem;
		}
	}
	sd->nr_frags = i;

	for (i = 0, rem = t->aad_len; rem > 0; i++, rem -= 10) {
		if (rem >= 10) {
			hex2bin(aad + i * 20, t->aad + i * 20, 10);
			aad_frags[i].buf = aad + i * 20;
			aad_frags[i].len = 10;
		} else {
			hex2bin(aad + i * 20, t->aad + i * 20, rem);
			aad_frags[i].buf = aad + i * 20;
			aad_frags[i].len = rem;
		}
	}
	sd->nr_frags_aad = i;

	hex2bin(sd->cipher_iv, t->iv, sd->session->cipher_iv_len);

	sd->callback = async_api_test_callback;
	sd->callback_arg = sd;

	ret = nlm_crypto_cipher_auth_op(sess, sd);

	ret = check_cipher_iv_frag_output(sd, t);
	print_output_response(ret, 0, "aad frags test passed\n");

	sd->src = src;
	sd->dst = dst;
	sd->aad = aad;
	free_digest_text(sd);
	nlm_crypto_cleanup_session(sd->session);
	free(sd->session);
	free(sd);
}

static void async_junk(void *jj)
{
	int *a;
	if (!jj)
		return;

	a = jj;
	*a = CALLBACK_VAL;
	Message("async callback\n");
}
void test_cipher_auth_api()
{
	struct test_data *t;
	struct test_params *tp;
	struct nlm_crypto_session *sess;
	uint8_t *src, *dst, *cipher_iv, *aad, *auth_dst;
	uint8_t *cipher_key, *auth_key;
	int cmp,ret;
	int this_data=0;

	sess = malloc(sizeof(struct nlm_crypto_session));
	if (!sess) {
		printf("Ran out of memory\n");
		printf("returning\n");
		return;
	}
	memset(sess, 0, sizeof(struct nlm_crypto_session));

	src = malloc(TXT_SIZE + 64);
	dst = malloc(TXT_SIZE + 64);
	aad = malloc(200);
	cipher_key = malloc(128);
	auth_key = malloc(128);
	auth_dst = malloc(128);
	cipher_iv = malloc(128);

	tp = &api_test_params;
	t = &aad_b4_iv_test_data;

	hex2bin(cipher_key, t->cipher_key, tp->cipher_key_len);
	hex2bin(auth_key, t->auth_key, t->auth_key_len);

	Message("create cipher auth session...\n");
	ret = nlm_crypto_setup_cipher_auth_session(sess, tp->cipher_alg, tp->cipher_mode,
				   cipher_key, NULL, tp->auth_alg, tp->auth_mode, 
				   auth_key, t->digest_len);
	if (ret) {
		printf("Error in create session\n");
			nlm_crypto_print_error_msg(ret);
			printf("returning\n");
			return;
	}
	Message("created cipher auth session\n");

	hex2bin(src, t->in, t->len);
	hex2bin(aad, t->aad, t->aad_len);
	hex2bin(cipher_iv, t->iv, tp->cipher_iv_len);

	Message("call nlm_crypto_cipher_auth\n");
	nlm_crypto_cipher_auth(sess, t->enc, src, dst, t->len, t->auth_len, 
			 cipher_iv, aad, t->aad_len, auth_dst, 0, 0, NULL, NULL, 0);

	if (ret) {
		printf("Error in cipher auth operation for test [%s]\n", __FUNCTION__);
		nlm_crypto_print_error_msg(ret);
		goto cleanup;
	}

	cmp = check_cipher_output(dst, t, 0);

	if (cmp)
		ret |= CIPHER_FAILED;

	cmp = 0;
		cmp = check_auth_output(auth_dst, t);
	if (cmp)
		ret |= AUTH_FAILED;

	print_output_response(ret, 0, "sync test passed\n");

	Message("call nlm_crypto_cipher_auth\n");
	nlm_crypto_cipher_auth(sess, t->enc, src, dst, t->len, t->auth_len, 
			 cipher_iv, aad, t->aad_len, auth_dst, 0, 0, async_junk, &this_data, NLM_CRYPTO_FLAG_ASYNC_OP);

	while ((volatile int)this_data != CALLBACK_VAL) {
		nlm_crypto_process_cms_msg();
	}
	Message("Async op completed. \n");

	if (ret) {
		printf("Error in cipher auth operation for test [%s]\n", __FUNCTION__);
		nlm_crypto_print_error_msg(ret);
		goto cleanup;
	}

	cmp = check_cipher_output(dst, t, 0);

	if (cmp)
		ret |= CIPHER_FAILED;

	cmp = 0;
		cmp = check_auth_output(auth_dst, t);
	if (cmp)
		ret |= AUTH_FAILED;

	print_output_response(ret, 0, "async test passed\n");

	hex2bin(src, t->aad, t->aad_len);
	hex2bin(src + t->aad_len, t->iv, tp->cipher_iv_len);
	hex2bin(src + t->aad_len + tp->cipher_iv_len, t->in, t->len);
	
	Message("call nlm_crypto_cipher_auth_offsets\n");
	ret = nlm_crypto_cipher_auth_offsets(sess , t->enc,
			     src, dst, t->len, t->len + t->aad_len +
			     tp->cipher_iv_len, t->aad_len, t->aad_len + 
			     tp->cipher_iv_len, 0, auth_dst, 0, NULL, NULL, 0);

	if (ret) {
		printf("Error in cipher auth operation for test [%s]\n", __FUNCTION__);
		nlm_crypto_print_error_msg(ret);
		goto cleanup;
	}

	cmp = check_cipher_output(dst + t->aad_len + tp->cipher_iv_len, t, 0);

	if (cmp)
		ret |= CIPHER_FAILED;

	cmp = 0;
		cmp = check_auth_output(auth_dst, t);
	if (cmp)
		ret |= AUTH_FAILED;

	print_output_response(ret, 0, "offsets test passed\n");

cleanup:
	nlm_crypto_cleanup_session(sess);

	free(src);
	free(dst);
	free(aad);
	free(cipher_key);
	free(auth_key);
	free(auth_dst);
}

void test_cipher_api()
{
	struct test_data *t;
	struct test_params *tp;
	struct nlm_crypto_session *sess;
	uint8_t *src, *dst, *cipher_iv;
	int cmp,ret;
	int this_data=0;
	uint8_t *cipher_key;

	sess = malloc(sizeof(struct nlm_crypto_session));
	if (!sess) {
		printf("Ran out of memory\n");
		printf("returning\n");
		return;
	}
	memset(sess, 0, sizeof(struct nlm_crypto_session));
	src = malloc(TXT_SIZE + 64);
	dst = malloc(TXT_SIZE + 64);
	cipher_key = malloc(128);
	cipher_iv = malloc(128);

	tp = &api_test_params;
//	t = &api_test_data;
	t = &aad_b4_iv_test_data;

	hex2bin(cipher_key, t->cipher_key, tp->cipher_key_len);

	Message("create cipher session\n");
	ret = nlm_crypto_setup_cipher_session(sess, tp->cipher_alg, tp->cipher_mode,
				   cipher_key, NULL);
	if (ret) {
		printf("Error in create session\n");
			nlm_crypto_print_error_msg(ret);
			printf("returning\n");
			return;
	}
	Message("created cipher session\n");

	hex2bin(src, t->in, t->len);
	hex2bin(cipher_iv, t->iv, tp->cipher_iv_len);

	Message("call nlm_crypto_cipher \n");
	nlm_crypto_cipher(sess, t->enc, src, dst, t->len, cipher_iv, 0, NULL, NULL, 0);

	if (ret) {
		printf("Error in cipher operation for test [%s]\n", __FUNCTION__);
		nlm_crypto_print_error_msg(ret);
		goto cleanup;
	}

	cmp = check_cipher_output(dst, t, 0);

	if (cmp)
		ret |= CIPHER_FAILED;

	print_output_response(ret, 0, "sync test passed\n");

	Message("call nlm_crypto_cipher \n");
	nlm_crypto_cipher(sess, t->enc, src, dst, t->len, cipher_iv, 0, async_junk, &this_data, NLM_CRYPTO_FLAG_ASYNC_OP);

	while ((volatile int)this_data != CALLBACK_VAL) {
		nlm_crypto_process_cms_msg();
	}
	Message("Async op completed. \n");

	if (ret) {
		printf("Error in cipher operation for test [%s]\n", __FUNCTION__);
		nlm_crypto_print_error_msg(ret);
		goto cleanup;
	}

	cmp = check_cipher_output(dst, t, 0);

	if (cmp)
		ret |= CIPHER_FAILED;

	print_output_response(ret, 0, "async test passed\n");

	hex2bin(src, t->iv, tp->cipher_iv_len);
	hex2bin(src + tp->cipher_iv_len, t->in, t->len);
	Message("call nlm_crypto_cipher_embedded_iv\n");
	nlm_crypto_cipher_embedded_iv(sess, t->enc, src, dst, t->len +
				tp->cipher_iv_len, 0, 0, 0, 0);
	if (ret) {
		printf("Error in cipher operation for test [%s]\n", __FUNCTION__);
		nlm_crypto_print_error_msg(ret);
		goto cleanup;
	}

	cmp = check_cipher_output(dst, t, tp->cipher_iv_len);

	if (cmp)
		ret |= CIPHER_FAILED;

	print_output_response(ret, 0, "embedded iv test passed\n");

cleanup:
	nlm_crypto_cleanup_session(sess);

	free(src);
	free(dst);
	free(cipher_key);
}


void test_auth_api()
{
	struct test_data *t;
	struct test_params *tp;
	struct nlm_crypto_session *sess;
	uint8_t *src, *auth_dst;
	uint8_t *auth_key;
	int cmp,ret;
	int this_data=0;

	sess = malloc(sizeof(struct nlm_crypto_session));
	if (!sess) {
		printf("Ran out of memory\n");
		printf("returning\n");
		return;
	}
	memset(sess, 0, sizeof(struct nlm_crypto_session));
	src = malloc(TXT_SIZE + 64);
	auth_key = malloc(128);
	auth_dst = malloc(128);

	tp = &auth_api_test_params;
	t = &auth_api_test_data;

	hex2bin(auth_key, t->auth_key, t->auth_key_len);

	Message("create auth session\n");
	ret = nlm_crypto_setup_auth_session(sess, tp->auth_alg, tp->auth_mode, 
				   auth_key, t->digest_len);
	if (ret) {
		printf("Error in create session\n");
			nlm_crypto_print_error_msg(ret);
			printf("returning\n");
			return;
	}
	Message("created auth session\n");

	hex2bin(src, t->in, t->len);

	Message("call nlm_crypto_auth\n");
	nlm_crypto_auth(sess, src, t->len, NULL, auth_dst, 0, NULL, NULL, 0);

	if (ret) {
		printf("Error in auth operation for test [%s]\n", __FUNCTION__);
		nlm_crypto_print_error_msg(ret);
		goto cleanup;
	}

	cmp = check_auth_output(auth_dst, t);
	if (cmp)
		ret |= AUTH_FAILED;

	print_output_response(ret, 0, "sync test passed\n");

	Message("call nlm_crypto_auth\n");
	nlm_crypto_auth(sess, src, t->len, NULL, auth_dst, 0, async_junk, &this_data, NLM_CRYPTO_FLAG_ASYNC_OP);

	while ((volatile int)this_data != CALLBACK_VAL) {
		nlm_crypto_process_cms_msg();
	}
	Message("Async op completed. \n");

	if (ret) {
		printf("Error in auth operation for test [%s]\n", __FUNCTION__);
		nlm_crypto_print_error_msg(ret);
		goto cleanup;
	}

	cmp = 0;
		cmp = check_auth_output(auth_dst, t);
	if (cmp)
		ret |= AUTH_FAILED;

	print_output_response(ret, 0, "async test passed\n");

cleanup:
	nlm_crypto_cleanup_session(sess);

	free(src);
	free(auth_key);
	free(auth_dst);
}
#endif
