#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/nlm_crypto.h>

#include "test_data.h"
#include "aes128_data.h"
#include "sha1_data.h"
#include "sha256_data.h"
#include "sha384_data.h"
#include "sha512_data.h"
#include "md5_data.h"
#include "sha1_hmac_data.h"
#include "sha256_hmac_data.h"
#include "sha384_hmac_data.h"
#include "sha512_hmac_data.h"
#include "md5_hmac_data.h"
#include "aes128_sha256_data.h"
#include "aes256_sha1_data.h"
#include "aes128_sha1_data.h"

//#define Message(a, b...) printf(a, ##b)

extern void test_cipher_auth(struct test_params *tp, struct test_data *td, unsigned int nr_tests);
#if 0
extern void test_async_api();
extern void test_frags();
extern void test_aad_frags(int flag);
extern void test_cipher_auth_api();
extern void test_cipher_api();
extern void test_auth_api();

void test_apis();
#endif
struct cipher_auth_test_config sec_test_cipher_config[] = {
	{
		.params = &aes128_cbc_test_params,
		.tests = cipher_aes128_cbc_auth_null_test_data,
		.nr_tests = NR_TESTS_AES128_CBC,
	},
};

struct cipher_auth_test_config sec_test_auth_config[] = {
	{
		.params = &sha1_test_params,
		.tests = cipher_null_auth_sha1_test_data,
		.nr_tests = NR_TESTS_SHA1,
	},
	{
		.params = &sha1_hmac_test_params,
		.tests = cipher_null_auth_sha1_hmac_test_data,
		.nr_tests = NR_TESTS_SHA1_HMAC,
	},
#if 0
	{
		.params = &sha256_test_params,
		.tests = cipher_null_auth_sha256_test_data,
		.nr_tests = NR_TESTS_SHA256,
	},
	{
		.params = &sha256_hmac_test_params,
		.tests = cipher_null_auth_sha256_hmac_test_data,
		.nr_tests = NR_TESTS_SHA256_HMAC,
	},
	{
		.params = &sha384_test_params,
		.tests = cipher_null_auth_sha384_test_data,
		.nr_tests = NR_TESTS_SHA384,
	},
	{
		.params = &sha384_hmac_test_params,
		.tests = cipher_null_auth_sha384_hmac_test_data,
		.nr_tests = NR_TESTS_SHA384_HMAC,
	},
	{
		.params = &sha512_test_params,
		.tests = cipher_null_auth_sha512_test_data,
		.nr_tests = NR_TESTS_SHA512,
	},
	{
		.params = &sha512_hmac_test_params,
		.tests = cipher_null_auth_sha512_hmac_test_data,
		.nr_tests = NR_TESTS_SHA512_HMAC,
	},
	{
		.params = &md5_test_params,
		.tests = cipher_null_auth_md5_test_data,
		.nr_tests = NR_TESTS_MD5,
	},
#endif
	{
		.params = &md5_hmac_test_params,
		.tests = cipher_null_auth_md5_hmac_test_data,
		.nr_tests = NR_TESTS_MD5_HMAC,
	},
};
struct cipher_auth_test_config sec_test_cipher_auth_config[] = {
	{
		.params = &aes128_cbc_sha1_test_params,
		.tests = cipher_aes128_cbc_auth_sha1_test_data,
		.nr_tests = NR_TESTS_AES128_CBC_SHA1,
	},
#if 0
	//aes256 not yet supported in simulator. 
	{
		.params = &aes256_cbc_sha1_test_params,
		.tests = cipher_aes256_cbc_auth_sha1_test_data,
		.nr_tests = NR_TESTS_AES256_CBC_SHA1,
	},
	{
		.params = &aes128_cbc_sha256_test_params,
		.tests = cipher_aes_cbc_auth_sha256_test_data,
		.nr_tests = NR_TESTS_AES128_CBC_SHA256,
	},
#endif
};

void sec_test(void);
int main(int argc, char *argv[])
{
	printf("----------- Security Application ------\n");

	sec_test();

	printf("Done. exiting %s...\n",__FUNCTION__);

	return 0;
}


void sec_test(void)
{
	int i, nr_algos;
	struct cipher_auth_test_config *curr_algo; 


	printf("#################### API testing ######################\n");

//	test_apis();

	printf("#################### Algorithm testing ################\n");

	nlm_crypto_init();

#if 1
	nr_algos = sizeof(sec_test_cipher_config) / sizeof (struct cipher_auth_test_config);
	for ( i = 0, curr_algo = sec_test_cipher_config; i < nr_algos; i++) {
		test_cipher_auth(curr_algo->params, curr_algo->tests, curr_algo->nr_tests);
		curr_algo++;
	}
#endif
#if 1
	nr_algos = sizeof(sec_test_auth_config) / sizeof (struct cipher_auth_test_config);
	for ( i = 0, curr_algo = sec_test_auth_config; i < nr_algos; i++) {
		test_cipher_auth(curr_algo->params, curr_algo->tests, curr_algo->nr_tests);
		curr_algo++;
	}

#endif
#if 1
	nr_algos = sizeof(sec_test_cipher_auth_config) / sizeof (struct cipher_auth_test_config);
	for ( i = 0, curr_algo = sec_test_cipher_auth_config; i < nr_algos; i++) {
		test_cipher_auth(curr_algo->params, curr_algo->tests, curr_algo->nr_tests);
		curr_algo++;
	}
#endif
}
#if 0
void test_apis()
{
	printf("...............Async flag testing............ \n");
	test_async_api();
	printf("................Frags testing..................... \n");
	test_frags();
	printf("................aad testing.................... \n");
	test_aad_frags(0); //flag -> aad comes before iv
	test_aad_frags(NLM_CRYPTO_FLAG_AAD_AFTER_IV);

	printf(".................cipher and auth api testing............ \n");
	test_cipher_auth_api();
	printf("...................cipher api testing.................... \n");
	test_cipher_api();
	printf("..................auth api testing....................... \n");
	test_auth_api();
}
#endif
