/***********************************************************************
  Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
  reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/
#include <stdio.h>
#include <sys/mman.h>
#include <sched.h>
#include <signal.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <mempool.h>
#include <sys/nlm_dev.h>
#include <fdtuser.h>

#define LOOPBACK_PACKET 1
#define DROP_PACKET 2
#define EXCEPTION_PACKET 3
#define FORWARD_PACKET 4


#define PHYS_TO_VIRT(addr) (linux_mem_vaddr + addr)

//#define HW_EMULATOR    1
#define SINGLE_THREADED 1


/* HACK! To be removed */
#define XLP_NET_TX_VC_BASE 476

#define MAX_PKTS       50
#define APP_PORT_BASE  0
#define NUM_APP_PORTS  3

#define ICMP_PKT_SIZE  74
#define PKT_ALIGN_LEN  64
#define MAX_CPUS       32
#define MAX_GMAC_PORT  18

#define CLONE_BUDDY_FLGS (CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND | SIGCHLD )

//#define DBG        1
#ifdef DBG
#define log_dbg     printf
#define log_pkt     printf
#else
#define log_dbg(...)
#define log_pkt(...)
//    #define log_err(...)
#endif
#define log_err
#define log_info   printf

#define FP_MAGIC	0x900ddeed
struct fp_stats
{
	unsigned long long loopback[32];
	unsigned long long dropped[32];
	unsigned long long forwarded[32];
	unsigned long long error[32];
};

struct linux_fp_shared_memory
{
	volatile unsigned int magic;
	int mode;
	unsigned int trigger_ipc;
	struct fp_stats stats;
};
volatile struct linux_fp_shared_memory *shmem_vaddr;
static int naerx_vc, saerx_vc, iofb_vc, ipcrx_vc;
/* FP sends messages to cp_dst and cp sends messages to fp_dst */
static int fp_dst, cp_dst;
uint64_t linux_mem_vaddr = 0ULL;

int process_xaui(void *args)
{
	uint64_t laddr;
	unsigned int cntl = 0, context = 0, port = 19;
	unsigned int flags;
	int dest;
	int rxc = 0, txc = 0;
	int cpu = getpid();
	volatile unsigned char *vaddr;
	uint32_t len;
	int fbid = 0;
	unsigned int fwd_port[4] = {1,0,3,2};   // 1 context per port. context-port mapping = 0-0, 1-1, 2-2, 3-3  
#ifndef RXCTXMAP_LOOKUP	
	// change this array according to the complex configuration in nae.dts
	unsigned int ctx2port[7] = {0, 4, 8, 12, 16, 17, MAX_GMAC_PORT};
#endif
	while (1) {
		/* RX */
		len = nae_recv(&laddr, &cntl);

		/* no message received */
		if (len == 0) continue;

		context = cntl & 0x3ff;

#ifndef RXCTXMAP_LOOKUP 
		port = ctx2port[context];
#else
                if ((port = nae_get_hwport(context)) >= MAX_GMAC_PORT) {
                           printf("Invalid Context port map: context %d port %d\n",context, port);
                }
#endif
		
		if(port > 15)	//Don't loopback packet from mgmt interface.
			continue;
                fbid = port + 32; // This matches with the VFBID table in the FDT

		if (len < 0) {
			/* printf("-%d len %d\n",ec, len); */
			flags = -len;

			if (flags & (NLM_NAE_RX_ERR | NLM_NAE_RX_TXC)) {
				nae_send(port, fbid, laddr, 0, NLM_NAE_TX_FREE_IN);
				shmem_vaddr->stats.error[cpu]++;
				continue;
			}

			if (flags & NLM_NAE_RX_TXERR) {
				/* printf("Transmit underrun/collision!\n"); */
				continue;
			}

			printf("Unsupported rx flags (%d)!\n", flags);
			exit(-1);
		}

		if(shmem_vaddr->mode == LOOPBACK_PACKET){
			rxc++;
			vaddr = (volatile unsigned char *)(unsigned long)
				PHYS_TO_VIRT(laddr);
			/* Modify the last byte of the packet for demo */
			/*vaddr[len -1] = 0xa5;*/
			/* TX */
			if (nae_send(context, fbid, laddr, len, NLM_NAE_TX_NONE) == 0) {
				txc++;
			}
			shmem_vaddr->stats.loopback[cpu]++;
		}
		else if (shmem_vaddr->mode == FORWARD_PACKET){
			rxc++;
			vaddr = (unsigned char *)(unsigned long)
				PHYS_TO_VIRT(laddr);

			/* TX */
			dest = fwd_port[context];
			
			if (nae_send(dest, fbid, laddr, len, NLM_NAE_TX_NONE) == 0) {
				txc++;
			}
			shmem_vaddr->stats.forwarded[cpu]++;
		}else if(shmem_vaddr->mode == DROP_PACKET){
			nae_send(port, fbid, laddr, 0, NLM_NAE_TX_FREE_IN);
			shmem_vaddr->stats.dropped[cpu]++;
			continue;
		}
	}
	printf("XLP XAUI Loopback app completed\n");
	exit(0);
}



extern void nlm_get_vc_params(int *, int *, int *, int *);
extern void nlm_get_ipc_dsts(int *, int *);

static int get_app_parameters(void)
{
	uint64_t phys_start, phys_size;
	uint32_t mem[4];
	extern int nlm_get_fdt_app_param(const char *param, void *data, int type);
	int mmap_fd;
	void *data;

	/*get naerx_vc, saerx_vc, iofb_vc, ipcrx_vc, fp_dst, cp_dst*/
	if(nlm_get_fdt_app_param("nae-rx-vc", &naerx_vc, 0)){
		printf("Invalid nae-rx-vc passed\n");
		return -1;
	}
	if(nlm_get_fdt_app_param("sae-rx-vc", &saerx_vc, 0)){
		printf("Invalid sae-rx-vc passed\n");
		return -1;
	}
	if(nlm_get_fdt_app_param("io-fb-vc", &iofb_vc, 0)){
		printf("Invalid io-fb-vc passed\n");
		return -1;
	}
	if(nlm_get_fdt_app_param("cp-fp-vc", &ipcrx_vc, 0)){
		printf("Invalid cp-fp-vc passed\n");
		return -1;
	}
	if(nlm_get_fdt_app_param("cp-to-fp-dst", &fp_dst, 0)){
		printf("Invalid cp-to-fp-dst passed\n");
		return -1;
	}
	if(nlm_get_fdt_app_param("fp-to-cp-dst", &cp_dst, 0)){
		printf("Invalid fp-to-cp-dst passed\n");
		return -1;
	}
	printf("naerx_vc=%d, saerx_vc=%d, iofb_vc=%d, ipcrx_vc=%d, fp_dst=%d, cp_dst=%d\n",
			naerx_vc, saerx_vc, iofb_vc, ipcrx_vc, fp_dst, cp_dst);

	mmap_fd = open("/dev/mem", O_RDWR);
	if(mmap_fd == -1){
		printf("Couldn't open /dev/mem..\n");
		return -1;
	}

	/*get shmem start and size and mmap it.*/
	if(nlm_get_fdt_app_param("shmem", mem, 0)){
		printf("Invalid shmem params\n");
		return -1;
	}
	phys_start = (((uint64_t)mem[0]) << 32 | mem[1]);
	phys_size = (((uint64_t)mem[2]) << 32 | mem[3]);
	printf("Shared memory %#llx @ %#llx\n", (unsigned long long)phys_size, (unsigned long long)phys_start);

        if((shmem_vaddr = mmap(NULL, phys_size, PROT_READ | PROT_WRITE, MAP_SHARED, mmap_fd, phys_start)) == MAP_FAILED){
		printf("Error : mmap failed for the shared mem, mempool does not work \n");
		return -1;
	}

	/*get linux-memory for packet pool and mmmap it*/
	if(nlm_get_fdt_app_param("linux-mem", mem, 0)){
		printf("Invalid linux-mem params\n");
		return -1;
	}
	phys_start = (((uint64_t)mem[0]) << 32 | mem[1]);
	phys_size = (((uint64_t)mem[2]) << 32 | mem[3]);
	printf("Linux memory %#llx @ %#llx\n", (unsigned long long)phys_size, (unsigned long long)phys_start);
        if((data = mmap(NULL, phys_size, PROT_READ | PROT_WRITE, MAP_SHARED, mmap_fd, phys_start)) == MAP_FAILED){
		printf("Error : mmap failed for the shared mem, mempool does not work \n");
		return -1;
	}
	linux_mem_vaddr = (uint64_t)(unsigned long)data;
	close(mmap_fd);
	return 0;
}

static void xaui_lb_app()
{
	long instance = 1;
	char *child_stack;
	int  childid = 0, size = 32 * 1024;
	uint32_t fp_online_mask = 1U<<getpid();
	for(;;) {
		/* parent will clone the child till no cpus are left
		 */
		if((child_stack = malloc(size)) == NULL) {
			printf("[%s] Mem alloc failed \n", __FUNCTION__);
			break;
		}

		/* stack grows from top to bottom
		 */
		child_stack = child_stack + size - 64;

		childid = clone(process_xaui, child_stack,
				CLONE_BUDDY_FLGS, NULL);
		if (childid < 0) {
			printf("[Parent] : Failed to clone!\n");
			break;
		}
		fp_online_mask |= (1U<<childid);
		instance++;
		printf("[Parent] : Child process (%d) created\n", childid);
	}
	/*update poe dist vec with fp online mask*/
	nae_set_poe_distvec(0, fp_online_mask, 0, 0, 0, 1<<naerx_vc);

	/* Also run the app on the master cpu */
	process_xaui(0);
}

int main(int argc, char *argv[])
{
	if(get_app_parameters())
		return -1;

	printf("Waiting for start command from Linux..\n");	
	while(shmem_vaddr->magic != FP_MAGIC);
	
	printf("FP_MAGIC found. Entering fast path\n");

	memset((void *)&shmem_vaddr->stats, 0, sizeof(struct fp_stats));
	
#ifdef HW_EMULATOR
	nlm_hal_fmn_init(0xf);
#endif

	xaui_lb_app();

	return 0;
}
