/***********************************************************************
  Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
  reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/

#include <stdio.h>
#include <sys/mman.h>
#include <sched.h>
#include <signal.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <mempool.h>
#include <sys/nlm_dev.h>



#define KBP_OK 0
#define KBP_FAIL -1

#define NLM_KBP_MAX_REQ_ID 2048

#define log_err 	printf
#define log_info	printf



/**************************************
Function to generate/free unique Request Id
**************************************/
uint32_t
nlm_kbp_get_req_id(uint32_t is_return_req_id,
					uint32_t return_req_id)
{
	static uint8_t req_id_vector[NLM_KBP_MAX_REQ_ID];
	uint32_t i = 0, req_id = 0;

	if (is_return_req_id == 1) {
		 req_id_vector[return_req_id] = 0;
		 return(0);
	}

	for (i = 0; i < NLM_KBP_MAX_REQ_ID; i++) {
		if(req_id_vector[i] == 0) {
			req_id_vector[i] = 1;
			req_id = i;
			break;
	 	}
	}

	return req_id;
}



/* Read data from KBP from the specified address*/
static int32_t nlm_kbp_read_from_kbp(
						uint32_t vc_id, 	/* vc id*/
						uint32_t address,	/* address to be read */
						uint32_t is_y_read,  /* read x data or y data */
						uint64_t* lsb_data_p,	 /* output lsb 64b data pointer */
						uint16_t* msb_data_p,	 /* output msb 16b data pointer */
						uint8_t* other_fields)	/* like vbit, parity error, dev id, etc*/
{
	kbp_rqt_data_t kbp_req = {0x0ULL,};
	kbp_rsp_data_t kbp_rsp = {0,};
	uint32_t cam_req_id = nlm_kbp_get_req_id(0, 0);

	kbp_req.req_data[0] = 0x0ULL;
	kbp_req.req_data[0]  = address;


	/* Initialize the TCAM Cammand params */
	kbp_req.req_rsp_type = KBP_DIR_REQ_DIR_RSP_TYPE;
	kbp_req.valid_ln = 0x2ULL;
	kbp_req.rsp_len  = 0x10ULL,
	kbp_req.req_len = 0x3ULL;
	kbp_req.empty_byte = 0x0ULL;

	kbp_req.req_header = 0x0ULL;

	if(is_y_read == 0)
		kbp_req.req_header = (cam_req_id << 18) | (2	<< 9) |(0 << 1);
	else
		kbp_req.req_header = (cam_req_id << 18) | (3	<< 9) |(0 << 1);


	 /* Send the command */
	if(KBP_FAIL == kbp_send(vc_id, &kbp_req))
	{
		return KBP_FAIL;
	}

	/* Receive  the response for the read request. */
	if(KBP_FAIL == kbp_recv(vc_id, &kbp_rsp))
	{
		return KBP_FAIL;
	}

	/* Format the output data*/
	*lsb_data_p = kbp_rsp.recv_msg[0];
	*msb_data_p = (uint16_t) kbp_rsp.recv_msg[1];
	*other_fields = (uint8_t)(kbp_rsp.recv_msg[1] >> 16);

	/* Free request id */
	nlm_kbp_get_req_id(1, cam_req_id);

	return KBP_OK;

}

/* Write data to KBP at the specified address*/
static int32_t
nlm_kbp_write_to_kbp(
						        uint32_t vc_id, /* vc id*/
						        uint64_t lsb_data, /* lsb 64b of data */
						        uint16_t msb_data, /* msb 16b of data */
						        uint64_t lsb_mask,  /* lsb 64b of mask */
						        uint16_t msb_mask, /* msb 16b of mask */
						        uint32_t address) 	/* address to be read */
{

	kbp_rqt_data_t kbp_req = {0x0ULL,};
	kbp_rsp_data_t kbp_rsp = {0,};
	uint64_t temp_data = 0x0ULL;

	uint32_t cam_req_id = nlm_kbp_get_req_id(0, 0);

	kbp_req.req_data[0] = 0x0ULL;
	kbp_req.req_data[0]  = address;
	kbp_req.req_data[0]  &= 0x00000000FFFFFFFF;

	kbp_req.req_data[0]  |= (lsb_data << 32);

	kbp_req.req_data[1]  = 0x0ULL;
	kbp_req.req_data[1]  = (lsb_data >> 32);
	kbp_req.req_data[1]  &= 0x00000000FFFFFFFF;

	temp_data = msb_data;
	kbp_req.req_data[1] |= (temp_data << 32);
	kbp_req.req_data[1]  &= 0x0000FFFFFFFFFFFF;

	if(((address) >> 24) & 0x2) /* if Write operation is for DB Entry then format mask also*/
	{
		temp_data = (lsb_mask << 16);
		temp_data &= 0x00000000FFFF0000;
		kbp_req.req_data[1] |= (temp_data << 32);

		kbp_req.req_data[2]  = 0x0ULL;
		temp_data = (lsb_mask >> 16);
		temp_data &= 0x0000FFFFFFFFFFFF;
		kbp_req.req_data[2]  = temp_data;

		temp_data = (msb_mask << 16);
		temp_data &= 0x00000000FFFF0000;
		kbp_req.req_data[2] |= (temp_data << 32);
	}


	/* Initialize the TCAM Cammand params */
	kbp_req.req_rsp_type = KBP_DIR_REQ_DIR_RSP_TYPE;
	kbp_req.valid_ln = 0x3ULL;
	kbp_req.rsp_len  = 0x10ULL,
	kbp_req.req_len = 0x4ULL;
	kbp_req.empty_byte = 0x8ULL;

	kbp_req.req_header = 0x0ULL;
	kbp_req.req_header = (cam_req_id << 18) |
						  (1 << 9) | (0 << 1);

	/* Send the command */
	if(KBP_FAIL == kbp_send(vc_id, &kbp_req))
	{
		return KBP_FAIL;
	}

	/* Receive  the response for the write request. */
	if(KBP_FAIL == kbp_recv(vc_id, &kbp_rsp))
	{
		return KBP_FAIL;
	}

	/* Free request id */
	nlm_kbp_get_req_id(1, cam_req_id);

	return KBP_OK;
}



int main(int argc, char *argv[])
{
	int32_t cpu = getpid();
	uint64_t lsb_data_in = 0;		/* LSB 64b of data*/
	uint16_t msb_data_in = 0;		/* MSB 16b of data*/
	uint64_t lsb_mask_in = 0;		/* LSB 64b of mask*/
	uint16_t msb_mask_in = 0;		/* MSB 16b of mask*/
	uint64_t lsb_data_out = 0;		/* LSB 64b of data*/
	uint16_t msb_data_out = 0;		/* MSB 16b of data*/
	uint8_t other_fields_out = 0; /* like vbit, parity error, dev id, etc*/
	uint32_t address = 0;
	int32_t ab_num = 0, ab_addr = 0;

	log_info("[cpu%d] Running kbp template app...\n", cpu);

	/* Return KBP_FAIL if link is not up*/
	if(0 != kbp_init(cpu, 1))
	{
		log_err("[cpu%d] kbp link initialization failed...try again...\n\n", cpu);
		return KBP_FAIL;
	}

	/* Write KBP register */
	address     = 0x4000;  	/* LTR#0 Block Select Reg 0 */
	lsb_data_in       = 0x5555aaaabbbbcccc;
	msb_data_in       = 0x0000;

	log_info("Writing register data at address = %x\n", address);
	log_info("Data (lsb 64b) = %lx\n", lsb_data_in);
	log_info("Data (msb 16b) = %x\n", msb_data_in);
	
	if(KBP_FAIL == nlm_kbp_write_to_kbp(cpu*4,
						     lsb_data_in,  msb_data_in, lsb_mask_in, msb_mask_in, address))
	{
		log_err("[cpu%d] nlm_kbp_write_to_kbp failed at address %x...\n\n", cpu, address);
		return KBP_FAIL;
	}

	log_info("Reading register data at address = %x\n", address);
	if(KBP_FAIL == nlm_kbp_read_from_kbp(cpu*4,
						     address,  0, &lsb_data_out, &msb_data_out, &other_fields_out))
	{
		log_err("[cpu%d] nlm_kbp_read_from_kbp failed at address %x...\n\n", cpu, address);
		return KBP_FAIL;
	}

	log_info("Register data = ");
	log_info("Data (lsb 64b) = %lx\n", lsb_data_out);
	log_info("Data (msb 16b) = %x\n", msb_data_out);

	/* Compare reg data read from KBP with data written to KBP*/
	if((lsb_data_in != lsb_data_out) || (msb_data_in != msb_data_out))
	{
		log_err("[cpu%d] Data read is not matching with data written at address %x...\n\n", cpu, address);
		log_err("[cpu%d] KBP write/read failure !!!! \n\n", cpu);
		return KBP_FAIL;
	}

	log_info("KBP register write and read successful !!! \n");

	/* Write KBP at AB num 0x10, address 0x0 */
	ab_num = 0x10;
	ab_addr = 0x0;
	address = (ab_num << 12) | ab_addr;

	 /* Insert VBit, address type bits and Wrmode bits in the address */
	 /* set vbit as valid, address type as db and data in x/y format */
	address |= (1 << 30)  | (1 << 25)   | (1 << 31);

	lsb_data_in       = 0x0123456789abcdef;
	msb_data_in       = 0xabcd;
	lsb_mask_in     = 0xfedcba9876543210;
	msb_mask_in       = 0x5432;

	log_info("Writing database at ab num = %x, ab addr = %x...\n\n", ab_num, ab_addr);
	log_info("Data X (lsb 64b) = %lx\n", lsb_data_in);
	log_info("Data X (msb 16b) = %x\n", msb_data_in);
	log_info("Data Y (lsb 64b) = %lx\n", lsb_mask_in);
	log_info("Data Y (msb 16b) = %x\n", msb_mask_in);

	if(KBP_FAIL == nlm_kbp_write_to_kbp(cpu*4,
							lsb_data_in,  msb_data_in, lsb_mask_in, msb_mask_in, address))
	{
		log_err("[cpu%d] nlm_kbp_write_to_kbp failed at ab num = %d, ab addr = %d...\n\n",
								cpu, ab_num, ab_addr);
		return KBP_FAIL;
	}

	/* Reading data from KBP at AB 0x10, address 0x0 */
	log_info("Reading data from ab num = %x, ab addr = %x...\n\n", ab_num, ab_addr);

	/* Read X data*/
	if(KBP_FAIL == nlm_kbp_read_from_kbp(cpu*4,
							address,  0, &lsb_data_out, &msb_data_out, &other_fields_out))
	{
		log_err("[cpu%d] nlm_kbp_read_from_kbp failed at ab num = %d, ab addr = %d...\n\n",
								cpu, ab_num, ab_addr);
		return KBP_FAIL;
	}

	/* Print X data read from KBP*/
	log_info("Data X (lsb 64b) = %lx\n", lsb_data_out);
	log_info("Data X (msb 16b) = %x\n", msb_data_out);

	/* Compare X data data read from KBP with data written to KBP*/
	if((lsb_data_in != lsb_data_out) || (msb_data_in != msb_data_out))
	{
		log_err("[cpu%d] Data read is not matching with data written at address %x...\n\n", cpu, address);
		log_err("[cpu%d] KBP write/read failure !!!! \n\n", cpu);
		return KBP_FAIL;
	}
	
	/* Read Y data*/
	if(KBP_FAIL == nlm_kbp_read_from_kbp(cpu*4,
							address,  1, &lsb_data_out, &msb_data_out, &other_fields_out))
	{
		log_err("[cpu%d] nlm_kbp_read_from_kbp failed at ab num = %d, ab addr = %d...\n\n",
								cpu, ab_num, ab_addr);
		return KBP_FAIL;
	}

	/* Print Y data read from KBP*/
	log_info("Data Y (lsb 64b) = %lx\n", lsb_data_out);
	log_info("Data Y (msb 16b) = %x\n", msb_data_out);

	/* Compare Y data read from KBP with data written to KBP*/
	if((lsb_mask_in != lsb_data_out) || (msb_mask_in != msb_data_out))
	{
		log_err("[cpu%d] Data read is not matching with data written at address %x...\n\n", cpu, address);
		log_err("[cpu%d] KBP write/read failure !!!! \n\n", cpu);
		return KBP_FAIL;
	}

	log_info("KBP database write and read successful !!! \n");

	/* Destroy */
	kbp_destroy(cpu, 1);

	log_info("KBP template application successful !!! Exiting...\n");

	return KBP_OK;
}

