/***********************************************************************
  Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
  reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/
#include <stdio.h>
#include <sys/mman.h>
#include <sched.h>
#include <signal.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <mempool.h>
#include <sys/nlm_dev.h>

//#define AMP_MODE 1
#define MASTER_CPU 0

#define MAX_PKTS       50
#define APP_PORT_BASE  0
#define NUM_APP_PORTS  4
#define NUM_FREE_DESC_PER_PORT 150
#define MAX_PKT_SIZE   (2 << 10) /* 2K */
#define XLP_CACHE_LINE_SIZE 64
#define NUM_CONTEXTS_PER_PORT 8 /* This is specified in nae.dts as well */

#define ICMP_PKT_SIZE  74
#define PKT_ALIGN_LEN  64
#define MAX_CPUS       32
#define MAX_GMAC_PORT  18

#define CLONE_BUDDY_FLGS (CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND | SIGCHLD )

//#define DBG        1
#ifdef DBG
#define log_dbg     printf
#define log_pkt     printf
#else
#define log_dbg(...)
#define log_pkt(...)
//    #define log_err(...)
#endif
#define log_err
#define log_info   printf

int if_fd[MAX_GMAC_PORT];
mpoolptr_t pkt_mem;
typedef struct _shared_pkt_info {
	mpoolptr_t mem_ptr;
	int init_done;
} shared_pkt_info_t;

shared_pkt_info_t *shm_pkt_info;

unsigned long long pkt_mem_limit;
static __inline__ int is_app_pkt_addr(unsigned long long paddr)
{
	unsigned int size = (NUM_APP_PORTS * NUM_FREE_DESC_PER_PORT *  MAX_PKT_SIZE);
	return ((paddr <= pkt_mem_limit) && (paddr >= (pkt_mem_limit - size)));
}

static __inline__ void poe_print(unsigned long long msg0) {
	log_dbg("POE nextfid  = %llu (0x%llx)\n"
		"    nextdist = %llu (0x%llx)\n"
		"    nextdest = %llu (0x%llx)\n"
		"    msgaddr  = 0x%llx\n"
		"    fid      = %llu (0x%llx)\n",
		(msg0 >> 48) & 0xffff, (msg0 >> 48) & 0xffff,
		(msg0 >> 44) & 0xf, (msg0 >> 44) & 0xf,
		(msg0 >> 32) & 0xfff, (msg0 >> 32) & 0xfff,
		(msg0 >> 16) & 0xffff,
		(msg0) & 0xffff, (msg0) & 0xffff);
}

static __inline__ void rx_print(unsigned long long msg0) {
	log_dbg("RX  context = %llu\n"
		"    length  = %llu (0x%llx)\n"
		"    address = 0x%010llx\n"
		"    unclass = %llu\n"
		"    err     = %llu\n"
		"    IPcksm  = %llu\n"
		"    TCPcksm = %llu\n"
		"    prepad  = %llu\n"
		"    p2p     = %llu\n",
		(msg0 >> 54) & 0x3ff,
		(msg0 >> 40) & 0x3fff, (msg0 >> 40) & 0x3fff,
		(msg0) & 0xffffffffc0ULL,
		(msg0 >> 5) & 0x1,
		(msg0 >> 4) & 0x1,
		(msg0 >> 3) & 0x1,
		(msg0 >> 2) & 0x1,
		(msg0 >> 1) & 0x1,
		(msg0) & 0x1);
}

static __inline__ void buf_print(unsigned char *buf, unsigned int len) {
	int i;
	for (i = 0; i < len; ++i) {
		log_dbg(" %02x", buf[i]);
		if (i % 8 == 7) log_dbg(" ");
		if (i % 32 == 31) log_dbg("\n");
	}
	log_dbg("\n");
}

/* Create a shared memory object to store mpoolptr to the packet memory */
void init_pkt_mem(void)
{
	int mem_fd;
	int size = sizeof(shared_pkt_info_t);
	int rv;

	/* Create a shared memory object and set its size */
	mem_fd = shm_open("shm_pkt_mem_ptr",  O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
	if (mem_fd < 0) {
		printf("Error: Failed to allocate shared memory for pakcets\n");
		return;
	}

	/* Set the size   */
	rv = ftruncate(mem_fd, size);
	if(rv < 0) {
		printf("Error: Failed to set size of shared pkt mem\n");
		return;
	}
 
	/* Map the shared memory object */
	shm_pkt_info = (shared_pkt_info_t *) mmap(NULL, size,
						  PROT_READ | PROT_WRITE,
						  MAP_SHARED, mem_fd, 0);
	if((void *) shm_pkt_info == MAP_FAILED) {
		printf("Error: Failed to mmap NAE shared pkt mem\n");
		return;
	}

	if (getpid() == MASTER_CPU) {

		shm_pkt_info->init_done = 0;

		/* Allocate Free desc memory */
		pkt_mem = mpcreate("nae_app_pkt_mem",
				   (NUM_APP_PORTS * NUM_FREE_DESC_PER_PORT), 
				   MAX_PKT_SIZE,
				   XLP_CACHE_LINE_SIZE, 0);

		if (pkt_mem == NULL) {
			printf("Error: Failed to allocate Free Desc Memory\n");
			return;
		}

		/* Save the mpoolptr in a shred object */
		shm_pkt_info->mem_ptr = pkt_mem;
		shm_pkt_info->init_done = 1;
		printf("nae_app: Allocated Packet Memory\n");

	} else { /* Non master cpus */

		while(shm_pkt_info->init_done == 0)
			;

		/* Master Cpu already copied this mpoolptr to shared mem */
		pkt_mem = shm_pkt_info->mem_ptr;
	}
	printf("Cpu%d has shared_pkt_mem @ %p\n", getpid(), pkt_mem);
}

int init_all_gmac(void)
{
	int inf = 0, j;
	char nae_str[50];
	int end_inf = APP_PORT_BASE + NUM_APP_PORTS;
	unsigned long long addr, desc;
	int hw_xaui_port_id ;
	unsigned int size = (NUM_APP_PORTS * NUM_FREE_DESC_PER_PORT *  MAX_PKT_SIZE);


	/* Open all the interfaces */
	for( inf = APP_PORT_BASE; inf < end_inf; inf ++) {

		sprintf(nae_str, "/sysconfig/node0/nae/if%d",inf);
		if_fd[inf] = open(nae_str, O_RDWR, S_IRUSR | S_IWUSR);
		if (if_fd[inf] < 0) {
			printf("HyperExec_ERROR: Failed to open gmac port%d (%s) !! \n", inf, nae_str);
			return -1;
		}
		printf("nae_app: Send Free desc to hwport %d \n",inf * 4);
		/* Send Free descriptors */
		for (j = 0; j < NUM_FREE_DESC_PER_PORT; ++j) {
			addr = (unsigned long) mpalloc(pkt_mem, 0);
			if (addr == 0) {
				printf("Failed to allocate Free Desc\n");
				return -1;
			}

			desc = mpgetphys(pkt_mem, (void *) (unsigned long) (addr & 0xffffffffc0ULL));
			if ((j == 0) && ( inf == 0)) {
				pkt_mem_limit = desc;
				printf("App packet memory range: %llx %llx\n", pkt_mem_limit, (pkt_mem_limit - size));
			}
			hw_xaui_port_id = inf * 4;
			nae_send(hw_xaui_port_id, 0, desc, 0, NLM_NAE_TX_FREE_IN);
		}
		printf("nae_app: Sent Free Descriptors for interface %d\n", inf);
	}

	/* Initialize POE Dist vector */
	nae_set_poe_distvec(0, 0xff, 0, 0, 0, 0x1);
	printf("nae_app: Set the POE dist vector\n");

	return 0;
}

#ifdef DBG
static void print_pkt_buf(char *data, int len)
{
	int i;
	for (i = 0; i < len; ++i) {
		printf("%02x ", data[i]);
		if (i && ((i % 16) == 0)) printf("\n");
	}
	printf("\n");

}
#endif

int process_xaui(void *args)
{
	uint64_t laddr;
	unsigned int cntl = 0, flags = 0, context = 0;;
	int size, rxc = 0, txc = 0;
	int ec = 0, fbid = 0;
	/* void *pkt;*/
	int cpu = getpid();
	int port;
#ifndef RXCTXMAP_LOOKUP 
        // change this array according to the complex configuration in nae.dts
        unsigned int ctx2port[7] = {0, 4, 8, 12, 16, 17, MAX_GMAC_PORT};
#endif 
         
	printf("Cpu%d in %s\n", cpu, __func__);
	while (1) {

		/* RX */
		size = nae_recv(&laddr, &cntl);

		/* no message received */
		if (size == 0) continue;

		context = (cntl & 0x3ff); //  / NUM_CONTEXTS_PER_PORT; /* Calculate the src port */

#ifndef RXCTXMAP_LOOKUP 
		port = ctx2port[context];
#else
                if ((port = nae_get_hwport(context)) >= MAX_GMAC_PORT) {
                           printf("Invalid Context port map: context %d port %d\n",context, port);
                }
#endif
                if(port > 15)        //Don't loopback packet from mgmt interface.
                        continue;


		fbid = port + 32; // This matches with the VFBID table in the FDT
#if 1
		if (is_app_pkt_addr(laddr) == 0) {
			printf("%d found a bad pkt address %lx port %d\n", cpu, (unsigned long) laddr, port);
		}
#endif
		if (size < 0) {
			ec++;
			/* printf("-%d size %d\n",ec, size); */
			flags = -size;

			if (flags & (NLM_NAE_RX_ERR | NLM_NAE_RX_TXC)) {
				nae_send(port, fbid, laddr, 0, NLM_NAE_TX_FREE_IN);
				continue;
			}

			if (flags & NLM_NAE_RX_TXERR) {
				/* printf("Transmit underrun/collision!\n"); */
				continue;
			}

			printf("Unsupported rx flags (%d)!\n", flags);
			exit(-1);
		}

		rxc++;
		//printf("%d: Rx%d laddr 0x%llx size %d flags 0x%x context %d\n", cpu, rxc, (unsigned long long) laddr, size, flags,context);
#if 0
		/* Do something with the pkt */
		if ((getpid() == 0) && (rxc == 10) ) {
			pkt = mpgetvirt(pkt_mem, laddr);
			print_pkt_buf((char *) (unsigned long) pkt, 64);
		}
#endif

#if 1

		/* TX */
		if (nae_send(context, fbid, laddr, size, 0) == 0) { 
			txc++;
		/* printf("%d\n",txc); */
		}
		
#else
	txc = 0; /* for compilation */
#endif
	}
	printf("XLP XAUI Loopback app completed\n");
	exit(0);
}

static void xaui_lb_app()
{
#ifndef AMP_MODE
	long instance = 1;
	char *child_stack;
	int  childid = 0, size = 32 * 1024;

	for(;;) {
		/* parent will clone the child till no cpus are left
		 */
		if((child_stack = malloc(size)) == NULL) {
			printf("[%s] Mem alloc failed \n", __FUNCTION__);
			break;
		}

		/* stack grows from top to bottom
		 */
		child_stack = child_stack + size - 64;

		childid = clone(process_xaui, child_stack,
				CLONE_BUDDY_FLGS, NULL);
		if (childid < 0) {
			printf("[Parent] : Failed to clone!\n");
			break;
		}

		instance++;
		printf("[Parent] : Child process (%d) created\n", childid);
	}
#endif
	/* Also run the app on the master cpu */
	process_xaui(0);
}

int main(int argc, char *argv[])
{
	int cpu = getpid();

	log_info("[cpu%d] Running nae template app...\n", cpu);

	init_pkt_mem();

	/* Initialize NAE interfaces only once from master cpu */
	if ((cpu == MASTER_CPU) && (init_all_gmac() < 0))
		return -1;

	xaui_lb_app();

	return 0;
}
